@@ -576,6 +576,7 @@ enum ikev2_trans_type_prf {
 	IKEv2_PRF_AES128_CMAC   = 8, /* RFC4615 */
 	/* 9 - 1023 Reserved to IANA    RFC4306 */
 	/* 1024 - 65535 Private Use     RFC4306 */
+	IKEv2_PRF_INVALID	= 65536
 };
 
 enum ikev2_trans_type_integ {
@@ -750,17 +751,11 @@ enum ikev1_ipsec_attr {
 #define ENCAPSULATION_MODE_UNSPECIFIED 0	/* not legal -- used internally */
 #define ENCAPSULATION_MODE_TUNNEL      1
 #define ENCAPSULATION_MODE_TRANSPORT   2
-
-#define ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS       61443
-#define ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS    61444
-#define ENCAPSULATION_MODE_UDP_TUNNEL_RFC          3
-#define ENCAPSULATION_MODE_UDP_TRANSPORT_RFC       4
-
 #ifdef NAT_TRAVERSAL
-#define ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS       61443
-#define ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS    61444
-#define ENCAPSULATION_MODE_UDP_TUNNEL_RFC          3
-#define ENCAPSULATION_MODE_UDP_TRANSPORT_RFC       4
+# define ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS       61443
+# define ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS    61444
+# define ENCAPSULATION_MODE_UDP_TUNNEL_RFC          3
+# define ENCAPSULATION_MODE_UDP_TRANSPORT_RFC       4
 #endif
 
 /* Auth Algorithm attribute */
@@ -998,6 +993,9 @@ typedef enum {
     R_U_THERE =       36136,
     R_U_THERE_ACK =   36137,
 
+    /* Netscreen / Juniper private use  - notification contains internal ip */
+    NETSCREEN_NHTB_INFORM = 40001,
+
     /* IKEv2 */
     UNSUPPORTED_CRITICAL_PAYLOAD = 1,
     INVALID_IKE_SPI              = 4,
