@@ -52,6 +52,7 @@ enum keyword_string_config_field {
     KSF_ALSOFLIP,
     KSF_ACCELERATION,
     KSF_CONNALIAS,
+    KSF_LISTEN,
     KSF_MAX
 };
 
@@ -66,6 +67,7 @@ enum keyword_numeric_config_field {
     KBF_UNIQUEIDS,
     KBF_PLUTOWAIT,
     KBF_OVERRIDEMTU,
+    KBF_CONNMTU,
     KBF_STRICTCRLPOLICY,
     KBF_NOCRSEND,
     KBF_NATTRAVERSAL,
@@ -101,6 +103,9 @@ enum keyword_numeric_config_field {
     KBF_CONNADDRFAMILY,
     KBF_FORCEBUSY, /* 40 */
     KBF_OVERLAPIP,
+    KBF_REMOTEPEERTYPE, /*Cisco interop: remote peer type*/
+    KBF_NMCONFIGURED, /*Network Manager support*/
+    KBF_SAREFTRACK, /* saref tracking paramter for _updown */
     KBF_WARNIGNORE, /* to ignore obsoleted keywords */
     KBF_MAX         
 };
@@ -159,8 +164,8 @@ enum keyword_numeric_conn_field {
     KNCF_MAX              
 };
 
-#define KEY_STRINGS_MAX (KSF_MAX > KSCF_MAX ? KSF_MAX : KSCF_MAX)+1
-#define KEY_NUMERIC_MAX (KBF_MAX > KNCF_MAX ? KBF_MAX : KNCF_MAX)+1
+#define KEY_STRINGS_MAX ((int)KSF_MAX > (int)KSCF_MAX ? (int)KSF_MAX : (int)KSCF_MAX)+1
+#define KEY_NUMERIC_MAX ((int)KBF_MAX > (int)KNCF_MAX ? (int)KBF_MAX : (int)KNCF_MAX)+1
 
 /* these are bits set in a word */
 enum keyword_valid {
