@@ -30,15 +30,6 @@ struct paththing {
   size_t   path_space;
 };
 
-struct pluto_paths {
-  struct paththing acerts;
-  struct paththing cacerts;
-  struct paththing crls;
-  struct paththing private;
-  struct paththing certs;
-  struct paththing aacerts;
-  struct paththing ocspcerts;
-};
 
 struct osw_conf_options {
     char *rootdir;                /* default is "" --- used for testing */
@@ -69,6 +60,7 @@ typedef struct {
 #endif
 
 extern const struct osw_conf_options *osw_init_options(void);
+extern void osw_conf_free_oco(void);
 extern const struct osw_conf_options *osw_init_ipsecdir(const char *ipsec_dir);
 extern const struct osw_conf_options *osw_init_rootdir(const char *root_dir);
 
