@@ -161,8 +161,8 @@ typedef enum {
 #define DBG_EMITTING	LELEM(3)	/* show encoding of messages */
 #define DBG_CONTROL	LELEM(4)	/* control flow within Pluto */
 #define DBG_LIFECYCLE	LELEM(5)	/* SA lifecycle */
-#define DBG_KLIPS	LELEM(6)	/* messages to KLIPS */
-#define DBG_NETKEY	LELEM(6)	/* messages to NETKEY/XFRM */
+#define DBG_KLIPS	LELEM(6)	/* messages with the kernel */
+#define DBG_NETKEY	LELEM(6)	/* same as previous entry!! */ 
 #define DBG_DNS		LELEM(7)	/* DNS activity */
 #define DBG_OPPO	LELEM(8)	/* opportunism */
 #define DBG_CONTROLMORE LELEM(9)	/* more detailed debugging */
@@ -383,6 +383,12 @@ enum four_options {
 	fo_insist  = 3   /* propose, and only accept if peer agrees */
 };
 
+enum saref_tracking {
+	sat_yes = 0, /* SAref tracking via _updown - the default */
+	sat_no = 1, /* no SAref tracking - third party will handle this */
+	sat_conntrack = 2, /* Saref tracking using connmark optimizations */
+};
+
 /* Policies for establishing an SA
  *
  * These are used to specify attributes (eg. encryption) and techniques
@@ -393,7 +399,11 @@ enum four_options {
 
 extern const char *prettypolicy(lset_t policy);
 
-/* ISAKMP auth techniques (none means never negotiate) */
+/* 
+ * ISAKMP auth techniques (none means never negotiate)
+ * a pluto policy is stored in a lset_t which is an unsigned long long,
+ * so we should have 64 bits to play with
+ */
 enum pluto_policy {
 	POLICY_PSK     = LELEM(0), 
 	POLICY_RSASIG  = LELEM(1),
@@ -460,10 +470,14 @@ enum pluto_policy {
 	POLICY_IKEV2_ALLOW   = LELEM(25), /* accept IKEv2?   0x0200 0000 */
 	POLICY_IKEV2_PROPOSE = LELEM(26), /* propose IKEv2?  0x0400 0000 */
 	POLICY_IKEV2_MASK = POLICY_IKEV1_DISABLE|POLICY_IKEV2_ALLOW|POLICY_IKEV2_PROPOSE,
+
 	POLICY_MODECFGDNS1  = LELEM(27),   /* should we offer a DNS server IP */
 	POLICY_MODECFGDNS2  = LELEM(28),   /* should we offer another DNS server IP */
 	POLICY_MODECFGWINS1 = LELEM(29),   /* should we offer a WINS server IP */
 	POLICY_MODECFGWINS2 = LELEM(30),   /* should we offer another WINS server IP */
+
+	POLICY_SAREF_TRACK    = LELEM(31), /* Saref tracking via _updown */
+	POLICY_SAREF_TRACK_CONNTRACK    = LELEM(32), /* use conntrack optimization */
 };
 
 /* Any IPsec policy?  If not, a connection description
@@ -542,16 +556,6 @@ enum pluto_policy {
 
 #define SA_LIFE_DURATION_K_DEFAULT  0xFFFFFFFFlu
 
-/* Encapsulation Mode attribute */
-
-#define ENCAPSULATION_MODE_UNSPECIFIED             0	/* not legal -- used internally */
-#define ENCAPSULATION_MODE_TUNNEL                  1
-#define ENCAPSULATION_MODE_TRANSPORT               2
-#define ENCAPSULATION_MODE_UDP_TUNNEL_RFC          3
-#define ENCAPSULATION_MODE_UDP_TRANSPORT_RFC       4
-#define ENCAPSULATION_MODE_UDP_TUNNEL_DRAFTS       61443
-#define ENCAPSULATION_MODE_UDP_TRANSPORT_DRAFTS    61444
-
 /* Oakley Lifetime Type attribute
  * draft-ietf-ipsec-ike-01.txt appendix A
  * As far as I can see, there is not specification for
