@@ -12,7 +12,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: confread.c,v 1.11 2004/04/11 15:17:30 ken Exp $
  */
 
 #include <stdlib.h>
@@ -81,9 +80,9 @@ void ipsecconf_default_values(struct starter_config *cfg)
 #ifdef HAVE_NM
 	cfg->conn_default.options[KBF_NMCONFIGURED] = NO;
 #endif
-
 	cfg->conn_default.policy = POLICY_RSASIG|POLICY_TUNNEL|POLICY_ENCRYPT|POLICY_PFS;
-	cfg->conn_default.policy |= POLICY_IKEV2_ALLOW;
+	cfg->conn_default.policy |= POLICY_IKEV2_ALLOW; /* ikev2=yes */
+	cfg->conn_default.policy |= POLICY_SAREF_TRACK;  /* sareftrack=yes */
 
 	cfg->conn_default.options[KBF_IKELIFETIME] = OAKLEY_ISAKMP_SA_LIFETIME_DEFAULT;
 	cfg->conn_default.options[KBF_SALIFETIME]  = SA_LIFE_DURATION_DEFAULT;
@@ -310,7 +309,7 @@ static int load_setup (struct starter_config *cfg
 	    case kt_comment:
 		break;
 	    case kt_obsolete:
-	        starter_log(LOG_LEVEL_INFO, "Warning: ignored obsolete keyword %s",kw->string);
+	        starter_log(LOG_LEVEL_INFO, "Warning: ignored obsolete keyword %s",kw->keyword.keydef->keyname);
 		break;
 	    }
 	}
@@ -519,7 +518,7 @@ static int validate_end(struct starter_conn *conn_st
 	    end->has_client = TRUE;
 	    end->has_client_wildcard = FALSE;
 	}
-    } 
+    }
 
     /* copy certificate path name */
     if(end->strings_set[KSCF_CERT]) {
@@ -664,6 +663,17 @@ bool translate_conn (struct starter_conn *conn
 		    free((*the_strings)[field]);
 	    }
 	    
+	    if(kw->string == NULL) 
+		{
+		*error = _tmp_err;
+		
+		snprintf(_tmp_err, sizeof(_tmp_err)
+			 , "Invalid %s value"
+			 , kw->keyword.keydef->keyname);
+		    err++;
+		    break;
+		}
+
 	    (*the_strings)[field] = xstrdup(kw->string);
 	    (*set_strings)[field] = assigned_value;
 	    break;
@@ -1057,8 +1067,37 @@ static int load_conn (struct starter_config *cfg
 	}
     }
 
+    if(conn->options_set[KBF_SAREFTRACK]) {
+	switch(conn->options[KBF_SAREFTRACK]) {
+	case sat_yes:
+	    /* this is the default */
+	    conn->policy |= POLICY_SAREF_TRACK;
+	    break;
+	    
+	case sat_conntrack:
+	    conn->policy |= POLICY_SAREF_TRACK|POLICY_SAREF_TRACK_CONNTRACK;
+	    break;
+	    
+	case sat_no:
+	    conn->policy &= ~POLICY_SAREF_TRACK;
+	    conn->policy &= ~POLICY_SAREF_TRACK_CONNTRACK;
+	    break;
+	}
+    }
+
     err += validate_end(conn, &conn->left,  TRUE,  resolvip, perr);
     err += validate_end(conn, &conn->right, FALSE, resolvip, perr);
+    /*
+     * TODO:
+     * verify both ends are using the same inet family, if one end
+     * is "%any" or "%defaultroute", then perhaps adjust it.
+     * ensource this for left,leftnexthop,right,rightnexthop
+     * Ideally, phase out connaddrfamily= which now wrongly assumes
+     * left,leftnextop,leftsubnet are the same inet family
+     * Currently, these tests are implicitely done, and wrongly
+     * in case of 6in4 and 4in6 tunnels
+     */
+
 
     if(conn->options_set[KBF_AUTO]) {
 	conn->desired_state = conn->options[KBF_AUTO];
@@ -1121,12 +1160,12 @@ struct starter_conn *alloc_add_conn(struct starter_config *cfg, char *name, err_
     struct starter_conn *conn;
     
     conn = (struct starter_conn *)malloc(sizeof(struct starter_conn));
-    memset(conn, 0, sizeof(struct starter_conn));
     if (!conn) {
 	if (perr) *perr = xstrdup("can't allocate mem in confread_load()");
 	return NULL;
     }
 
+    memset(conn, 0, sizeof(struct starter_conn));
     conn_default(name, conn, &cfg->conn_default);
     conn->name = xstrdup(name);
     conn->desired_state = STARTUP_NO;
@@ -1173,7 +1212,8 @@ int init_load_conn(struct starter_config *cfg
 struct starter_config *confread_load(const char *file
 				     , err_t *perr
 				     , bool resolvip
-				     , char *ctlbase)
+				     , char *ctlbase
+				     , bool setuponly)
 {
 	struct starter_config *cfg = NULL;
 	struct config_parsed *cfgp;
@@ -1209,14 +1249,19 @@ struct starter_config *confread_load(const char *file
 	 */
 	err += load_setup(cfg, cfgp, perr);
 
-	if(err) {return NULL;}
+	if(err) {
+		parser_free_conf(cfgp);
+		confread_free(cfg);
+		return NULL;
+	}
 
-	/**
-	 * Find %default and %oedefault conn
-	 *
-	 */
-	for(sconn = cfgp->sections.tqh_first; (!err) && sconn != NULL; sconn = sconn->link.tqe_next)
-	{
+	if(!setuponly) {
+	   /**
+	    * Find %default and %oedefault conn
+	    *
+	    */
+	   for(sconn = cfgp->sections.tqh_first; (!err) && sconn != NULL; sconn = sconn->link.tqe_next)
+	   {
 		if (strcmp(sconn->name,"%default")==0) {
 			starter_log(LOG_LEVEL_DEBUG, "Loading default conn");
 			err += load_conn (cfg, &cfg->conn_default,
@@ -1235,13 +1280,13 @@ struct starter_config *confread_load(const char *file
 			    cfg->got_oedefault=TRUE;
 			}
 		}
-	}
+	   }
 
-	/**
-	 * Load other conns
-	 */
-	for(sconn = cfgp->sections.tqh_first; sconn != NULL; sconn = sconn->link.tqe_next)
-	{
+	   /**
+	    * Load other conns
+	    */
+	   for(sconn = cfgp->sections.tqh_first; sconn != NULL; sconn = sconn->link.tqe_next)
+	   {
 		if (strcmp(sconn->name,"%default")==0) continue;
 		if (strcmp(sconn->name,"%oedefault")==0) continue;
 
@@ -1254,12 +1299,13 @@ struct starter_config *confread_load(const char *file
 		    return NULL;
 		}
 		err += connerr;
-	}
+	   }
 
-	/* if we have OE on, then create any missing OE conns! */
-	if(cfg->setup.options[KBF_OPPOENCRYPT]) {
-	    starter_log(LOG_LEVEL_DEBUG, "Enabling OE conns\n");
-	    add_any_oeconns(cfg, cfgp);
+	   /* if we have OE on, then create any missing OE conns! */
+	   if(cfg->setup.options[KBF_OPPOENCRYPT]) {
+	       starter_log(LOG_LEVEL_DEBUG, "Enabling OE conns\n");
+	       add_any_oeconns(cfg, cfgp);
+	   }
 	}
 
 	parser_free_conf(cfgp);
@@ -1307,6 +1353,7 @@ void confread_free(struct starter_config *cfg)
 	struct starter_conn *conn, *c;
 	FREE_LST(cfg->setup.interfaces);
 	FREE_STR(cfg->setup.virtual_private);
+	FREE_STR(cfg->setup.listen);
 	for(i=0 ;i<KSF_MAX; i++)
 	{
 	    FREE_STR(cfg->setup.strings[i]);
