@@ -1,7 +1,7 @@
 /*
  * Openswan config file parser (keywords.c)
  * Copyright (C) 2003-2006 Michael Richardson <mcr@xelerance.com>
- * Copyright (C) 2007-2008 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2007-2010 Paul Wouters <paul@xelerance.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -176,6 +176,17 @@ struct keyword_enum_value kw_proto_stack_list[]={
 struct keyword_enum_values kw_proto_stack=
     { kw_proto_stack_list, sizeof(kw_proto_stack_list)/sizeof(struct keyword_enum_value)};
 
+/*
+ * Values for sareftrack={yes, no, conntrack }
+ */
+struct keyword_enum_value kw_sareftrack_values[]={
+    { "yes",          sat_yes },
+    { "no",           sat_no },
+    { "conntrack",    sat_conntrack },
+};
+
+struct keyword_enum_values kw_sareftrack_list=
+    { kw_sareftrack_values, sizeof(kw_sareftrack_values)/sizeof(struct keyword_enum_value)};
 
 /*
  *  Cisco interop: remote peer type
@@ -207,7 +218,7 @@ struct keyword_enum_values kw_nm_configured=
 extern struct keyword_enum_values kw_host_list;
 
 
-
+#ifdef DEBUG
 struct keyword_enum_value kw_plutodebug_values[]={
     { "none",     DBG_NONE },
     { "all",      DBG_ALL },
@@ -264,7 +275,7 @@ struct keyword_enum_value kw_klipsdebug_values[]={
 
 struct keyword_enum_values kw_klipsdebug_list=
     { kw_klipsdebug_values, sizeof(kw_klipsdebug_values)/sizeof(struct keyword_enum_value)};
-    
+#endif
 
 struct keyword_enum_value kw_phase2types_values[]={
     { "ah+esp",   POLICY_ENCRYPT|POLICY_AUTHENTICATE },
@@ -295,8 +306,10 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"interfaces",     kv_config, kt_string,    KSF_INTERFACES,NOT_ENUM},
     {"myid",           kv_config, kt_string,    KSF_MYID,NOT_ENUM},
     {"syslog",         kv_config, kt_string,    KSF_SYSLOG,NOT_ENUM},
+#ifdef DEBUG
     {"klipsdebug",     kv_config, kt_list,      KBF_KLIPSDEBUG, &kw_klipsdebug_list},
     {"plutodebug",     kv_config, kt_list,      KBF_PLUTODEBUG, &kw_plutodebug_list},
+#endif
     {"plutoopts",      kv_config, kt_string,    KSF_PLUTOOPTS,NOT_ENUM},
     {"plutostderrlog", kv_config, kt_filename,  KSF_PLUTOSTDERRLOG,NOT_ENUM},
     {"plutorestartoncrash", kv_config, kt_bool, KBF_PLUTORESTARTONCRASH,NOT_ENUM},
@@ -322,6 +335,7 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"keep_alive", kv_config,kt_number,    KBF_KEEPALIVE, NOT_ENUM},
     {"force_keepalive", kv_config,kt_bool,    KBF_FORCE_KEEPALIVE, NOT_ENUM},
 #endif
+    {"listen",     kv_config, kt_string, KSF_LISTEN,NOT_ENUM},
     {"protostack",     kv_config, kt_string,    KSF_PROTOSTACK, &kw_proto_stack},
     {"nhelpers",kv_config,kt_number, KBF_NHELPERS, NOT_ENUM},
 
@@ -361,6 +375,7 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"authby",         kv_conn|kv_auto, kt_enum,   KBF_AUTHBY,     &kw_authby_list},
     {"keyexchange",    kv_conn|kv_auto, kt_enum,   KBF_KEYEXCHANGE, &kw_keyexchange_list},
     {"ikev2",          kv_conn|kv_auto|kv_processed,kt_enum,KBF_IKEv2,&kw_fourvalued_list},
+    {"sareftrack",     kv_conn|kv_auto|kv_processed,kt_enum,KBF_SAREFTRACK,&kw_sareftrack_list},
     {"pfs",            kv_conn|kv_auto, kt_bool,   KBF_PFS,          NOT_ENUM},
     {"keylife",        kv_conn|kv_auto|kv_alias, kt_time,   KBF_SALIFETIME,NOT_ENUM},
     {"lifetime",       kv_conn|kv_auto|kv_alias, kt_time,   KBF_SALIFETIME,NOT_ENUM},
@@ -402,10 +417,11 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"metric",         kv_conn|kv_auto, kt_number, KBF_METRIC, NOT_ENUM},
 
     /* DPD */ 
-    {"dpddelay", kv_conn|kv_auto,kt_number, KBF_DPDDELAY, NOT_ENUM},
-    {"dpdtimeout", kv_conn|kv_auto,kt_number,KBF_DPDTIMEOUT , NOT_ENUM},
-    {"dpdaction", kv_conn|kv_auto,kt_enum, KBF_DPDACTION , &kw_dpdaction_list},
+    {"dpddelay",       kv_conn|kv_auto,kt_number, KBF_DPDDELAY, NOT_ENUM},
+    {"dpdtimeout",     kv_conn|kv_auto,kt_number,KBF_DPDTIMEOUT , NOT_ENUM},
+    {"dpdaction",      kv_conn|kv_auto,kt_enum, KBF_DPDACTION , &kw_dpdaction_list},
 
+    {"mtu",            kv_conn|kv_auto,kt_number, KBF_CONNMTU, NOT_ENUM},
 
     /* aggr/xauth/modeconfig */ 
     {"aggrmode",    kv_conn|kv_auto, kt_invertbool,      KBF_AGGRMODE,NOT_ENUM},
@@ -534,7 +550,7 @@ unsigned int parser_enum_list(struct keyword_def *kd, const char *s, bool list)
     valresult = 0;
 
     /*
-     * split up the string into comma seperated pieces, and look each piece up in the
+     * split up the string into comma separated pieces, and look each piece up in the
      * value list provided in the definition.
      */
 
