@@ -14,7 +14,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
  * License for more details.
  *
- * RCSID $Id: openswan.h,v 1.95 2005/08/25 01:24:40 paul Exp $
  */
 #define	_OPENSWAN_H	/* seen it, no need to see it again */
 
@@ -188,6 +187,21 @@ typedef struct {
 	ip_address addr;
 	int maskbits;
 } ip_subnet;
+#define ip_address_family(a)	((a)->u.v4.sin_family)
+#define ip_address_cmp(a, b) \
+	(ip_address_family((a)) != ip_address_family((b)) || \
+	(ip_address_family((a)) == AF_INET ? \
+			((a)->u.v4.sin_addr.s_addr != (b)->u.v4.sin_addr.s_addr) : \
+			memcmp((a)->u.v6.sin6_addr.s6_addr32, \
+					(b)->u.v6.sin6_addr.s6_addr32, sizeof(u_int32_t) * 4) \
+			))
+#define	ip_address_isany(a) \
+	(ip_address_family((a)) == AF_INET6 ? \
+			((a)->u.v6.sin6_addr.s6_addr[0] == 0 && \
+				(a)->u.v6.sin6_addr.s6_addr[1] == 0 && \
+				(a)->u.v6.sin6_addr.s6_addr[2] == 0 && \
+				(a)->u.v6.sin6_addr.s6_addr[3] == 0) : \
+			((a)->u.v4.sin_addr.s_addr == 0))
 
 /* and the SA ID stuff */
 #ifdef __KERNEL__
@@ -257,6 +271,11 @@ typedef uint32_t IPsecSAref_t;
 # define BLANK_FORMAT ""
 #endif
 
+#ifdef COMPILER_HAS_NO_PRINTF_LIKE
+# undef PRINTF_LIKE
+# define PRINTF_LIKE(n)	/* ignore */
+#endif
+
 
 /*
  * function to log stuff from libraries that may be used in multiple
@@ -283,6 +302,8 @@ err_t ttoaddr_num(const char *src, size_t srclen, int af, ip_address *dst);
 
 err_t tnatoaddr(const char *src, size_t srclen, int af, ip_address *dst);
 size_t addrtot(const ip_address *src, int format, char *buf, size_t buflen);
+size_t inet_addrtot(int type,const void *src, int format, char *buf, size_t buflen);
+size_t sin_addrtot(const void *sin, int format, char *dst, size_t dstlen);
 /* RFC 1886 old IPv6 reverse-lookup format is the bulkiest */
 #define	ADDRTOT_BUF	(32*2 + 3 + 1 + 3 + 1 + 1)
 err_t ttosubnet(const char *src, size_t srclen, int af, ip_subnet *dst);
@@ -426,6 +447,14 @@ subnettoa(
 	char *dst,
 	size_t dstlen
 );
+size_t				/* space needed for full conversion */
+subnet6toa(
+	struct in6_addr *addr,
+	struct in6_addr *mask,
+	int format,		/* character; 0 means default */
+	char *dst,
+	size_t dstlen
+);
 #define	SUBNETTOA_BUF	32	/* large enough for worst case result */
 
 /* ranges */
@@ -505,6 +534,7 @@ goodmask(
 	struct in_addr mask
 );
 extern int masktobits(struct in_addr mask);
+extern int mask6tobits(struct in6_addr *mask);
 extern struct in_addr  bitstomask(int n);
 extern struct in6_addr bitstomask6(int n);
 
