@@ -13,15 +13,13 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ipsec_encap.h,v 1.19 2004-04-05 19:55:04 mcr Exp $
+ * RCSID $Id: ipsec_encap.h,v 1.19 2004/04/05 19:55:04 mcr Exp $
  */
 
 #ifndef _IPSEC_ENCAP_H_
 
-#define SENT_IP4	16	/* data is two struct in_addr + proto + ports*/
-			/* (2 * sizeof(struct in_addr)) */
-			/* sizeof(struct sockaddr_encap)
-			   - offsetof(struct sockaddr_encap, Sen.Sip4.Src) */
+#define SENT_IP4	0x01 /* match OpenBSD for what it's worth */
+#define SENT_IP6	0x02 /* match OpenBSD for what it's worth */
 
 struct sockaddr_encap
 {
@@ -38,8 +36,16 @@ struct sockaddr_encap
 			__u16 Sport;
 			__u16 Dport;
 		} Sip4;
+		struct			/* SENT_IP6 */
+		{
+			struct in6_addr Src;
+			struct in6_addr Dst;
+			__u8 Proto;
+			__u16 Sport;
+			__u16 Dport;
+		} Sip6;
 	} Sen;
-};
+} __attribute__((packed));
 
 #define sen_ip_src	Sen.Sip4.Src
 #define sen_ip_dst	Sen.Sip4.Dst
@@ -47,6 +53,12 @@ struct sockaddr_encap
 #define sen_sport       Sen.Sip4.Sport
 #define sen_dport       Sen.Sip4.Dport
 
+#define sen_ip6_src	Sen.Sip6.Src
+#define sen_ip6_dst	Sen.Sip6.Dst
+#define sen_proto6      Sen.Sip6.Proto
+#define sen_sport6      Sen.Sip6.Sport
+#define sen_dport6      Sen.Sip6.Dport
+
 #ifndef AF_ENCAP
 #define AF_ENCAP 26
 #endif /* AF_ENCAP */
@@ -56,7 +68,7 @@ struct sockaddr_encap
 
 /*
  * $Log: ipsec_encap.h,v $
- * Revision 1.19  2004-04-05 19:55:04  mcr
+ * Revision 1.19  2004/04/05 19:55:04  mcr
  * Moved from linux/include/freeswan/ipsec_encap.h,v
  *
  * Revision 1.18  2003/10/31 02:27:05  mcr
