@@ -3,8 +3,8 @@
  * header file for Openswan library functions
  * Copyright (C) 1998, 1999, 2000  Henry Spencer.
  * Copyright (C) 1999, 2000, 2001  Richard Guy Briggs
- * Copyright (C) 2003 - 2009 Paul Wouters <paul@xelerance.com>
- * Copyright (C) 2008 - 2009 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2003 - 2011 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2008 - 2011 David McCullough <david_mccullough@securecomputing.com>
  * 
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Library General Public License as published by
@@ -43,10 +43,17 @@
  * We use a lot of config defines,  on older kernels that means we
  * need to include config.h
  */
-#ifndef AUTOCONF_INCLUDED
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,38) && !defined(AUTOCONF_INCLUDED)
 #include <linux/config.h>
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
+/* Only enable IPv6 support on newer kernels with IPv6 enabled */
+# if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE)
+#  define CONFIG_KLIPS_IPV6 1
+# endif
+#endif
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
 # define SPINLOCK
 # define PROC_FS_21
@@ -235,6 +242,13 @@
 # define nfmark mark
 #else
 # define HAVE_KMEM_CACHE_T
+# if defined(RHEL_RELEASE_CODE)
+#  if RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(5,0)
+#   define FLOW_HAS_NO_MARK
+#  endif
+# elif defined(CONFIG_SLE_VERSION) && defined(CONFIG_SLE_SP) && (CONFIG_SLE_VERSION == 10)
+#  define FLOW_HAS_NO_MARK
+# endif
 #endif
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
@@ -251,7 +265,6 @@
  
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,22) 
 #  define HAVE_KERNEL_TSTAMP
-#  define HAVE_KMEM_CACHE_MACRO
 #  define grab_socket_timeval(tv, sock)  { (tv) = ktime_to_timeval((sock).sk_stamp); }
 #else
 #  define grab_socket_timeval(tv, sock)  { (tv) = (sock).sk_stamp; }
@@ -282,6 +295,7 @@
 #  define tcp_hdr(skb)  ((skb)->h.th)
 #  define udp_hdr(skb)  ((skb)->h.uh)
 #  define skb_transport_header(skb)  ((skb)->h.raw)
+#  define skb_network_offset(skb)  ((skb)->nh.raw - (skb)->data)
 #  define skb_set_transport_header(skb,off)  ((skb)->h.raw = (skb)->data + (off))
 #  define skb_reset_transport_header(skb) ((skb)->h.raw = (skb)->data - (skb)->head)
 #  define skb_mac_header(skb)  ((skb)->mac.raw)
@@ -344,6 +358,16 @@
 #define l_inet_addr_type	inet_addr_type
 #endif
 
+#include <linux/spinlock.h>
+#include <linux/interrupt.h>
+#include <net/addrconf.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,25)
+# define ip6_chk_addr(a) (ipv6_chk_addr(&init_net, a, NULL, 1) ? IS_MYADDR : 0)
+#else
+# define ip6_chk_addr(a) (ipv6_chk_addr(a, NULL, 1) ? IS_MYADDR : 0)
+#endif
+#define l_ipv6_addr_type(a)	ip6_chk_addr(a)
+
 /* not sure when network name spaces got introduced, but it is in 2.6.26 */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26)
 # define HAVE_NETWORK_NAMESPACE 1
@@ -363,6 +387,12 @@
 #endif
 #endif
 
+#if LINUX_VERSION_CODE == KERNEL_VERSION(2,6,30)
+# ifndef CONFIG_COMPAT_NET_DEV_OPS
+#  define USE_NETDEV_OPS
+# endif
+#endif
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31)
 # define USE_NETDEV_OPS
 #else
@@ -375,11 +405,14 @@
 # define skb_dst(s)		(s)->dst
 #endif
 
-#if 0
-/* nicely, latest netdevice.h includes this define */
-#ifndef HAVE_NETDEV_PRIV
-#define netdev_priv(dev) (dev->priv)
-#endif
+/* The SLES10 kernel is known to not have these defines */
+#ifdef CONFIG_KLIPS_IPV6
+# ifndef IN6ADDR_ANY_INIT
+#  define IN6ADDR_ANY_INIT { { { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } } }
+# endif
+# ifndef IN6ADDR_LINKLOCAL_ALLNODES_INIT
+#  define IN6ADDR_LINKLOCAL_ALLNODES_INIT { { { 0xff,2,0,0,0,0,0,0,0,0,0,0,0,0,0,1 } } }
+# endif
 #endif
 
 #if !defined(HAVE_CURRENT_UID)
@@ -405,6 +438,8 @@
        volatile unsigned int lock;
      } rwlock_t;                                                                     
 
+#  define SPIN_LOCK_UNLOCKED {0}
+
 #  define spin_lock_init(x) { (x)->lock = 0;}
 #  define rw_lock_init(x) { (x)->lock = 0; }
 
@@ -488,16 +523,50 @@
 # endif
 #endif
 
-#ifndef NF_INET_LOCAL_OUT
-# define NF_INET_LOCAL_OUT NF_IP_LOCAL_OUT
+#include <linux/netfilter.h>
+#include <linux/netfilter_ipv4.h>
+#include <linux/netfilter_ipv6.h>
+#ifdef NF_IP_LOCAL_OUT
+# define OSW_NF_INET_LOCAL_OUT	NF_IP_LOCAL_OUT
+#endif
+#ifndef OSW_NF_INET_LOCAL_OUT
+# define OSW_NF_INET_LOCAL_OUT NF_INET_LOCAL_OUT
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,33)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,33)
 #define	inet_sport	sport
 #define	inet_dport	dport
-#define	CTL_NAME(n)
-#else
 #define	CTL_NAME(n)	.ctl_name = n,
+#else
+#define	CTL_NAME(n)
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,35)
+#define HAVE_SOCKET_WQ
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,37)
+# define PRIVATE_ARP_BROKEN_OPS
+#endif
+
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,38)
+# ifndef DEFINE_SPINLOCK
+#  define DEFINE_SPINLOCK(x) spinlock_t x = SPIN_LOCK_UNLOCKED
+# endif
+# define flowi_tos nl_u.ip4_u.tos
+# define flowi_proto proto
+# define flowi_mark mark
+# define flowi_oif oif
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,39)
+# define nl_u u
+# define ip4_u ip4
+# define ip6_u ip6
+#endif
+
+#ifndef DEFINE_RWLOCK
+#define DEFINE_RWLOCK(x) rwlock_t x = RW_LOCK_UNLOCKED
 #endif
 
 #if __KERNEL__
@@ -513,5 +582,21 @@
 # endif
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36)
+#define	ipsec_route_dst(x)	(x)->dst
+#else
+#define	ipsec_route_dst(x)	(x)->u.dst
+#endif
+
+# if defined(CONFIG_SLE_VERSION) && defined(CONFIG_SLE_SP) && (CONFIG_SLE_VERSION == 10 && CONFIG_SLE_SP >= 3)
+# define HAVE_BACKPORTED_NEW_CRYPTOAPI 1
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,1,0)
+#define	HAVE_NETDEV_PRIV
+#define	HAVE_NET_DEVICE_OPS
+#define	HAVE_NETIF_QUEUE
+#endif
+
 #endif /* _OPENSWAN_KVERSIONS_H */
 
