@@ -78,16 +78,16 @@ enum ipsec_xmit_value
 struct ipsec_xmit_state
 {
 	struct sk_buff *skb;		/* working skb pointer */
+	struct sk_buff *pre_ipcomp_skb; /* skb before ipcomp was attempted */
 	struct net_device *dev;		/* working dev pointer */
-	struct ipsecpriv *prv;		/* Our device' private space */
+	struct ipsecpriv *iprv;		/* Our device' private space */
+	struct mastpriv *mprv;
 	struct sk_buff *oskb;		/* Original skb pointer */
 	struct net_device_stats *stats;	/* This device's statistics */
-	struct iphdr  *iph;		/* Our new IP header */
-	__u32   newdst;			/* The other SG's IP address */
-	__u32	orgdst;			/* Original IP destination address */
-	__u32	orgedst;		/* 1st SG's IP address */
-	__u32   newsrc;			/* The new source SG's IP address */
-	__u32	orgsrc;			/* Original IP source address */
+	void	*iph;		/* Our new IP header */
+
+	ip_address	orgedst;		/* 1st SG's IP address */
+
 	int	iphlen;			/* IP header length */
 	int	pyldsz;			/* upper protocol payload size */
 	int	headroom;
@@ -114,6 +114,7 @@ struct ipsec_xmit_state
 	short physmtu;
 	short cur_mtu;          /* copy of prv->mtu, cause prv may == NULL */
 	short mtudiff;
+	__u8 next_header;               /* protocol of the nested header */
 #ifdef NET_21
 	struct rtable *route;
 #endif /* NET_21 */
@@ -134,6 +135,10 @@ struct ipsec_xmit_state
 	 * xmit flags
 	 */
 	uint16_t mast_mode:1;
+	uint16_t set_dst:1;
+
+	/* if carrying IPv6,  IPPROTO_IPV6, else IPPROTO_IPIP */
+	uint8_t ipip_proto;
 
 	/*
 	 * xmit state machine use
@@ -142,12 +147,6 @@ struct ipsec_xmit_state
 			enum ipsec_xmit_value stat);
 	int		state;
 	int		next_state;
-#ifdef CONFIG_KLIPS_OCF
-	struct work_struct	workq;
-#ifdef DECLARE_TASKLET
-	struct tasklet_struct	tasklet;
-#endif
-#endif
 #ifdef CONFIG_KLIPS_ALG
 	struct ipsec_alg_auth *ixt_a;
 	struct ipsec_alg_enc *ixt_e;
@@ -163,13 +162,16 @@ struct ipsec_xmit_state
 	__u16 ttl, check;
 };
 
-enum ipsec_xmit_value
-ipsec_xmit_sanity_check_dev(struct ipsec_xmit_state *ixs);
+extern enum ipsec_xmit_value
+ipsec_xmit_sanity_check_ipsec_dev(struct ipsec_xmit_state *ixs);
 
-enum ipsec_xmit_value
+extern enum ipsec_xmit_value
+ipsec_xmit_sanity_check_mast_dev(struct ipsec_xmit_state *ixs);
+
+extern enum ipsec_xmit_value
 ipsec_xmit_sanity_check_skb(struct ipsec_xmit_state *ixs);
 
-enum ipsec_xmit_value
+extern enum ipsec_xmit_value
 ipsec_xmit_encap_bundle(struct ipsec_xmit_state *ixs);
 
 extern void ipsec_xsm(struct ipsec_xmit_state *ixs);
@@ -181,13 +183,10 @@ extern struct kmem_cache *ipsec_ixs_cache;
 extern int ipsec_ixs_max;
 extern atomic_t ipsec_ixs_cnt;
 
-extern void ipsec_extract_ports(struct iphdr * iph, struct sockaddr_encap * er);
-
-/* avoid forward reference complain on < 2.5 */
-struct flowi;
+extern void ipsec_extract_ports(struct sk_buff *skb, unsigned char nexthdr, int nexthdroff, struct sockaddr_encap * er);
 
 extern enum ipsec_xmit_value
-ipsec_xmit_send(struct ipsec_xmit_state*ixs, struct flowi *fl);
+ipsec_xmit_send(struct ipsec_xmit_state *ixs);
 
 extern enum ipsec_xmit_value
 ipsec_nat_encap(struct ipsec_xmit_state*ixs);
