@@ -17,13 +17,14 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * $Id: ipsec_kern24.c,v 1.2 2005-05-20 03:19:18 mcr Exp $
+ * $Id: ipsec_kern24.c,v 1.2 2005/05/20 03:19:18 mcr Exp $
  *
  */
 
 #include <linux/kernel.h>
 #include <linux/mm.h>
 #include <linux/spinlock.h>
+#include "openswan/ipsec_kversion.h"
 
 /*
  * printk rate limiting, lifted from the networking subsystem.
@@ -32,7 +33,7 @@
  * every printk_ratelimit_jiffies to make a denial-of-service
  * attack impossible.
  */
-static spinlock_t ratelimit_lock = SPIN_LOCK_UNLOCKED;
+static DEFINE_SPINLOCK(ratelimit_lock);
 
 int __printk_ratelimit(int ratelimit_jiffies, int ratelimit_burst)
 {
