@@ -17,10 +17,10 @@ char ipsec_mast_c_version[] = "Please use ipsec --version instead";
 
 #define __NO_VERSION__
 #include <linux/module.h>
-#ifndef AUTOCONF_INCLUDED
+#include <linux/version.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,38) && !defined(AUTOCONF_INCLUDED)
 #include <linux/config.h>
 #endif	/* for CONFIG_IP_FORWARD */
-#include <linux/version.h>
 #include <linux/kernel.h> /* printk() */
 
 #include "openswan/ipsec_param.h"
@@ -88,7 +88,7 @@ static __u32 zeroes[64];
 DEBUG_NO_STATIC int
 ipsec_mast_open(struct net_device *dev)
 {
-        struct mastpriv *prv = netdev_priv(dev); 
+	struct mastpriv *prv = netdev_to_mastpriv(dev); 
 
 	prv = prv;
 
@@ -212,8 +212,8 @@ ipsec_mast_send(struct ipsec_xmit_state*ixs)
 			    ixs->dev->name);
 		return IPSEC_XMIT_RECURSDETECT;
 	}
-	dst_release(ixs->skb->dst);
-	ixs->skb->dst = &ixs->route->u.dst;
+	dst_release(skb_dst(ixs->skb));
+	skb_dst_set(ixs->skb, &ixs->route->u.dst);
 	ixs->stats->tx_bytes += ixs->skb->len;
 	if(ixs->skb->len < ixs->skb->nh.raw - ixs->skb->data) {
 		ixs->stats->tx_errors++;
@@ -236,7 +236,7 @@ ipsec_mast_send(struct ipsec_xmit_state*ixs)
 	{
 		int err;
 
-		err = NF_HOOK(PF_INET, NF_IP_LOCAL_OUT, ixs->skb, NULL, ixs->route->u.dst.dev,
+		err = NF_HOOK(PF_INET, OSW_NF_INET_LOCAL_OUT, ixs->skb, NULL, ixs->route->u.dst.dev,
 			      ipsec_mast_xmit2);
 		if(err != NET_XMIT_SUCCESS && err != NET_XMIT_CN) {
 			if(net_ratelimit())
@@ -278,17 +278,7 @@ ipsec_mast_xsm_complete(
 	}
 #endif
 
-#ifdef NETDEV_25	/* 2.6 kernels */
-	/* now send the packet again */
-	{
-		struct flowi fl;
-		
-		memset(&fl, 0, sizeof(fl));
-		ipsec_xmit_send(ixs, &fl);
-	}
-#else
-	ipsec_xmit_send(ixs, NULL);
-#endif
+	ipsec_xmit_send(ixs);
 
 cleanup:
 	ipsec_xmit_cleanup(ixs);
@@ -304,6 +294,135 @@ cleanup:
 	ipsec_xmit_state_delete(ixs);
 }
 
+/*
+ * Verify that the skb can go out on this ipsp.
+ * Return 0 if OK, error code otherwise.
+ */
+static int
+ipsec_mast_check_outbound_policy(struct ipsec_xmit_state *ixs)
+{
+	int failed_outbound_check = 0;
+	struct ipsec_sa *ipsp = ixs->ipsp;
+
+	if (!ixs || !ixs->ipsp || !ixs->iph)
+		return -EFAULT;
+
+	/* Note: "xor" (^) logically replaces "not equal"
+	 * (!=) and "bitwise or" (|) logically replaces
+	 * "boolean or" (||).  This is done to speed up
+	 * execution by doing only bitwise operations and
+	 * no branch operations */
+	if (osw_ip_hdr_version(ixs) == 4) {
+		struct iphdr *ipp = osw_ip4_hdr(ixs);
+		if (ip_address_family(&ipsp->ips_said.dst) != AF_INET) {
+			failed_outbound_check = 1;
+		} else if (((ipp->saddr & ipsp->ips_mask_s.u.v4.sin_addr.s_addr)
+				^ ipsp->ips_flow_s.u.v4.sin_addr.s_addr)
+				| ((ipp->daddr & ipsp->ips_mask_d.u.v4.sin_addr.s_addr)
+				^ ipsp->ips_flow_d.u.v4.sin_addr.s_addr)) {
+			failed_outbound_check = 1;
+		}
+	} else if (osw_ip_hdr_version(ixs) == 6) {
+		struct ipv6hdr *ipp6 = osw_ip6_hdr(ixs);
+		if (ip_address_family(&ipsp->ips_said.dst) != AF_INET6) {
+			failed_outbound_check = 1;
+		} else if (((ipp6->saddr.s6_addr32[0] & ipsp->ips_mask_s.u.v6.sin6_addr.s6_addr32[0])
+				^ ipsp->ips_flow_s.u.v6.sin6_addr.s6_addr32[0])
+				| ((ipp6->daddr.s6_addr32[0] & ipsp->ips_mask_d.u.v6.sin6_addr.s6_addr32[0])
+				^ ipsp->ips_flow_d.u.v6.sin6_addr.s6_addr32[0])) {
+			failed_outbound_check = 1;
+		} else if (((ipp6->saddr.s6_addr32[1] & ipsp->ips_mask_s.u.v6.sin6_addr.s6_addr32[1])
+				^ ipsp->ips_flow_s.u.v6.sin6_addr.s6_addr32[1])
+				| ((ipp6->daddr.s6_addr32[1] & ipsp->ips_mask_d.u.v6.sin6_addr.s6_addr32[1])
+				^ ipsp->ips_flow_d.u.v6.sin6_addr.s6_addr32[1])) {
+			failed_outbound_check = 1;
+		} else if (((ipp6->saddr.s6_addr32[2] & ipsp->ips_mask_s.u.v6.sin6_addr.s6_addr32[2])
+				^ ipsp->ips_flow_s.u.v6.sin6_addr.s6_addr32[2])
+				| ((ipp6->daddr.s6_addr32[2] & ipsp->ips_mask_d.u.v6.sin6_addr.s6_addr32[2])
+				^ ipsp->ips_flow_d.u.v6.sin6_addr.s6_addr32[2])) {
+			failed_outbound_check = 1;
+		} else if (((ipp6->saddr.s6_addr32[3] & ipsp->ips_mask_s.u.v6.sin6_addr.s6_addr32[3])
+				^ ipsp->ips_flow_s.u.v6.sin6_addr.s6_addr32[3])
+				| ((ipp6->daddr.s6_addr32[3] & ipsp->ips_mask_d.u.v6.sin6_addr.s6_addr32[3])
+				^ ipsp->ips_flow_d.u.v6.sin6_addr.s6_addr32[3])) {
+			failed_outbound_check = 1;
+		}
+	}
+	if (failed_outbound_check) {
+		char saddr_txt[ADDRTOA_BUF], daddr_txt[ADDRTOA_BUF];
+		char sflow_txt[SUBNETTOA_BUF], dflow_txt[SUBNETTOA_BUF];
+
+		if (ipsp->ips_flow_s.u.v4.sin_family == AF_INET6) {
+			subnet6toa(&ipsp->ips_flow_s.u.v6.sin6_addr,
+					&ipsp->ips_mask_s.u.v6.sin6_addr,
+					0, sflow_txt, sizeof(sflow_txt));
+			subnet6toa(&ipsp->ips_flow_d.u.v6.sin6_addr,
+					&ipsp->ips_mask_d.u.v6.sin6_addr,
+					0, dflow_txt, sizeof(dflow_txt));
+			inet_addrtot(AF_INET6, &osw_ip6_hdr(ixs)->saddr, 0, saddr_txt,
+					sizeof(saddr_txt));
+			inet_addrtot(AF_INET6, &osw_ip6_hdr(ixs)->daddr, 0, daddr_txt,
+					sizeof(daddr_txt));
+		} else {
+			subnettoa(ipsp->ips_flow_s.u.v4.sin_addr,
+					ipsp->ips_mask_s.u.v4.sin_addr,
+					0, sflow_txt, sizeof(sflow_txt));
+			subnettoa(ipsp->ips_flow_d.u.v4.sin_addr,
+					ipsp->ips_mask_d.u.v4.sin_addr,
+					0, dflow_txt, sizeof(dflow_txt));
+			inet_addrtot(AF_INET, &osw_ip4_hdr(ixs)->saddr, 0, saddr_txt,
+					sizeof(saddr_txt));
+			inet_addrtot(AF_INET, &osw_ip4_hdr(ixs)->daddr, 0, daddr_txt,
+					sizeof(daddr_txt));
+		}
+
+		if (!ixs->sa_len) ixs->sa_len = KLIPS_SATOT(debug_mast,
+				&ixs->outgoing_said, 0,
+				ixs->sa_txt, sizeof(ixs->sa_txt));
+
+		KLIPS_PRINT(debug_mast,
+			    "klips_debug:ipsec_mast_check_outbound_policy: "
+			    "SA:%s, inner tunnel policy [%s -> %s] does not agree with pkt contents [%s -> %s].\n",
+			    ixs->sa_len ? ixs->sa_txt : " (error)",
+			    sflow_txt, dflow_txt, saddr_txt, daddr_txt);
+		if(ixs->stats)
+			ixs->stats->rx_dropped++;
+		return -EACCES;
+	}
+
+#if 0
+	{
+		char sflow_txt[SUBNETTOA_BUF], dflow_txt[SUBNETTOA_BUF];
+		char saddr_txt[ADDRTOA_BUF], daddr_txt[ADDRTOA_BUF];
+		struct in_addr ipaddr;
+
+		subnettoa(ixs->ipsp->ips_flow_s.u.v4.sin_addr,
+			  ixs->ipsp->ips_mask_s.u.v4.sin_addr,
+			  0, sflow_txt, sizeof(sflow_txt));
+		subnettoa(ixs->ipsp->ips_flow_d.u.v4.sin_addr,
+			  ixs->ipsp->ips_mask_d.u.v4.sin_addr,
+			  0, dflow_txt, sizeof(dflow_txt));
+
+		ipaddr.s_addr = ixs->iph->saddr;
+		addrtoa(ipaddr, 0, saddr_txt, sizeof(saddr_txt));
+		ipaddr.s_addr = ixs->iph->daddr;
+		addrtoa(ipaddr, 0, daddr_txt, sizeof(daddr_txt));
+
+		if (!ixs->sa_len) ixs->sa_len = KLIPS_SATOT(debug_mast,
+				&ixs->outgoing_said, 0,
+				ixs->sa_txt, sizeof(ixs->sa_txt));
+
+		KLIPS_PRINT(debug_mast,
+			    "klips_debug:ipsec_mast_check_outbound_policy: "
+			    "SA:%s, inner tunnel policy [%s -> %s] agrees with pkt contents [%s -> %s].\n",
+			    ixs->sa_len ? ixs->sa_txt : " (error)",
+			    sflow_txt, dflow_txt, saddr_txt, daddr_txt);
+	}
+#endif
+
+	return 0;
+}
+
 /*
  *	This function assumes it is being called from dev_queue_xmit()
  *	and that skb is filled properly by that function.
@@ -321,23 +440,23 @@ ipsec_mast_start_xmit(struct sk_buff *skb, struct net_device *dev)
 		return 0;
 	}
 		
-	ixs = ipsec_xmit_state_new();
-	if(ixs == NULL) {
-		printk("ipsec_mast_start_xmit: "
-			"failed to allocate IXS\n");
-		return 0;
-	}
+	ixs = ipsec_xmit_state_new(dev);
+	if(ixs == NULL)
+		return NETDEV_TX_BUSY;
 
+	ixs->dev = dev;
 	ixs->skb = skb;
 	SAref = 0;
 #ifdef NETDEV_25
-	if(skb->nfmark & 0x80000000) {
+#if defined(CONFIG_NETFILTER)
+	if(skb->nfmark & IPSEC_NFMARK_IS_SAREF_BIT) {
 		SAref = NFmark2IPsecSAref(skb->nfmark);
 		KLIPS_PRINT(debug_mast, "klips_debug:ipsec_mast_start_xmit: "
 				"getting SAref=%d from nfmark\n",
 				SAref);
 	}
 #endif
+#endif
 
 #ifdef CONFIG_INET_IPSEC_SAREF
 	if(skb->sp && skb->sp->ref != IPSEC_SAREF_NULL) {
@@ -348,7 +467,17 @@ ipsec_mast_start_xmit(struct sk_buff *skb, struct net_device *dev)
 	}
 #endif
 
-	ipsec_xmit_sanity_check_skb(ixs);
+	if (ipsec_xmit_sanity_check_mast_dev(ixs) != IPSEC_XMIT_OK) {
+		ipsec_xmit_cleanup(ixs);
+		ipsec_xmit_state_delete(ixs);
+		return 0;
+	}
+
+	if (ipsec_xmit_sanity_check_skb(ixs) != IPSEC_XMIT_OK) {
+		ipsec_xmit_cleanup(ixs);
+		ipsec_xmit_state_delete(ixs);
+		return 0;
+	}
 
 	ixs->ipsp = ipsec_sa_getbyref(SAref, IPSEC_REFOTHER);
 	if(ixs->ipsp == NULL) {
@@ -360,15 +489,24 @@ ipsec_mast_start_xmit(struct sk_buff *skb, struct net_device *dev)
 		return 0;
 	}
 
+	/* make sure this packet can go out on this SA */
+	if (ipsec_mast_check_outbound_policy(ixs)) {
+		ipsec_xmit_cleanup(ixs);
+		ipsec_xmit_state_delete(ixs);
+		return 0;
+	}
+
 	/* fill in outgoing_said using the ipsp we have */
 	ixs->outgoing_said = ixs->ipsp->ips_said;
 
 #ifdef NETDEV_25
+#if defined(CONFIG_NETFILTER)
 	/* prevent recursion through the saref route */
 	if(skb->nfmark & 0x80000000) {
 		skb->nfmark = 0;
 	}
 #endif
+#endif
 #if 0
 	/* TODO: do we have to also have to do this? */
 	if(skb->sp && skb->sp->ref != IPSEC_SAREF_NULL) {
@@ -377,14 +515,6 @@ ipsec_mast_start_xmit(struct sk_buff *skb, struct net_device *dev)
 	}
 #endif
 
-	/*
-	 * we should be calculating the MTU by looking up a route
-	 * based upon the destination in the SA, and then cache
-	 * it into the SA, but we don't do that right now.
-	 */
-	ixs->cur_mtu = 1460;
-	ixs->physmtu = 1460;
-
 	ixs->mast_mode = 1;
 	ixs->xsm_complete = ipsec_mast_xsm_complete;
 	ixs->state = IPSEC_XSM_INIT2;	/* we start later in the process */
@@ -397,7 +527,7 @@ ipsec_mast_start_xmit(struct sk_buff *skb, struct net_device *dev)
 DEBUG_NO_STATIC struct net_device_stats *
 ipsec_mast_get_stats(struct net_device *dev)
 {
-	return &(((struct mastpriv *)netdev_priv(dev))->mystats);
+	return &(netdev_to_mastpriv(dev)->mystats);
 }
 
 #if 0
@@ -409,7 +539,7 @@ DEBUG_NO_STATIC int
 ipsec_mast_hard_header(struct sk_buff *skb, struct net_device *dev,
 	unsigned short type, void *daddr, void *saddr, unsigned len)
 {
-	struct mastpriv *prv = (struct mastpriv *)netdev_priv(dev);
+	struct mastpriv *mprv = netdev_to_mastpriv(dev);
 	struct net_device_stats *stats;	/* This device's statistics */
 	int ret = 0;
 	
@@ -483,7 +613,7 @@ ipsec_mast_hard_header(struct sk_buff *skb, struct net_device *dev,
 DEBUG_NO_STATIC int
 ipsec_mast_rebuild_header(struct sk_buff *skb)
 {
-	struct mastpriv *prv = (struct mastpriv *)netdev_priv(skb->dev);
+	struct mastpriv *prv = netdev_to_mastpriv(skb->dev);
 
 	prv = prv;
 	return 0;
@@ -492,7 +622,7 @@ ipsec_mast_rebuild_header(struct sk_buff *skb)
 DEBUG_NO_STATIC int
 ipsec_mast_set_mac_address(struct net_device *dev, void *addr)
 {
-	struct mastpriv *prv = (struct mastpriv *)netdev_priv(dev);
+	struct mastpriv *prv = netdev_to_mastpriv(dev);
 	
 	prv = prv;
 	return 0;
@@ -502,7 +632,7 @@ ipsec_mast_set_mac_address(struct net_device *dev, void *addr)
 DEBUG_NO_STATIC void
 ipsec_mast_cache_update(struct hh_cache *hh, struct net_device *dev, unsigned char *  haddr)
 {
-	struct mastpriv *prv = (struct mastpriv *)netdev_priv(dev);
+	struct mastpriv *prv = netdev_to_mastpriv(dev);
 	
 	if(dev == NULL) {
 		KLIPS_PRINT(debug_mast & DB_MAST_REVEC,
@@ -533,7 +663,9 @@ ipsec_mast_neigh_setup(struct neighbour *n)
 		    "klips_debug:ipsec_mast_neigh_setup:\n");
 
         if (n->nud_state == NUD_NONE) {
+#ifndef PRIVATE_ARP_BROKEN_OPS
                 n->ops = &arp_broken_ops;
+#endif
                 n->output = n->ops->output;
         }
         return 0;
@@ -561,10 +693,10 @@ ipsec_mast_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd)
 	/* struct ipsecmastconf *cf = (struct ipsecmastconf *)&ifr->ifr_data;*/
 	/* overlay our struct ipsecmast onto ifr.ifr_ifru union (hope it fits!) */
 	struct ipsecmastconf *cf=(struct ipsecmastconf *)ifr->ifr_ifru.ifru_newname;       
-	struct ipsecpriv *prv = netdev_priv(dev);
+	struct mastpriv *mprv = netdev_to_mastpriv(dev);
 
 	cf = cf;
-	prv=prv;
+	mprv = mprv;
 	
 	if(dev == NULL) {
 		KLIPS_PRINT(debug_mast & DB_MAST_INIT,
@@ -594,7 +726,7 @@ int
 ipsec_mast_device_event(struct notifier_block *unused, unsigned long event, void *ptr)
 {
 	struct net_device *dev = ptr;
-	struct mastpriv *priv = netdev_priv(dev);
+	struct mastpriv *priv = netdev_to_mastpriv(dev);
 
 	priv = priv;
 
@@ -716,7 +848,7 @@ int
 ipsec_mast_probe(struct net_device *dev)
 {
 	int i;
-	struct ipsecpriv *ipriv;
+	struct mastpriv *mprv;
 
 	KLIPS_PRINT(debug_mast,
 		    "klips_debug:ipsec_mast_probe: "
@@ -744,8 +876,9 @@ ipsec_mast_probe(struct net_device *dev)
 	if (dev->priv == NULL)
 		return -ENOMEM;
 #endif
-	ipriv = netdev_priv(dev);
-	memset(netdev_priv(dev), 0, sizeof(struct mastpriv));
+	mprv = netdev_priv(dev);
+	memset(mprv, 0, sizeof(struct mastpriv));
+	mprv->magic = MASTPRIV_MAGIC;
 
 	for(i = 0; i < sizeof(zeroes); i++) {
 		((__u8*)(zeroes))[i] = 0;
@@ -804,7 +937,7 @@ int ipsec_mast_createnum(int vifnum)
 	int vifentry;
 	char name[IFNAMSIZ];
 
-	if(vifnum > IPSEC_NUM_IFMAX) {
+	if(vifnum >= IPSEC_NUM_IFMAX) {
 		return -ENOENT;
 	}
 
@@ -859,7 +992,7 @@ ipsec_mast_deletenum(int vifnum)
 {
 	struct net_device *dev_ipsec;
 	
-	if(vifnum > IPSEC_NUM_IFMAX) {
+	if(vifnum >= IPSEC_NUM_IFMAX) {
 		return -ENOENT;
 	}
 
@@ -872,9 +1005,7 @@ ipsec_mast_deletenum(int vifnum)
 	mastdevices[vifnum]=NULL;
 	ipsec_dev_put(dev_ipsec);
 	
-	KLIPS_PRINT(debug_tunnel, "Unregistering %s (refcnt=%d)\n",
-		    dev_ipsec->name,
-		    atomic_read(&dev_ipsec->refcnt));
+	KLIPS_PRINT(debug_tunnel, "Unregistering %s\n", dev_ipsec->name);
 	unregister_netdev(dev_ipsec);
 	KLIPS_PRINT(debug_tunnel, "Unregisted %s\n", dev_ipsec->name);
 #ifndef NETDEV_23
@@ -917,16 +1048,6 @@ ipsec_mast_get_device(int vifnum)
 	}
 }
 
-int
-ipsec_is_mast_device(const struct net_device *dev)
-{
-#ifndef USE_NETDEV_OPS
-	return dev && (dev->hard_start_xmit == ipsec_mast_start_xmit);
-#else
-	return dev && (dev->netdev_ops == &ipsec_mast_ops);
-#endif
-}
-
 unsigned int
 ipsec_mast_is_transport(int vifnum)
 {
@@ -959,14 +1080,13 @@ ipsec_mast_cleanup_devices(void)
 	for(i = 0; i <= mastdevices_max; i++) {
 		if(mastdevices[i]!=NULL) {
 			dev_mast = mastdevices[i];
+			mastdevices[i]=NULL;
 			ipsec_dev_put(dev_mast);
 			unregister_netdev(dev_mast);
 #ifndef alloc_netdev
 			kfree(dev_mast->priv);
 			dev_mast->priv=NULL;
 #endif
-			ipsec_dev_put(mastdevices[i]);
-			mastdevices[i]=NULL;
 		}
 	}
 	return error;
