@@ -18,14 +18,14 @@
  * Split out from ipsec_init.c version 1.70.
  */
 
-#ifndef AUTOCONF_INCLUDED
-#include <linux/config.h>
-#endif
 #include <linux/version.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,38) && !defined(AUTOCONF_INCLUDED)
+# include <linux/config.h>
+#endif
 #define __NO_VERSION__
 #include <linux/module.h>
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) 
-#include <linux/moduleparam.h> 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,26)
+# include <linux/moduleparam.h> 
 #endif 
 #include <linux/kernel.h> /* printk() */
 #include <linux/ip.h>          /* struct iphdr */
@@ -91,6 +91,11 @@
 #include <openswan/pfkeyv2.h>
 #include <openswan/pfkey.h>
 
+#include <linux/in.h>
+#if defined(IP_IPSEC_REFINFO) || defined(IP_IPSEC_BINDREF)
+#define IPSEC_PROC_SHOW_SAREF_INFO
+#endif
+
 #ifdef CONFIG_PROC_FS
 
 #ifdef IPSEC_PROC_SUBDIRS
@@ -188,14 +193,14 @@ ipsec_spi_format(struct ipsec_sa *sa_p,
 		       "in " : "out");
 
 	if(sa_p->ips_addr_s) {
-		addrtoa(((struct sockaddr_in*)(sa_p->ips_addr_s))->sin_addr,
-			0, buf_s, sizeof(buf_s));
-		len += ipsec_snprintf(buffer+len, length-len, " src=%s",
-			       buf_s);
+		sin_addrtot(sa_p->ips_addr_s, 0, buf_s, sizeof(buf_s));
+		len += ipsec_snprintf(buffer+len, length-len, " src=%s", buf_s);
 	}
 
 	if((sa_p->ips_said.proto == IPPROTO_IPIP)
-	   && (sa_p->ips_flags & SADB_X_SAFLAGS_INFLOW)) {
+	   && (sa_p->ips_flags & (SADB_X_SAFLAGS_INFLOW
+			   |SADB_X_SAFLAGS_POLICYONLY))) {
+		if (sa_p->ips_flow_s.u.v4.sin_family == AF_INET) {
 		subnettoa(sa_p->ips_flow_s.u.v4.sin_addr,
 			  sa_p->ips_mask_s.u.v4.sin_addr,
 			  0,
@@ -207,6 +212,19 @@ ipsec_spi_format(struct ipsec_sa *sa_p,
 			  0,
 			  buf_d,
 			  sizeof(buf_d));
+		} else {
+		subnet6toa(&sa_p->ips_flow_s.u.v6.sin6_addr,
+			  &sa_p->ips_mask_s.u.v6.sin6_addr,
+			  0,
+			  buf_s,
+			  sizeof(buf_s));
+
+		subnet6toa(&sa_p->ips_flow_d.u.v6.sin6_addr,
+			  &sa_p->ips_mask_d.u.v6.sin6_addr,
+			  0,
+			  buf_d,
+			  sizeof(buf_d));
+		}
 
 		len += ipsec_snprintf(buffer+len, length-len, " policy=%s->%s",
 			       buf_s, buf_d);
@@ -626,11 +644,11 @@ ipsec_tncfg_get_info(char *buffer,
 		    (int)offset,
 		    length);
 
-	for(i = 0; i < IPSEC_NUM_IF; i++) {
+	for(i = 0; i < IPSEC_NUM_IFMAX; i++) {
 		ipsec_snprintf(name, (ssize_t) sizeof(name), IPSEC_DEV_FORMAT, i);
 		dev = __ipsec_dev_get(name);
 		if(dev) {
-			priv = (struct ipsecpriv *)netdev_priv(dev);
+			priv = netdev_to_ipsecpriv(dev);
 			len += ipsec_snprintf(buffer+len, length-len, "%s",
 				       dev->name);
 			if(priv) {
@@ -715,6 +733,81 @@ ipsec_version_get_info(char *buffer,
 	return len;
 }
 
+#ifdef IPSEC_PROC_SHOW_SAREF_INFO
+IPSEC_PROCFS_DEBUG_NO_STATIC
+int
+ipsec_saref_get_info(char *buffer,
+		       char **start,
+		       off_t offset,
+		       int length  IPSEC_PROC_LAST_ARG)
+{
+	int rc, len = 0;
+	off_t begin = 0;
+
+	KLIPS_PRINT(debug_tunnel & DB_TN_PROCFS,
+		    "klips_debug:ipsec_saref_get_info: "
+		    "buffer=0p%p, *start=0p%p, offset=%d, length=%d\n",
+		    buffer,
+		    *start,
+		    (int)offset,
+		    length);
+
+#ifdef IP_IPSEC_REFINFO
+	len += rc = ipsec_snprintf(buffer + len,length-len, "refinfo patch applied\n",
+		       ipsec_version_code());
+	if (rc<0) return 0;
+#endif
+
+#ifdef IP_IPSEC_BINDREF
+	len += rc = ipsec_snprintf(buffer + len,length-len, "bindref patch applied\n",
+		       ipsec_version_code());
+	if (rc<0) return 0;
+#endif
+
+#ifdef CONFIG_INET_IPSEC_SAREF
+	len += rc = ipsec_snprintf(buffer + len,length-len, "saref enabled\n",
+		       ipsec_version_code());
+#else
+	len += rc = ipsec_snprintf(buffer + len,length-len, "saref disabled\n",
+		       ipsec_version_code());
+#endif
+	if (rc<0) return 0;
+
+	*start = buffer + (offset - begin);	/* Start of wanted data */
+	len -= (offset - begin);			/* Start slop */
+	if (len > length)
+		len = length;
+	return len;
+}
+#endif
+
+#ifdef CONFIG_KLIPS_OCF
+unsigned int ocf_available = 1;
+#else
+unsigned int ocf_available = 0;
+#endif
+module_param(ocf_available,int,0644);
+
+IPSEC_PROCFS_DEBUG_NO_STATIC
+int
+ipsec_ocf_get_info(char *buffer,
+		    char **start,
+		    off_t offset,
+		    int length  IPSEC_PROC_LAST_ARG)
+{
+	int len = 0;
+	off_t begin = 0;
+
+	len += ipsec_snprintf(buffer + len,
+			      length-len, "%d\n", ocf_available);
+
+	*start = buffer + (offset - begin);	/* Start of wanted data */
+	len -= (offset - begin);			/* Start slop */
+	if (len > length)
+		len = length;
+	return len;
+}
+
 #ifdef CONFIG_IPSEC_NAT_TRAVERSAL
 unsigned int natt_available = 1;
 #elif defined (HAVE_UDP_ENCAP_CONVERT)
@@ -924,6 +1017,18 @@ struct proc_dir_entry ipsec_version =
 	NULL, NULL, NULL, NULL, NULL
 };
 
+#ifdef IPSEC_PROC_SHOW_SAREF_INFO
+struct proc_dir_entry ipsec_saref =
+{
+	0,
+	13, "ipsec_saref",
+	S_IFREG | S_IRUGO, 1, 0, 0, 0,
+	&proc_net_inode_operations,
+	ipsec_saref_get_info,
+	NULL, NULL, NULL, NULL, NULL
+};
+#endif
+
 struct proc_dir_entry ipsec_klipsdebug =
 {
 	0,
@@ -969,7 +1074,11 @@ static struct ipsec_proc_list proc_items[]={
 	{"trap_count", &proc_stats_dir,     NULL,             ipsec_stats_get_int_info, NULL, &ipsec_xmit_trap_count},
 	{"trap_sendcount", &proc_stats_dir, NULL,             ipsec_stats_get_int_info, NULL, &ipsec_xmit_trap_sendcount},
 	{"natt",       &proc_net_ipsec_dir, NULL,             ipsec_natt_get_info,    NULL, NULL},
+	{"ocf",       &proc_net_ipsec_dir, NULL,             ipsec_ocf_get_info,    NULL, NULL},
 	{"version",    &proc_net_ipsec_dir, NULL,             ipsec_version_get_info,    NULL, NULL},
+#ifdef IPSEC_PROC_SHOW_SAREF_INFO
+	{"saref",      &proc_net_ipsec_dir, NULL,             ipsec_saref_get_info,    NULL, NULL},
+#endif
 	{NULL,         NULL,                NULL,             NULL,      NULL, NULL}
 };
 #endif
@@ -1039,7 +1148,8 @@ ipsec_proc_init()
 				item = proc_mkdir(it->name, *it->parent);
 				*it->dir = item;
 			} else {
-				item = create_proc_entry(it->name, 0400, *it->parent);
+				/* FIXME: we put the mode in the struct proc_dir_entry, but it is not used here?? */
+				item = create_proc_entry(it->name, strcmp(it->name, "version") == 0 ? 0444 : 0400, *it->parent);
 			}
 			if(item) {
 				item->read_proc  = it->readthing;
