@@ -12,15 +12,12 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pfkey_v2_parse.c,v 1.65 2005/04/06 17:46:05 mcr Exp $
  */
 
 /*
  *		Template from klips/net/ipsec/ipsec/ipsec_parser.c.
  */
 
-char pfkey_v2_parse_c_version[] = "$Id: pfkey_v2_parse.c,v 1.65 2005/04/06 17:46:05 mcr Exp $";
-
 /*
  * Some ugly stuff to allow consistent debugging code for use in the
  * kernel and in user space
@@ -44,9 +41,9 @@ char pfkey_v2_parse_c_version[] = "$Id: pfkey_v2_parse.c,v 1.65 2005/04/06 17:46
 # include <linux/netdevice.h>   /* struct device, and other headers */
 # include <linux/etherdevice.h> /* eth_type_trans */
 # include <linux/ip.h>          /* struct iphdr */ 
-# if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE)
+# if defined(CONFIG_KLIPS_IPV6)
 #  include <linux/ipv6.h>        /* struct ipv6hdr */
-# endif /* if defined(CONFIG_IPV6) || defined(CONFIG_IPV6_MODULE) */
+# endif /* if defined(CONFIG_KLIPS_IPV6) */
 extern int debug_pfkey;
 
 # include <openswan.h>
@@ -79,14 +76,13 @@ unsigned int pfkey_lib_debug = PF_KEY_DEBUG_PARSE_NONE;
 int (*pfkey_debug_func)(const char *message, ...) PRINTF_LIKE(1);
 int (*pfkey_error_func)(const char *message, ...) PRINTF_LIKE(1);
 
-
 #define SENDERR(_x) do { error = -(_x); goto errlab; } while (0)
 
-struct satype_tbl {
+static struct satype_tbl {
 	uint8_t proto;
 	uint8_t satype;
 	char* name;
-} static satype_tbl[] = {
+} satype_tbl[] = {
 #ifdef __KERNEL__
 	{ IPPROTO_ESP,	K_SADB_SATYPE_ESP,	"ESP"  },
 	{ IPPROTO_AH,	K_SADB_SATYPE_AH,		"AH"   },
@@ -384,11 +380,7 @@ pfkey_address_parse(struct sadb_ext *pfkey_ext)
 	switch(s->sa_family) {
 	case AF_INET:
 		saddr_len = sizeof(struct sockaddr_in);
-		sprintf(ipaddr_txt, "%d.%d.%d.%d"
-			, (((struct sockaddr_in*)s)->sin_addr.s_addr >>  0) & 0xFF
-			, (((struct sockaddr_in*)s)->sin_addr.s_addr >>  8) & 0xFF
-			, (((struct sockaddr_in*)s)->sin_addr.s_addr >> 16) & 0xFF
-			, (((struct sockaddr_in*)s)->sin_addr.s_addr >> 24) & 0xFF);
+		sin_addrtot(s, 0, ipaddr_txt, sizeof(ipaddr_txt));
 		DEBUGGING(PF_KEY_DEBUG_PARSE_STRUCT,
 			  "pfkey_address_parse: "
 			  "found exttype=%u(%s) family=%d(AF_INET) address=%s proto=%u port=%u.\n",
@@ -401,15 +393,7 @@ pfkey_address_parse(struct sadb_ext *pfkey_ext)
 		break;
 	case AF_INET6:
 		saddr_len = sizeof(struct sockaddr_in6);
-		sprintf(ipaddr_txt, "%x:%x:%x:%x:%x:%x:%x:%x"
-			, ntohs(((struct sockaddr_in6*)s)->sin6_addr.s6_addr16[0])
-			, ntohs(((struct sockaddr_in6*)s)->sin6_addr.s6_addr16[1])
-			, ntohs(((struct sockaddr_in6*)s)->sin6_addr.s6_addr16[2])
-			, ntohs(((struct sockaddr_in6*)s)->sin6_addr.s6_addr16[3])
-			, ntohs(((struct sockaddr_in6*)s)->sin6_addr.s6_addr16[4])
-			, ntohs(((struct sockaddr_in6*)s)->sin6_addr.s6_addr16[5])
-			, ntohs(((struct sockaddr_in6*)s)->sin6_addr.s6_addr16[6])
-			, ntohs(((struct sockaddr_in6*)s)->sin6_addr.s6_addr16[7]));
+		sin_addrtot(s, 0, ipaddr_txt, sizeof(ipaddr_txt));
 		DEBUGGING(PF_KEY_DEBUG_PARSE_STRUCT,
 			  "pfkey_address_parse: "
 			  "found exttype=%u(%s) family=%d(AF_INET6) address=%s proto=%u port=%u.\n",
