@@ -33,15 +33,14 @@
 
 #define LOCK_PRNG() \
 	int ul = 0; \
-	if (!spin_is_locked(&tdb_lock)) { \
-		spin_lock_bh(&tdb_lock); \
+	if (spin_trylock_bh(&tdb_lock)) { \
 		ul = 1; \
 	} else
 
 #define UNLOCK_PRNG() \
 	if (ul) { \
-		spin_unlock_bh(&tdb_lock); \
 		ul = 0; \
+		spin_unlock_bh(&tdb_lock); \
 	} else
 
 #else
@@ -234,8 +233,8 @@ regress()
 		exit(1);
 	}
 	if (prng_count(&pr) != sizeof(buf)) {
-		fprintf(stderr, "got %u bytes, but count is %lu\n",
-					sizeof(buf), prng_count(&pr));
+		fprintf(stderr, "got %lu bytes, but count is %lu\n",
+			(unsigned long)sizeof(buf), prng_count(&pr));
 		exit(1);
 	}
 	prng_final(&pr);
