@@ -12,7 +12,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ikeping.c,v 1.13 2005/07/08 02:56:38 paul Exp $
  */
 
 #include <stdio.h>
@@ -195,6 +194,7 @@ receive_ping(int afamily, int s, int reply, int natt)
 	unsigned int sendlen;
 	const char *xchg_name;
 	int xchg;
+	u_int32_t tmp_ic, tmp_rc;
 
 	rport = 500;
 	xchg  = 0;
@@ -254,7 +254,6 @@ receive_ping(int afamily, int s, int reply, int natt)
 	printf("received %d(%s) packet from %s/%d of len: %d\n",
 	       ih.isa_xchg, xchg_name, buf, ntohs(rport), n);
 
-	u_int32_t tmp_ic, tmp_rc;
 	memcpy(&tmp_ic, ih.isa_icookie, sizeof(u_int32_t));
 	memcpy(&tmp_rc, ih.isa_rcookie, sizeof(u_int32_t));
 	printf("\trcookie=%08x_%08x icookie=%08x_%08x msgid=%08x\n",
@@ -303,10 +302,10 @@ main(int argc, char **argv)
   int   afamily;
   int   pfamily;
   int   c;
-  int   numSenders, numReceived, noDNS;
+  int   numSenders, numReceived;
   int   natt;
   int   waitTime;
-  int   verbose, timedOut;
+  int   verbose;
   ip_address laddr, raddr;
   char *afam = "";
 
@@ -318,10 +317,9 @@ main(int argc, char **argv)
   verbose=0;
   natt=0;
   listen_only=0;
-  noDNS=0;
   bzero(&laddr, sizeof(laddr));
 
-  while((c = getopt_long(argc, argv, "hVnvsp:b:46E:w:", long_opts, 0))!=EOF) {
+  while((c = getopt_long(argc, argv, "hVvsp:b:46E:w:", long_opts, 0))!=EOF) {
       switch (c) {
       case 'h':	        /* --help */
 	  help();
@@ -335,10 +333,6 @@ main(int argc, char **argv)
 	  verbose++;
 	  continue;
 	  
-      case 'n':
-	  noDNS=1;
-	  break;
-	  
       case 'T':
 	  natt++;
 	  break;
@@ -483,7 +477,6 @@ main(int argc, char **argv)
 	  }
   }
 
-  timedOut = 0;
   numReceived=0;
 
   /* really should catch ^C and print stats on exit */
