@@ -187,7 +187,7 @@ write_pipe(int fd, const unsigned char *stuff)
     res_query(dname, class, type, answer, anslen)
 # define res_nclose(statp) res_close()
 
-static struct __res_state *statp = &_res;
+#define statp  ((struct __res_state *)(&_res))
 
 #else /* !OLD_RESOLVER */
 
@@ -358,6 +358,9 @@ send_eof(struct worker_info *w)
     /* reap child */
     p = waitpid(w->pid, &status, 0);
     /* ignore result -- what could we do with it? */
+    if(p == -1) {
+	    syslog(LOG_ERR, "waitpid(2) failed, ignored");
+    }
 }
 
 static void
