@@ -19,19 +19,18 @@
 
 #ifndef USE_LWRES	/* whole file! */
 
-/* even though we aren't using LWRES, we can still use the header files */
-#include "arpa/nameser.h"
+#include <resolv.h>
 
 /* The interface in RHL6.x and BIND distribution 8.2.2 are different,
  * so we build some of our own :-(
  */
 
 # ifndef NS_MAXDNAME
-#   define NS_MAXDNAME MAXDNAME /* I hope this is long enough for IPv6 */
+#   define NS_MAXDNAME 1025
 # endif
 
 # ifndef NS_PACKETSZ
-#   define NS_PACKETSZ PACKETSZ
+#   define NS_PACKETSZ 512
 # endif
 
 /* protocol version */
@@ -48,7 +47,7 @@ struct adns_query {
     unsigned int qmagic;
     unsigned long serial;
     lset_t debugging;	/* only used #ifdef DEBUG, but don't want layout to change */
-    u_char name_buf[NS_MAXDNAME + 2];
+    char name_buf[NS_MAXDNAME + 2];
     int type;	/* T_KEY or T_TXT */
 };
 
