@@ -1,9 +1,9 @@
 /* information about connections between hosts and clients
  *
  * Copyright (C) 1998-2002  D. Hugh Redelmeier.
- * Copyright (C) 2003-2009 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2003-2010 Paul Wouters <paul@xelerance.com>
  * Copyright (C) 2003-2008 Michael Richardson <mcr@xelerance.com>
- * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
+ * Copyright (C) 2009-2010 Avesh Agarwal <avagarwa@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -25,8 +25,8 @@
 #include <netinet/in.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
-#include <netinet/in.h>
 #include <arpa/inet.h>
+#include <resolv.h>
 
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
@@ -318,6 +318,11 @@ delete_connection(struct connection *c, bool relations)
     set_debugging(old_cur_debugging);
 #endif
     pfreeany(c->name);
+#ifdef XAUTH
+    pfreeany(c->cisco_dns_info);
+    pfreeany(c->cisco_domain_info);
+    pfreeany(c->cisco_banner);
+#endif
 #ifdef DYNAMICDNS
     pfreeany(c->dnshostname);
 #endif /* DYNAMICDNS */
@@ -633,7 +638,7 @@ format_end(char *buf
 	const char *plus = "+";
 	endopts[0]='\0';
 
-	if(id_obrackets && id_obrackets[0]=='[')
+	if(id_obrackets[0]=='[')
 	{
 	    id_comma=",";
 	} else {
@@ -664,7 +669,7 @@ format_end(char *buf
 	    const char *send_cert = "";
 	    char s[32];
 
-	    send_cert=""; // Length 3 because cert.type is 1-11
+	    send_cert=""; /* Length 3 because cert.type is 1-11 */
 	    
 	    switch(this->sendcert) {
 	    case cert_neversend:
@@ -755,6 +760,12 @@ unshare_connection_strings(struct connection *c)
 
     c->name = clone_str(c->name, "connection name");
 
+#ifdef XAUTH
+    c->cisco_dns_info = clone_str(c->cisco_dns_info, "connection cisco_dns_info");
+    c->cisco_domain_info = clone_str(c->cisco_domain_info, "connection cisco_domain_info");
+    c->cisco_banner = clone_str(c->cisco_banner, "connection cisco_banner");
+#endif
+
 #ifdef DYNAMICDNS
     c->dnshostname = clone_str(c->dnshostname, "connection dnshostname");
 #endif /* DYNAMICDNS */
@@ -770,11 +781,15 @@ unshare_connection_strings(struct connection *c)
 	
     /* increment references to algo's, if any */
     if(c->alg_info_ike) {
+#ifdef KERNEL_ALG
 	alg_info_addref(IKETOINFO(c->alg_info_ike));
+#endif
     }
 
     if(c->alg_info_esp) {
+#ifdef KERNEL_ALG
 	alg_info_addref(ESPTOINFO(c->alg_info_esp));
+#endif
     }
 }
 
@@ -785,7 +800,6 @@ load_end_certificate(const char *filename, struct end *dst)
     time_t valid_until;
     cert_t cert;
     err_t ugh = NULL;
-    bool cached_cert = FALSE;
 
     memset(&dst->cert, 0, sizeof(dst->cert));
 
@@ -822,15 +836,10 @@ load_end_certificate(const char *filename, struct end *dst)
     case CERT_PGP:
 	select_pgpcert_id(cert.u.pgp, &dst->id);
 	
-	if (cached_cert)
-	    dst->cert = cert;
-	else
-	{
-	    valid_until = cert.u.pgp->until;
-	    add_pgp_public_key(cert.u.pgp, cert.u.pgp->until, DAL_LOCAL);
-	    dst->cert.type = cert.type;
-	    dst->cert.u.pgp = pluto_add_pgpcert(cert.u.pgp);
-	}
+	valid_until = cert.u.pgp->until;
+	add_pgp_public_key(cert.u.pgp, cert.u.pgp->until, DAL_LOCAL);
+	dst->cert.type = cert.type;
+	dst->cert.u.pgp = pluto_add_pgpcert(cert.u.pgp);
 	break;
 	
     case CERT_X509_SIGNATURE:
@@ -838,12 +847,9 @@ load_end_certificate(const char *filename, struct end *dst)
 	    select_x509cert_id(cert.u.x509, &dst->id);
 	}
 	
-	if (!cached_cert)
-	{
-	    /* check validity of cert */
-	    valid_until = cert.u.x509->notAfter;
-	    ugh = check_validity(cert.u.x509, &valid_until);
-	}
+	/* check validity of cert */
+	valid_until = cert.u.x509->notAfter;
+	ugh = check_validity(cert.u.x509, &valid_until);
 	if (ugh != NULL)
 	{
 	    openswan_log("  %s", ugh);
@@ -854,14 +860,10 @@ load_end_certificate(const char *filename, struct end *dst)
 	    DBG(DBG_CONTROL,
 		DBG_log("certificate is valid")
 		);
-	    if (cached_cert)
-		dst->cert = cert;
-	    else
-	    {
-		add_x509_public_key(&dst->id, cert.u.x509, valid_until, DAL_LOCAL);
-		dst->cert.type = cert.type;
-		dst->cert.u.x509 = add_x509cert(cert.u.x509);
-	    }
+	    add_x509_public_key(&dst->id, cert.u.x509, valid_until, DAL_LOCAL);
+	    dst->cert.type = cert.type;
+	    dst->cert.u.x509 = add_x509cert(cert.u.x509);
+
 	    /* if no CA is defined, use issuer as default */
 	    if (dst->ca.ptr == NULL)
 		dst->ca = dst->cert.u.x509->issuer;
@@ -981,11 +983,16 @@ extract_end(struct end *dst, const struct whack_end *src, const char *which)
      */
     {
 	err_t er;
+	int port;
 	
 	switch(dst->host_type) {
 	case KH_IPHOSTNAME:
 	    er = ttoaddr(dst->host_addr_name, 0, 0, &dst->host_addr);
-	    
+
+	    /*The above call wipes out the port, put it again*/
+	    port = htons(dst->port);
+	    setportof(port, &dst->host_addr);
+		
 	    if(er) {
 		loglog(RC_COMMENT, "failed to convert '%s' at load time: %s"
 		       , dst->host_addr_name, er);
@@ -1015,7 +1022,7 @@ static bool
 check_connection_end(const struct whack_end *this, const struct whack_end *that
 , const struct whack_message *wm)
 {
-    if (this->host_type == KH_IPADDR
+    if ((this->host_type == KH_IPADDR || this->host_type == KH_IFACE)
 	&& (wm->addr_family != addrtypeof(&this->host_addr)
 	    || wm->addr_family != addrtypeof(&this->host_nexthop)))
     {
@@ -1028,18 +1035,6 @@ check_connection_end(const struct whack_end *this, const struct whack_end *that
 	       , addrtypeof(&this->host_nexthop));
 	return FALSE;
     }
-    /* this check actually prevents IPv4 in IPv6 and vv, so it will
-     * have to go away at some point.
-     */
-    if ((this->has_client? wm->tunnel_addr_family : wm->addr_family)
-	!= subnettypeof(&this->client))
-    {
-	/* this should have been diagnosed by whack, so we need not be clear
-	 * !!! overloaded use of RC_CLASH
-	 */
-	loglog(RC_CLASH, "address family inconsistency in this client connection");
-	return FALSE;
-    }
 
     if (subnettypeof(&this->client) != subnettypeof(&that->client))
     {
@@ -1094,11 +1089,19 @@ check_connection_end(const struct whack_end *this, const struct whack_end *that
 	    }
 	}
     }
+
+#if 0
+    /*
+     * Virtual IP is also valid with rightsubnet=vnet:%priv or with
+     * rightprotoport=17/%any
+     */
     if ((this->virt) && (!isanyaddr(&this->host_addr) || this->has_client)) {
 	loglog(RC_CLASH,
 	    "virtual IP must only be used with %%any and without client");
 	return FALSE;
     }
+#endif
+
     return TRUE;	/* happy */
 }
 
@@ -1147,6 +1150,13 @@ add_connection(const struct whack_message *wm)
     {
 	loglog(RC_DUPNAME, "attempt to redefine connection \"%s\"", wm->name);
     }
+#if 0
+    /*
+     * A valid proposal done by others to which we need to respond is something like
+     *  port 80 only, which is assymetric, eg leftprotoport=6/80 rightprotoport=6/%any
+     * So this check is disabled, but it has not been verified this assumption is not
+     *  assumed elsewhere.  -- Paul
+     */
     else if (wm->right.protocol != wm->left.protocol)
     {
 	/* this should haven been diagnosed by whack
@@ -1154,6 +1164,7 @@ add_connection(const struct whack_message *wm)
 	 */
 	loglog(RC_CLASH, "the protocol must be the same for leftport and rightport");
     }
+#endif
     else if(wm->ike != NULL
 	    && ((alg_info_ike = alg_info_ike_create_from_str(wm->ike, &ugh))==NULL
 		|| alg_info_ike->alg_info_cnt==0)) {
@@ -1184,7 +1195,17 @@ add_connection(const struct whack_message *wm)
 	c->name = wm->name;
 	c->connalias = wm->connalias;
 
+#ifdef XAUTH_USEPAM
+	c->pamh = NULL;
+#endif
+
+#ifdef XAUTH
+	c->cisco_dns_info = NULL;
+	c->cisco_domain_info = NULL;
+	c->cisco_banner = NULL;
+#endif
 #ifdef DYNAMICDNS
+	c->dnshostname = NULL;
 	if (wm->dnshostname)
 		c->dnshostname = wm->dnshostname;
 #endif /* DYNAMICDNS */
@@ -1282,12 +1303,34 @@ add_connection(const struct whack_message *wm)
 	c->sa_rekey_fuzz = wm->sa_rekey_fuzz;
 	c->sa_keying_tries = wm->sa_keying_tries;
 
+	if (c->sa_rekey_margin >= c->sa_ipsec_life_seconds) {
+		time_t new_rkm;
+
+		new_rkm = c->sa_ipsec_life_seconds / 2;
+
+		openswan_log("conn: %s, rekeymargin (%lus) > salifetime (%lus); "
+				"reducing rekeymargin to %lu seconds", c->name,
+				c->sa_rekey_margin, c->sa_ipsec_life_seconds,
+				new_rkm);
+
+		c->sa_rekey_margin = new_rkm;
+	}
+
 	/* RFC 3706 DPD */
         c->dpd_delay = wm->dpd_delay;
         c->dpd_timeout = wm->dpd_timeout;
         c->dpd_action = wm->dpd_action;
 
+        /* Cisco interop: remote peer type */
+        c->remotepeertype=wm->remotepeertype;
+
+        /* Network Manager support */
+#ifdef HAVE_NM
+	c->nmconfigured=wm->nmconfigured;
+#endif
+
 	c->metric = wm->metric;
+	c->connmtu = wm->connmtu;
 
 	c->forceencaps = wm->forceencaps;
 
@@ -1362,10 +1405,18 @@ add_connection(const struct whack_message *wm)
 	     * or wildcard ID */
 	    c->kind = CK_TEMPLATE;
 	}
+	else if ((wm->left.virt != NULL) || (wm->right.virt != NULL))
+	{
+	   /* If we have a subnet=vnet: needing instantiation so we can accept multiple subnets from the remote peer */
+	    c->kind = CK_TEMPLATE;
+	}
 	else
 	{
 	    c->kind = CK_PERMANENT;
 	}
+
+
+
 	set_policy_prio(c);	/* must be after kind is set */
 
 #ifdef DEBUG
@@ -1376,7 +1427,11 @@ add_connection(const struct whack_message *wm)
 
 	passert(!(wm->left.virt && wm->right.virt));
 	if (wm->left.virt || wm->right.virt) {
-	    passert(isanyaddr(&c->spd.that.host_addr));
+	    /*
+	     * This now happens with wildcards on non-instantiations,
+	     * such as rightsubnet=vnet:%priv or rightprotoport=17/%any
+	     * passert(isanyaddr(&c->spd.that.host_addr));
+	     */
 	    c->spd.that.virt = create_virtual(c,
 		wm->left.virt ? wm->left.virt : wm->right.virt);
 	    if (c->spd.that.virt)
@@ -1384,12 +1439,6 @@ add_connection(const struct whack_message *wm)
 	}
 
 	unshare_connection_strings(c);
-#ifdef KERNEL_ALG
-	alg_info_addref((struct alg_info *)c->alg_info_esp);
-#endif
-#ifdef IKE_ALG
-	alg_info_addref((struct alg_info *)c->alg_info_ike);
-#endif
 
 	(void)orient(c);
 	connect_to_host_pair(c);
@@ -1402,6 +1451,7 @@ add_connection(const struct whack_message *wm)
 	    (void) format_connection(topo, sizeof(topo), c, &c->spd);
 
 	    DBG_log("%s", topo);
+	);
 
 #if 0
 	    /* Make sure that address families can be correctly inferred
@@ -1418,6 +1468,7 @@ add_connection(const struct whack_message *wm)
 		    == subnettypeof(&c->spd.that.client));
 #endif
 
+	DBG(DBG_CONTROL,
 	    DBG_log("ike_life: %lus; ipsec_life: %lus; rekey_margin: %lus;"
 		" rekey_fuzz: %lu%%; keyingtries: %lu; policy: %s"
 		, (unsigned long) c->sa_ike_life_seconds
@@ -1524,7 +1575,9 @@ instantiate(struct connection *c, const ip_address *him
 	    , const struct id *his_id)
 {
     struct connection *d;
+#ifdef DEBUG
     int wildcards;
+#endif
 
     passert(c->kind == CK_TEMPLATE);
     passert(c->spd.next == NULL);
@@ -1541,14 +1594,6 @@ instantiate(struct connection *c, const ip_address *him
     unshare_ietfAttrList(&d->spd.this.groups);
     unshare_ietfAttrList(&d->spd.that.groups);
 
-
-#ifdef KERNEL_ALG
-    alg_info_addref((struct alg_info *)d->alg_info_esp);
-#endif
-#ifdef IKE_ALG
-    alg_info_addref((struct alg_info *)d->alg_info_ike);
-#endif
-
     d->kind = CK_INSTANCE;
 
     passert(oriented(*d));
@@ -1619,7 +1664,6 @@ oppo_instantiate(struct connection *c
 		 , const ip_address *peer_client)
 {
     struct connection *d = instantiate(c, him, his_id);
-    char instbuf[512];
 
     DBG(DBG_CONTROL,
 	DBG_log("oppo instantiate d=%s from c=%s with c->routing %s, d->routing %s"
@@ -1627,6 +1671,7 @@ oppo_instantiate(struct connection *c
 		, enum_name(&routing_story, c->spd.routing)
 		, enum_name(&routing_story, d->spd.routing)));
     DBG(DBG_CONTROL,
+	char instbuf[512];
 	DBG_log("new oppo instance: %s"
 		, (format_connection(instbuf, sizeof(instbuf), d, &d->spd), instbuf)));
 
@@ -1799,18 +1844,17 @@ find_connection_for_clients(struct spd_route **srp,
     best_sr = NULL;
 
     passert(!isanyaddr(our_client) && !isanyaddr(peer_client));
-#ifdef DEBUG
-    if (DBGP(DBG_CONTROL))
-    {
-	char ocb[ADDRTOT_BUF], pcb[ADDRTOT_BUF];
+
+    DBG(DBG_CONTROL,
+	char ocb[ADDRTOT_BUF];
+	char pcb[ADDRTOT_BUF];
 
 	addrtot(our_client, 0, ocb, sizeof(ocb));
 	addrtot(peer_client, 0, pcb, sizeof(pcb));
 	DBG_log("find_connection: "
 		"looking for policy for connection: %s:%d/%d -> %s:%d/%d"
-		, ocb, transport_proto, our_port, pcb, transport_proto, peer_port);
-    }
-#endif /* DEBUG */
+		, ocb, transport_proto, our_port, pcb, transport_proto, peer_port));
+
 
     for (c = connections; c != NULL; c = c->ac_next)
     {
@@ -1826,18 +1870,17 @@ find_connection_for_clients(struct spd_route **srp,
  	    && (!sr->this.port || our_port == sr->this.port)
  	    && (!sr->that.port || peer_port == sr->that.port))
 	    {
-		char cib[CONN_INST_BUF];
-		char cib2[CONN_INST_BUF];
 
  		policy_prio_t prio = 8 * (c->prio + (c->kind == CK_INSTANCE))
  				   + 2 * (sr->this.port == our_port)
  				   + 2 * (sr->that.port == peer_port)
  				   +     (sr->this.protocol == transport_proto);
 
-#ifdef DEBUG
-		if (DBGP(DBG_CONTROL|DBG_CONTROLMORE))
+		DBG(DBG_CONTROLMORE,
 		{
-		    char c_ocb[SUBNETTOT_BUF], c_pcb[SUBNETTOT_BUF];
+		    char cib[CONN_INST_BUF];
+		    char c_ocb[SUBNETTOT_BUF];
+		    char c_pcb[SUBNETTOT_BUF];
 
 		    subnettot(&c->spd.this.client, 0, c_ocb, sizeof(c_ocb));
 		    subnettot(&c->spd.that.client, 0, c_pcb, sizeof(c_pcb));
@@ -1847,7 +1890,7 @@ find_connection_for_clients(struct spd_route **srp,
 			    , (fmt_conn_instance(c, cib), cib)
 			    , c_ocb, c_pcb, prio);
 		}
-#endif /* DEBUG */
+		);
 
 		if (best == NULL)
 		{
@@ -1857,6 +1900,9 @@ find_connection_for_clients(struct spd_route **srp,
 		}
 
 		DBG(DBG_CONTROLMORE,
+		{
+		    char cib[CONN_INST_BUF];
+		    char cib2[CONN_INST_BUF];
 		    DBG_log("find_connection: comparing best \"%s\"%s [pri:%ld]{%p} (child %s) to \"%s\"%s [pri:%ld]{%p} (child %s)"
 			    , best->name
 			    , (fmt_conn_instance(best, cib), cib)
@@ -1867,7 +1913,9 @@ find_connection_for_clients(struct spd_route **srp,
 			    , (fmt_conn_instance(c, cib2), cib2)
 			    , prio
 			    , c
-			    , (c->policy_next ? c->policy_next->name : "none")));
+			    , (c->policy_next ? c->policy_next->name : "none"));
+		}
+		);
 
 		if (prio > best_prio)
 		{
@@ -2023,6 +2071,7 @@ build_outgoing_opportunistic_connection(struct gw_info *gw
  */
 struct connection *
 route_owner(struct connection *c
+	    , struct spd_route *cur_spd
 	    , struct spd_route **srp
 	    , struct connection **erop
 	    , struct spd_route **esrp)
@@ -2037,11 +2086,19 @@ route_owner(struct connection *c
     passert(oriented(*c));
     best_sr  = NULL;
     best_esr = NULL;
-    best_routing = c->spd.routing;
+    best_routing = cur_spd->routing;
     best_erouting = best_routing;
 
     for (d = connections; d != NULL; d = d->ac_next)
     {
+
+#ifdef KLIPS_MAST
+	/* in mast mode we must also delete the iptables rule */
+	if (kern_interface == USE_MASTKLIPS)
+	    if (compatible_overlapping_connections(c, d))
+		continue;
+#endif
+
 	for (srd = &d->spd; srd; srd = srd->next)
 	{
 	    if (srd->routing == RT_UNROUTED)
@@ -2049,6 +2106,9 @@ route_owner(struct connection *c
 
 	    for (src = &c->spd; src; src=src->next)
 	    {
+		if (src==srd)
+		    continue;
+
 		if (!samesubnet(&src->that.client, &srd->that.client))
 		    continue;
 		if (src->that.protocol != srd->that.protocol)
@@ -2093,9 +2153,9 @@ route_owner(struct connection *c
 	    err_t m = builddiag("route owner of \"%s\"%s %s:"
 		, c->name
 		, (fmt_conn_instance(c, cib), cib)
-		, enum_name(&routing_story, c->spd.routing));
+		, enum_name(&routing_story, cur_spd->routing));
 
-	    if (!routed(best_ro->spd.routing))
+	    if (!routed(best_routing))
 		m = builddiag("%s NULL", m);
 	    else if (best_ro == c)
 		m = builddiag("%s self", m);
@@ -2103,7 +2163,7 @@ route_owner(struct connection *c
 		m = builddiag("%s \"%s\"%s %s", m
 		    , best_ro->name
 		    , (fmt_conn_instance(best_ro, cib), cib)
-		    , enum_name(&routing_story, best_ro->spd.routing));
+		    , enum_name(&routing_story, best_routing));
 
 	    if (erop != NULL)
 	    {
@@ -2145,8 +2205,9 @@ find_host_connection2(const char *func
 		     , const ip_address *him, u_int16_t his_port, lset_t policy)
 {
     struct connection *c;
-    char mebuf[ADDRTOT_BUF], himbuf[ADDRTOT_BUF];
     DBG(DBG_CONTROLMORE,
+        char mebuf[ADDRTOT_BUF];
+        char himbuf[ADDRTOT_BUF];
 	DBG_log("find_host_connection2 called from %s, me=%s:%d him=%s:%d policy=%s", func
 		, (addrtot(me,  0, mebuf,  sizeof(mebuf)),mebuf),   my_port
 		, him ?  (addrtot(him, 0, himbuf, sizeof(himbuf)),himbuf) : "%any"
@@ -2177,7 +2238,7 @@ find_host_connection2(const char *func
     for(; c != NULL && NEVER_NEGOTIATE(c->policy); c = c->hp_next);
 
     DBG(DBG_CONTROLMORE,
-	DBG_log("find_host_connection returns %s", c ? c->name : "empty"));
+	DBG_log("find_host_connection2 returns %s", c ? c->name : "empty"));
     return c;
 }
 
@@ -2546,7 +2607,7 @@ is_virtual_net_used(struct connection *c, const ip_subnet *peer_net, const struc
 			cname="neither";
 		    }
 
-		    openswan_log("overlap is forbidded (%s%s%s agree%s to overlap)"
+		    openswan_log("overlap is forbidden (%s%s%s agree%s to overlap)"
 				 , cname
 				 , cbuf
 				 , doesnot
@@ -3189,7 +3250,16 @@ show_one_connection(struct connection *c)
 	      , c->spd.that.key_from_DNS_on_demand? "+rKOD" : ""
 	      , prio
 	      , ifn);
-    
+   
+    if(c->connmtu > 0 || c->metric > 0) {
+	whack_log(RC_COMMENT
+		  , "\"%s\"%s:   network params: metric:%lu; mtu:%lu; "
+		  , c->name
+		  , instance
+		  , (unsigned long)c->metric 
+		  , (unsigned long)c->connmtu);
+    }
+ 
     /* slightly complicated stuff to avoid extra crap */
     if(c->dpd_timeout > 0 || DBGP(DBG_DPD)) {
 	whack_log(RC_COMMENT
