@@ -1,8 +1,11 @@
 /* 
  * Cryptographic helper function - calculate DH
  * Copyright (C) 2007-2008 Michael C. Richardson <mcr@xelerance.com>
- * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
- * Copyright (C) 2009 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2008 Antony Antony <antony@xelerance.com>
+ * Copyright (C) 2009 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2009-2010 Avesh Agarwal <avagarwa@redhat.com>
+ * Copyright (C) 2009-2010 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2010 Tuomo Soini <tis@foobar.fi>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -891,7 +894,7 @@ calc_skeyids_iv(struct pcr_skeyid_q *skq
    PK11_FreeSymKey(tkey22);
    PK11_FreeSymKey(tkey23);
 
-   DBG(DBG_CRYPT, DBG_log("NSS: Freed symkeys 1-23 (24 does not exists)\n"));
+   DBG(DBG_CRYPT, DBG_log("NSS: Freed symkeys 1-23\n"));
 
    freeanychunk(hmac_opad);
    freeanychunk(hmac_ipad);
@@ -1179,7 +1182,6 @@ calc_skeyseed_v2(struct pcr_skeyid_q *skq
     )
 {
     struct v2prf_stuff vpss;
-    chunk_t gi, gr;
     size_t total_keysize;
     memset(&vpss, 0, sizeof(vpss));
 
@@ -1192,8 +1194,6 @@ calc_skeyseed_v2(struct pcr_skeyid_q *skq
     PK11SymKey *skeyseed_k, *SK_d_k, *SK_ai_k, *SK_ar_k, *SK_ei_k, *SK_er_k, *SK_pi_k, *SK_pr_k;
 #endif
     /* this doesn't take any memory, it's just moving pointers around */
-    setchunk_fromwire(gi,      &skq->gi, skq);
-    setchunk_fromwire(gr,      &skq->gr, skq);
     setchunk_fromwire(vpss.ni, &skq->ni, skq);
     setchunk_fromwire(vpss.nr, &skq->nr, skq);
     setchunk_fromwire(vpss.spii, &skq->icookie, skq);
@@ -1206,7 +1206,7 @@ calc_skeyseed_v2(struct pcr_skeyid_q *skq
 		  , (long unsigned)keysize));
 
 #ifdef HAVE_LIBNSS
-    const struct hash_desc *hasher = crypto_get_hasher(skq->prf_hash);
+    const struct hash_desc *hasher = (struct hash_desc *)ike_alg_ikev2_find(IKE_ALG_HASH, skq->prf_hash, 0);
     passert(hasher);
 
 
@@ -1227,7 +1227,7 @@ calc_skeyseed_v2(struct pcr_skeyid_q *skq
     passert(skeyseed_k);
 
 #else
-    vpss.prf_hasher = crypto_get_hasher(skq->prf_hash);
+    vpss.prf_hasher = (struct hash_desc *)ike_alg_ikev2_find(IKE_ALG_HASH, skq->prf_hash, 0);
     passert(vpss.prf_hasher);
 
     /* generate SKEYSEED from key=(Ni|Nr), hash of shared */
@@ -1270,7 +1270,7 @@ calc_skeyseed_v2(struct pcr_skeyid_q *skq
 	/* SK_p needs PRF hasher*2 key bits */
 	/* SK_e needs keysize*2 key bits */
 	/* SK_a needs hash's key bits size */
-	const struct hash_desc *integ_hasher = crypto_get_hasher(skq->integ_hash);
+	const struct hash_desc *integ_hasher = (struct hash_desc *)ike_alg_ikev2_find(IKE_ALG_INTEG, skq->integ_hash, 0);
 #ifdef HAVE_LIBNSS
        int skd_bytes = hasher->hash_key_size;
        int skp_bytes = hasher->hash_key_size;       
@@ -1285,14 +1285,14 @@ calc_skeyseed_v2(struct pcr_skeyid_q *skq
 	vpss.t.len = 0;
 	total_keysize = skd_bytes + (2*(ska_bytes + ske_bytes + skp_bytes));
 
-	if(DBGP(DBG_CRYPT)) {
+	DBG(DBG_CRYPT,
 	    DBG_log("PRF+ input");
 	    DBG_dump_chunk("Ni", vpss.ni);
 	    DBG_dump_chunk("Nr", vpss.nr);
 	    DBG_dump_chunk("SPIi", vpss.spii);
 	    DBG_dump_chunk("SPIr", vpss.spir);
 	    DBG_log("Total keysize needed %d", (int)total_keysize);
-	}
+	);
 #ifdef HAVE_LIBNSS
 	counter.ptr = &vpss.counter[0];
 	counter.len =1;
