@@ -795,6 +795,22 @@ aggr_inR1_outI2_tail(struct msg_digest *md
     if (!encrypt_message(&md->rbody, st))
 	return STF_INTERNAL_ERROR;	/* ??? we may be partly committed */
 
+    /* It seems as per Cisco implementation, XAUTH and MODECFG 
+    * are not supposed to be performed again during rekey */
+    if(c->newest_isakmp_sa != SOS_NOBODY && 
+	st->st_connection->spd.this.xauth_client && 
+	st->st_connection->remotepeertype == CISCO) {
+    DBG(DBG_CONTROL, DBG_log("Skipping XAUTH for rekey for Cisco Peer compatibility."));
+    st->hidden_variables.st_xauth_client_done = TRUE;
+    st->st_oakley.xauth = 0; 
+
+	if(st->st_connection->spd.this.modecfg_client) {
+	DBG(DBG_CONTROL, DBG_log("Skipping XAUTH for rekey for Cisco Peer compatibility."));
+	st->hidden_variables.st_modecfg_vars_set = TRUE;
+	st->hidden_variables.st_modecfg_started = TRUE;
+	}
+    }
+
     c->newest_isakmp_sa = st->st_serialno;
 
     /* save last IV from phase 1 so it can be restored later so anything 
@@ -883,6 +899,22 @@ aggr_inI2_tail(struct msg_digest *md
 
     /**************** done input ****************/
 
+    /* It seems as per Cisco implementation, XAUTH and MODECFG 
+     * are not supposed to be performed again during rekey */
+    if(c->newest_isakmp_sa != SOS_NOBODY && 
+	st->st_connection->spd.this.xauth_client && 
+	st->st_connection->remotepeertype == CISCO) {
+	   DBG(DBG_CONTROL, DBG_log("Skipping XAUTH for rekey for Cisco Peer compatibility."));
+	   st->hidden_variables.st_xauth_client_done = TRUE;
+	   st->st_oakley.xauth = 0; 
+
+	   if(st->st_connection->spd.this.modecfg_client) {
+		DBG(DBG_CONTROL, DBG_log("Skipping ModeCFG for rekey for Cisco Peer compatibility."));
+		st->hidden_variables.st_modecfg_vars_set = TRUE;
+		st->hidden_variables.st_modecfg_started = TRUE; 
+	   }
+    }
+
     c->newest_isakmp_sa = st->st_serialno;
 
     update_iv(st);	/* Finalize our Phase 1 IV */
@@ -1089,8 +1121,8 @@ aggr_outI1_tail(struct pluto_crypto_req_cont *pcrc
 		    , &oakley_am_sadb[policy_index], st
 		    , TRUE, TRUE, ISAKMP_NEXT_KE))
 	{
-	    return STF_INTERNAL_ERROR;
 	    cur_state = NULL;
+	    return STF_INTERNAL_ERROR;
 	}
 
 	/* save initiator SA for later HASH */
