@@ -2,8 +2,12 @@
  * Copyright (C) 1997 Angelos D. Keromytis.
  * Copyright (C) 1998-2002  D. Hugh Redelmeier.
  * Copyright (C) 2003-2008 Michael C. Richardson <mcr@xelerance.com>
- * Copyright (C) 2003-2009 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2003-2010 Paul Wouters <paul@xelerance.com>
  * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
+ * Copyright (C) 2008 Ilia Sotnikov
+ * Copyright (C) 2009 Seong-hun Lim
+ * Copyright (C) 2008-2009 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2010 Tuomo Soini <tis@foobar.fi>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -710,10 +714,6 @@ main_inI1_outR1(struct msg_digest *md)
     int next;
     int numvidtosend = 1;  /* we always send DPD VID */
 
-#ifdef LEAK_DETECTIVE
-	report_leaks();         /* report memory leaks now, after all free()s */
-#endif
-
 #ifdef NAT_TRAVERSAL
     if (md->quirks.nat_traversal_vid && nat_traversal_enabled) {
 	DBG(DBG_NATT, DBG_log("nat-t detected, sending nat-t VID"));
@@ -816,9 +816,17 @@ main_inI1_outR1(struct msg_digest *md)
 	    /* Create a temporary connection that is a copy of this one.
 	     * His ID isn't declared yet.
 	     */
+	   DBG(DBG_CONTROL, DBG_log("instantiating \"%s\" for initial Main Mode message received on %s:%u"
+		, c->name, ip_str(&md->iface->ip_addr), pluto_port));
 	    c = rw_instantiate(c, &md->sender
 			       , NULL, NULL);
 	}
+     } else {
+	/* we found a non-wildcard conn. double check if it needs instantiation anyway (eg vnet=) */
+	if ((c->kind == CK_TEMPLATE) && c->spd.that.virt) {
+	   DBG(DBG_CONTROL, DBG_log("local endpoint has virt (vnet/vhost) set without wildcards - needs instantiation"));
+	   c = rw_instantiate(c,&md->sender,NULL,NULL);
+	}
     }
 
 #ifdef XAUTH
@@ -1323,7 +1331,6 @@ main_inI2_outR2_tail(struct pluto_crypto_req_cont *pcrc
     struct ke_continuation *ke = (struct ke_continuation *)pcrc;
     struct msg_digest *md = ke->md;
     struct state *st = md->st;
-    int next_payload;
 
     /* send CR if auth is RSA and no preloaded RSA public key exists*/
     bool send_cr = FALSE;
@@ -1347,7 +1354,9 @@ main_inI2_outR2_tail(struct pluto_crypto_req_cont *pcrc
 	}
 
 #ifdef DEBUG
+ {
     /* Nr out */
+    int next_payload;
     next_payload = ISAKMP_NEXT_NONE;
 
     if(cur_debugging & IMPAIR_BUST_MR2)
@@ -1376,6 +1385,7 @@ main_inI2_outR2_tail(struct pluto_crypto_req_cont *pcrc
 	    return STF_INTERNAL_ERROR;
 	close_output_pbs(&vid_pbs);
     }
+ }
 #else
     /* Nr out */
     if (!ship_nonce(&st->st_nr, r
@@ -2423,7 +2433,7 @@ send_notification(struct state *sndst, u_int16_t type, struct state *encst,
     pb_stream pbs, r_hdr_pbs;
     u_char *r_hashval, *r_hash_start;
     static time_t last_malformed;
-    time_t n = time((time_t)NULL);
+    time_t n = time(NULL);
     struct isakmp_hdr hdr;           /* keep it around for TPM */
 
     r_hashval = NULL;
