@@ -701,7 +701,8 @@ init_phase2_iv(struct state *st, const msgid_t *msgid)
 
 static stf_status
 quick_outI1_tail(struct pluto_crypto_req_cont *pcrc
-		 , struct pluto_crypto_req *r);
+		 , struct pluto_crypto_req *r
+		 , struct state *st);
 
 static void
 quick_outI1_continue(struct pluto_crypto_req_cont *pcrc
@@ -709,7 +710,7 @@ quick_outI1_continue(struct pluto_crypto_req_cont *pcrc
 		     , err_t ugh)
 {
     struct qke_continuation *qke = (struct qke_continuation *)pcrc;
-    struct state *const st = qke->st;
+    struct state *const st = state_with_serialno(qke->qke_pcrc.pcrc_serialno);
     stf_status e;
 
     DBG(DBG_CONTROLMORE
@@ -732,7 +733,9 @@ quick_outI1_continue(struct pluto_crypto_req_cont *pcrc
 
     set_cur_state(st);	/* we must reset before exit */
     set_suspended(st, NULL);
-    e = quick_outI1_tail(pcrc, r);
+    e = quick_outI1_tail(pcrc, r, st);
+    if (e == STF_INTERNAL_ERROR)
+	loglog(RC_LOG_SERIOUS, "%s: quick_outI1_tail() failed with STF_INTERNAL_ERROR", __FUNCTION__);
 
     reset_globals();
 }
@@ -815,8 +818,6 @@ quick_outI1(int whack_sock
 		     , isakmp_sa->st_serialno, st->st_msgid, p2alg, pfsgroupname);
     }
 
-    qke->st = st;
-    qke->isakmp_sa = isakmp_sa;
     qke->replacing = replacing;
     pcrc_init(&qke->qke_pcrc);
     qke->qke_pcrc.pcrc_func = quick_outI1_continue;
@@ -834,12 +835,12 @@ quick_outI1(int whack_sock
     
 static stf_status
 quick_outI1_tail(struct pluto_crypto_req_cont *pcrc
-		 , struct pluto_crypto_req *r)
+		 , struct pluto_crypto_req *r
+		 , struct state *st)
 {
     struct qke_continuation *qke = (struct qke_continuation *)pcrc;
-    struct state *st = qke->st;
+    struct state *isakmp_sa = state_with_serialno(st->st_clonedfrom);
     struct connection *c = st->st_connection;
-    struct state *isakmp_sa = qke->isakmp_sa;
     pb_stream rbody;
     u_char	/* set by START_HASH_PAYLOAD: */
 	*r_hashval,	/* where in reply to jam hash value */
@@ -848,7 +849,11 @@ quick_outI1_tail(struct pluto_crypto_req_cont *pcrc
 		      c->spd.this.protocol || c->spd.that.protocol ||
 		      c->spd.this.port || c->spd.that.port;
 
-    st->st_connection = c;
+    if(isakmp_sa == NULL) {
+	/* phase1 state got deleted while cryptohelper was working */
+	loglog(RC_LOG_SERIOUS,"phase2 initiation failed because parent ISAKMP #%lu is gone", st->st_clonedfrom);
+	return STF_FATAL;
+    }
 
 #ifdef NAT_TRAVERSAL
     if (isakmp_sa->hidden_variables.st_nat_traversal & NAT_T_DETECTED) {
@@ -1186,6 +1191,21 @@ quick_inI1_outR1(struct msg_digest *md)
 	}
 	/* End Hack for MS 818043 NAT-T Update */
 
+#ifdef NAT_TRAVERSAL
+	/* Hack for MacOS/iPhone see bug #1204 */
+	if ((p1st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) &&
+	    (p1st->hidden_variables.st_nat_traversal & LELEM(NAT_TRAVERSAL_OSX))){
+		struct hidden_variables hv; 
+		hv = p1st->hidden_variables; 
+		nat_traversal_natoa_lookup(md, &hv); 
+		if(isanyaddr(&hv.st_nat_oa)){
+		   happy(addrtosubnet(&p1st->st_remoteaddr, &b.his.net));
+		   loglog(RC_LOG_SERIOUS, "Applying workaround for Mac OS X NAT-OA bug, ignoring proposed subnet");
+		}
+	}
+	/* End Hack for MacOS/iPhone */
+#endif
+
 	b.his.proto = id_pd->payload.ipsec_id.isaiid_protoid;
 	b.his.port = id_pd->payload.ipsec_id.isaiid_port;
 	b.his.net.addr.u.v4.sin_port = htons(b.his.port);
@@ -1224,11 +1244,12 @@ quick_inI1_outR1(struct msg_digest *md)
 	    hv = p1st->hidden_variables; 
 	    nat_traversal_natoa_lookup(md, &hv); 
 	    
-	    addrtosubnet(&hv.st_nat_oa,&b.his.net);
-	    subnettot(&b.his.net, 0, subnet_buf, sizeof(subnet_buf));
-
-	    loglog(RC_LOG_SERIOUS, "IDci was FQDN: %s, using NAT_OA=%s as IDci"
-		   , idfqdn, subnet_buf);
+	    if (!isanyaddr(&hv.st_nat_oa)){
+	    	addrtosubnet(&hv.st_nat_oa,&b.his.net);
+	    	subnettot(&b.his.net, 0, subnet_buf, sizeof(subnet_buf));
+	    	loglog(RC_LOG_SERIOUS, "IDci was FQDN: %s, using NAT_OA=%s %d as IDci"
+			   , idfqdn, subnet_buf,isanyaddr(&hv.st_nat_oa));
+	    }
 	}
 #endif
     }
@@ -1968,8 +1989,6 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 	    ci = pcim_ongoing_crypto;
 	    if(ci < st->st_import) ci = st->st_import;
 
-	    qke->st = st;
-	    qke->isakmp_sa = p1st;
 	    qke->md = md;
 	    pcrc_init(&qke->qke_pcrc);
 	    qke->qke_pcrc.pcrc_func = quick_inI1_outR1_cryptocontinue1;
@@ -1994,7 +2013,7 @@ quick_inI1_outR1_cryptocontinue1(struct pluto_crypto_req_cont *pcrc
 {
     struct qke_continuation *qke = (struct qke_continuation *)pcrc;
     struct msg_digest *md = qke->md;
-    struct state *const st = qke->st;
+    struct state *const st = state_with_serialno(qke->qke_pcrc.pcrc_serialno);
     stf_status e;
 
     DBG(DBG_CONTROLMORE
@@ -2037,8 +2056,12 @@ quick_inI1_outR1_cryptocontinue1(struct pluto_crypto_req_cont *pcrc
 			    , st->st_import
 			    , RESPONDER
 			    , st->st_pfs_group->group);
-	
-	if(e != STF_SUSPEND) {
+
+	/* In the STF_INLINE, quick_inI1_outR1_cryptocontinue1 has already
+	 * called complete_v1_state_transition and it has freed *dh. It
+	 * called quick_inI1_outR1_cryptocontinue2 which did the release_md too.
+	 */
+	if(e != STF_SUSPEND && e != STF_INLINE) {
 	    if(dh->md != NULL) {
 		complete_v1_state_transition(&qke->md, e);
 		if(dh->md) release_md(qke->md);
@@ -2117,7 +2140,6 @@ quick_inI1_outR1_cryptotail(struct dh_continuation *dh
 {
     struct msg_digest *md = dh->md;
     struct state *st = md->st;
-    struct connection *c = st->st_connection;
     struct payload_digest *const id_pd = md->chain[ISAKMP_NEXT_ID];
     struct payload_digest *const sapd = md->chain[ISAKMP_NEXT_SA];
     struct isakmp_sa sa = sapd->payload.sa;
@@ -2143,7 +2165,6 @@ quick_inI1_outR1_cryptotail(struct dh_continuation *dh
     /* HASH(2) out -- first pass */
     START_HASH_PAYLOAD(md->rbody, ISAKMP_NEXT_SA);
 
-    passert(st->st_connection == c);
     passert(st->st_connection != NULL);
 
     /* sa header is unchanged -- except for np */
@@ -2495,7 +2516,8 @@ quick_inR1_outI2_cryptotail(struct dh_continuation *dh
     {
 	u_char	/* set by START_HASH_PAYLOAD: */
 	    *r_hashval,	/* where in reply to jam hash value */
-	    *r_hash_start;	/* start of what is to be hashed */
+	    *r_hash_start;      /* start of what is to be hashed */
+
 
 #ifdef IMPAIR_UNALIGNED_I2_MSG
 	{
