@@ -1,6 +1,11 @@
 /* demultiplex incoming IKE messages
  * Copyright (C) 1997 Angelos D. Keromytis.
- * Copyright (C) 1998-2002  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2010  D. Hugh Redelmeier.
+ * Copyright (C) 2007-2008 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2009 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2008-2010 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2010 Simon Deziel <simon@xelerance.com>
+ * Copyright (C) 2010 Tuomo Soini <tis@foobar.fi>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -301,6 +306,9 @@ process_v2_packet(struct msg_digest **mdp)
     if(md->hdr.isa_flags & ISAKMP_FLAGS_I) {
 	/* then I am the responder */
 	rcookiezero = is_zero_cookie(md->hdr.isa_rcookie);
+	if (!rcookiezero) {
+		openswan_log("received packet that claimed to be (I)nitiator, but rcookie is not zero?");
+	}
 
 	md->role = RESPONDER;
 
@@ -703,7 +711,6 @@ static void success_v2_state_transition(struct msg_digest **mdp)
     /* if requested, send the new reply packet */
     if (svm->flags & SMF2_REPLY)
     {
-	char buf[ADDRTOT_BUF];
 
 	/* free previously transmitted packet */
 	freeanychunk(st->st_tpacket);
@@ -713,8 +720,9 @@ static void success_v2_state_transition(struct msg_digest **mdp)
 	    nat_traversal_change_port_lookup(md, st);
 	}
 #endif
-	DBG(DBG_CONTROL
-	    , DBG_log("sending reply packet to %s:%u (from port %u)"
+	DBG(DBG_CONTROL,
+	    char buf[ADDRTOT_BUF];
+	    DBG_log("sending reply packet to %s:%u (from port %u)"
 		      , (addrtot(&st->st_remoteaddr
 				 , 0, buf, sizeof(buf)), buf)
 		      , st->st_remoteport
@@ -757,7 +765,6 @@ static void success_v2_state_transition(struct msg_digest **mdp)
     {
 	time_t delay;
 	enum event_type kind = svm->timeout_event;
-	bool agreed_time = FALSE;
 	struct connection *c = st->st_connection;
 
 	switch (kind)
@@ -804,13 +811,6 @@ static void success_v2_state_transition(struct msg_digest **mdp)
 	     * Note: for ISAKMP SA, we let the negotiated
 	     * time stand (implemented by earlier logic).
 	     */
-	    if (agreed_time
-		&& (c->policy & POLICY_DONT_REKEY))
-	    {
-		kind = (svm->flags & SMF2_INITIATOR)
-		    ? EVENT_SA_REPLACE_IF_USED
-		    : EVENT_SA_EXPIRE;
-	    }
 	    if (kind != EVENT_SA_EXPIRE)
 	    {
 		unsigned long marg = c->sa_rekey_margin;
