@@ -1,7 +1,8 @@
 /* IKEv2 - CHILD SA - calculations
  *
- * Copyright (C) 2007 Michael Richardson <mcr@xelerance.com>
- * Copyright (C) 2008-2009 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2007-2008 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2009-2010 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2010 Tuomo Soini <tis@foobar.fi>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -96,6 +97,26 @@ struct traffic_selector ikev2_subnettots(struct end *e)
 	break;
     }
 
+    /* 
+     * The IKEv2 code used to send 0-65535 as port regardless of
+     * the local policy specified. if local policy states a specific 
+     * protocol and port, then send that protocol value and port to 
+     * other end  -- Avesh
+     * Paul: TODO: I believe IKEv2 allows multiple port ranges?
+     */
+
+    ts.ipprotoid = e->protocol;
+
+    /*if port is %any or 0*/
+    if(e->port == 0 || e->has_port_wildcard) {
+	/* Paul: TODO 0 might have to mean "any single 1 port" - check the IKEv2 RFC */
+	ts.startport = 0;
+	ts.endport = 65535;
+    } else {
+	ts.startport = e->port;
+	ts.endport = e->port;
+    }
+	
     return ts;
 }
 
@@ -127,9 +148,18 @@ stf_status ikev2_emit_ts(struct msg_digest *md   UNUSED
 	its1.isat1_sellen = 40;
 	break;
     }
-    its1.isat1_ipprotoid = 0;      /* all protocols */
-    its1.isat1_startport = 0;      /* all ports */
-    its1.isat1_endport = 65535;  
+
+    /* 
+     * The IKEv2 code used to send 0-65535 as port regardless of
+     * the local policy specified. if local policy states a specific 
+     * protocol and port, then send that protocol value and port to 
+     * other end  -- Avesh
+     * Paul: TODO: I believe IKEv2 allows multiple port ranges?
+     */
+
+    its1.isat1_ipprotoid = ts->ipprotoid;      /* protocol as per local policy*/
+    its1.isat1_startport = htons(ts->startport);      /* ports as per local policy*/
+    its1.isat1_endport = htons(ts->endport);  
     if(!out_struct(&its1, &ikev2_ts1_desc, &ts_pbs, &ts_pbs2))
 	return STF_INTERNAL_ERROR;
     
@@ -185,12 +215,17 @@ stf_status ikev2_calc_emit_ts(struct msg_digest *md
 			    , ts_r, RESPONDER);
 	}
 	else {
-		if ( md->chain[ISAKMP_NEXT_v2N] && (md->chain[ISAKMP_NEXT_v2N]->payload.v2n.isan_type == USE_TRANSPORT_MODE) ) {
+		struct payload_digest *p;
+		for(p = md->chain[ISAKMP_NEXT_v2N]; p != NULL; p = p->next)
+		{
+			if ( p->payload.v2n.isan_type == USE_TRANSPORT_MODE ) {
 			DBG_log("Received USE_TRANSPORT_MODE from the other end, next payload is USE_TRANSPORT_MODE notification");
 			ret = ikev2_emit_ts(md, outpbs, ISAKMP_NEXT_v2N
 						, ts_r, RESPONDER);
+			break;
+			}
 		}
-		else {
+		if(!p){
                         ret = ikev2_emit_ts(md, outpbs, ISAKMP_NEXT_NONE
                                                 , ts_r, RESPONDER);
 		}
@@ -262,8 +297,9 @@ ikev2_parse_ts(struct payload_digest *const ts_pd
 	    }
 
 	    array[i].ipprotoid = ts1.isat1_ipprotoid;
-	    array[i].startport = ts1.isat1_startport;
-	    array[i].endport   = ts1.isat1_startport;
+	    /*should be converted to host byte order for local processing*/
+	    array[i].startport = ntohs(ts1.isat1_startport);
+	    array[i].endport   = ntohs(ts1.isat1_endport);
 	}
     }
     
@@ -281,10 +317,11 @@ static int ikev2_evaluate_connection_fit(struct connection *d
 {
     unsigned int tsi_ni, tsr_ni;
     int bestfit = -1;
+/* not used? */
+#if 0
     int best_tsr, best_tsi; 
+#endif
     struct end *ei, *er;
-#ifdef DEBUG
-    char ei3[SUBNETTOT_BUF],er3[SUBNETTOT_BUF];
     
     if(role == INITIATOR) {
 	ei = &sr->this;
@@ -294,8 +331,10 @@ static int ikev2_evaluate_connection_fit(struct connection *d
 	er = &sr->this;
     }
 	
-    if (DBGP(DBG_CONTROLMORE))
+    DBG(DBG_CONTROLMORE,
     {
+	char ei3[SUBNETTOT_BUF];
+	char er3[SUBNETTOT_BUF];
 	subnettot(&ei->client,  0, ei3, sizeof(ei3));
 	subnettot(&er->client,  0, er3, sizeof(er3));
 	DBG_log("  ikev2_eval_conn evaluating "
@@ -304,17 +343,19 @@ static int ikev2_evaluate_connection_fit(struct connection *d
 		, er3, er->protocol, er->port
 		, is_virtual_connection(d) ? "(virt)" : "");
     }
-#endif /* DEBUG */
+    );
    
     /* compare tsi/r array to this/that, evaluating how well it fits */
     for(tsi_ni = 0; tsi_ni < tsi_n; tsi_ni++) {
 	for(tsr_ni=0; tsr_ni<tsr_n; tsr_ni++) {
 	    /* does it fit at all? */
 
-	    if (DBGP(DBG_CONTROLMORE))
+	    DBG(DBG_CONTROLMORE,
 	    {
-		char lbi[ADDRTOT_BUF], hbi[ADDRTOT_BUF];
-		char lbr[ADDRTOT_BUF], hbr[ADDRTOT_BUF];
+		char lbi[ADDRTOT_BUF];
+		char hbi[ADDRTOT_BUF];
+		char lbr[ADDRTOT_BUF];
+		char hbr[ADDRTOT_BUF];
 		addrtot(&tsi[tsi_ni].low,  0, lbi, sizeof(lbi));
 		addrtot(&tsi[tsi_ni].high, 0, hbi, sizeof(hbi));
 		addrtot(&tsr[tsr_ni].low,  0, lbr, sizeof(lbr));
@@ -324,7 +365,7 @@ static int ikev2_evaluate_connection_fit(struct connection *d
 			, tsi_ni, lbi, hbi
 			, tsr_ni, lbr, hbr);
 	    }
-
+	    );
 	    /* do addresses fit into the policy? */
 	    if(addrinsubnet(&tsi[tsi_ni].low, &ei->client)
 	       && addrinsubnet(&tsi[tsi_ni].high, &ei->client)
@@ -346,16 +387,20 @@ static int ikev2_evaluate_connection_fit(struct connection *d
 		int fitbits2  = maskbits2 + ts_range2;
 		int fitbits = (fitbits1 << 8) + fitbits2;
 
-		if (DBGP(DBG_CONTROLMORE))
+		DBG(DBG_CONTROLMORE,
 		{
 		    DBG_log("      has ts_range1=%u maskbits1=%u ts_range2=%u maskbits2=%u fitbits=%d <> %d"
 			    , ts_range1, maskbits1, ts_range2, maskbits2
 			    , fitbits, bestfit);
 		}
+		);
 
 		if(fitbits > bestfit) {
+/* not used ? */
+#if 0
 		    best_tsi = tsi_ni;
 		    best_tsr = tsr_ni;
+#endif
 		    bestfit = fitbits;
 		}
 	    }
@@ -417,7 +462,10 @@ stf_status ikev2_child_sa_respond(struct msg_digest *md
      * similar to find_client_connection/fc_try.
      */
     {
+/* b is not used? */
+#if 0
 	struct connection *b = c;
+#endif
 	struct connection *d;
 	int bestfit, newfit;
 	struct spd_route *sra, *bsr;
@@ -431,7 +479,10 @@ stf_status ikev2_child_sa_respond(struct msg_digest *md
 						   ,tsi,tsr,tsi_n,tsr_n);
 	    if(bfit > bestfit) {
 		bestfit = bfit;
+/* b is not used ? */
+#if 0
 		b = c;
+#endif
 		bsr = sra;
 	    }
 	}
@@ -478,7 +529,10 @@ stf_status ikev2_child_sa_respond(struct msg_digest *md
 							 ,tsi,tsr,tsi_n,tsr_n);
 		    if(newfit > bestfit) {
 			bestfit = newfit;
+/* not used? */
+#if 0
 			b=d;
+#endif
 			bsr = sr;
 		    }
 		}
@@ -490,36 +544,44 @@ stf_status ikev2_child_sa_respond(struct msg_digest *md
 	 * the state structure as the tsi/tsr
 	 *
 	 */
-	st1->st_ts_this = ikev2_subnettots(&bsr->this);
-	st1->st_ts_that = ikev2_subnettots(&bsr->that);
+	/* Paul: should we STF_FAIL here instead of checking for NULL */
+	if (bsr != NULL) {
+		st1->st_ts_this = ikev2_subnettots(&bsr->this);
+		st1->st_ts_that = ikev2_subnettots(&bsr->that);
+	}
     }
     ret = ikev2_calc_emit_ts(md, outpbs, role
 			     , c, c->policy);
     if(ret != STF_OK) return ret;
 
     if( role == RESPONDER ) {
-	if ( md->chain[ISAKMP_NEXT_v2N] && (md->chain[ISAKMP_NEXT_v2N]->payload.v2n.isan_type == USE_TRANSPORT_MODE) ) {
+	chunk_t child_spi, notifiy_data;
+	struct payload_digest *p;
+	for(p = md->chain[ISAKMP_NEXT_v2N]; p != NULL; p = p->next)
+	{
+	   if ( p->payload.v2n.isan_type == USE_TRANSPORT_MODE ) {
 
-	if(st1->st_connection->policy & POLICY_TUNNEL) {
+	   if(st1->st_connection->policy & POLICY_TUNNEL) {
 		DBG_log("Although local policy is tunnel, received USE_TRANSPORT_MODE");
 		DBG_log("So switching to transport mode, and responding with USE_TRANSPORT_MODE notify");
-	}
-	else {
-                DBG_log("Local policy is transport, received USE_TRANSPORT_MODE");
+	   }
+	   else {
+		DBG_log("Local policy is transport, received USE_TRANSPORT_MODE");
 		DBG_log("Now responding with USE_TRANSPORT_MODE notify");
-	}
+	   }
 
-	chunk_t child_spi, notifiy_data;
-	memset(&child_spi, 0, sizeof(child_spi));
-	memset(&notifiy_data, 0, sizeof(notifiy_data));
-	ship_v2N (ISAKMP_NEXT_NONE, ISAKMP_PAYLOAD_NONCRITICAL, /*PROTO_ISAKMP*/ 0,
+	   memset(&child_spi, 0, sizeof(child_spi));
+	   memset(&notifiy_data, 0, sizeof(notifiy_data));
+	   ship_v2N (ISAKMP_NEXT_NONE, ISAKMP_PAYLOAD_NONCRITICAL, /*PROTO_ISAKMP*/ 0,
 			&child_spi,
 			USE_TRANSPORT_MODE, &notifiy_data, outpbs);
 
-		if (st1->st_esp.present == TRUE) {
+	   if (st1->st_esp.present == TRUE) {
 		/*openswan supports only "esp" with ikev2 it seems, look at ikev2_parse_child_sa_body handling*/
 		st1->st_esp.attrs.encapsulation = ENCAPSULATION_MODE_TRANSPORT;
-                }
+	   }
+	   break;
+	   }
 	}
     }
 
