@@ -1,8 +1,11 @@
 /* 
  * IKEv2 parent SA creation routines
- * Copyright (C) 2007  Michael Richardson <mcr@xelerance.com>
- * Copyright (C) 2008  Paul Wouters <paul@xelerance.com>
- * Copyright (C) 2008  Antony Antony <antony@xelerance.com>
+ * Copyright (C) 2007-2008 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2008-2010 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2008 Antony Antony <antony@xelerance.com>
+ * Copyright (C) 2008-2009 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2010 Avesh Agarwal <avagarwa@redhat.com>
+ * Copyright (C) 2010 Tuomo Soini <tis@foobar.fi
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -334,7 +337,10 @@ ikev2_parent_outI1_common(struct msg_digest *md
 			  , struct state *st)
 {
     /* struct connection *c = st->st_connection; */
-    int numvidtosend = 1;  /* we always send Openswan VID */
+    int numvidtosend = 0; 
+#ifdef PLUTO_SENDS_VENDORID
+    numvidtosend++;  /* we send Openswan VID */
+#endif
 
     /* set up reply */
     init_pbs(&reply_stream, reply_buffer, sizeof(reply_buffer), "reply packet");
@@ -422,7 +428,8 @@ ikev2_parent_outI1_common(struct msg_digest *md
 	close_output_pbs(&pb);
     }
 
-    /* Send DPD VID */
+#ifdef PLUTO_SENDS_VENDORID
+    /* Send VendorID VID */
     {
 	int np = --numvidtosend > 0 ? ISAKMP_NEXT_v2V : ISAKMP_NEXT_NONE;
 
@@ -430,6 +437,7 @@ ikev2_parent_outI1_common(struct msg_digest *md
 			     , pluto_vendorid, strlen(pluto_vendorid), "Vendor ID"))
 	    return STF_INTERNAL_ERROR;
     }
+#endif
 
     close_message(&md->rbody);
     close_output_pbs(&reply_stream);
@@ -552,13 +560,14 @@ stf_status ikev2parent_inI1outR1(struct msg_digest *md)
        	if ( md->chain[ISAKMP_NEXT_v2KE] &&   md->chain[ISAKMP_NEXT_v2N] &&
        	     (md->chain[ISAKMP_NEXT_v2N]->payload.v2n.isan_type == COOKIE))
 		{
+			u_int8_t spisize;
+	        const pb_stream *dc_pbs;
+			chunk_t blob; 
 			DBG(DBG_CONTROLMORE
 	        	, DBG_log("received a DOS cookie in I1 verify it"));
 	       	/* we received dcookie we send earlier verify it */
-			u_int8_t spisize 
-					= md->chain[ISAKMP_NEXT_v2N]->payload.v2n.isan_spisize;
-	        const pb_stream *dc_pbs = &md->chain[ISAKMP_NEXT_v2N]->pbs;
-			chunk_t blob; 
+			spisize = md->chain[ISAKMP_NEXT_v2N]->payload.v2n.isan_spisize;
+	        dc_pbs = &md->chain[ISAKMP_NEXT_v2N]->pbs;
 			blob.ptr = dc_pbs->cur + spisize;
 			blob.len = pbs_left(dc_pbs) - spisize;
 			DBG(DBG_CONTROLMORE
@@ -692,7 +701,10 @@ ikev2_parent_inI1outR1_tail(struct pluto_crypto_req_cont *pcrc
     struct payload_digest *const sa_pd = md->chain[ISAKMP_NEXT_v2SA];
     struct state *const st = md->st;
     pb_stream *keyex_pbs;
-    int    numvidtosend=1;
+    int    numvidtosend = 0;
+#ifdef PLUTO_SENDS_VENDORID
+    numvidtosend++;  /* we send Openswan VID */
+#endif
 
     /* note that we don't update the state here yet */
     memcpy(st->st_icookie, md->hdr.isa_icookie, COOKIE_SIZE);
@@ -765,7 +777,8 @@ ikev2_parent_inI1outR1_tail(struct pluto_crypto_req_cont *pcrc
 	close_output_pbs(&pb);
     }
 
-    /* Send DPD VID */
+#ifdef PLUTO_SENDS_VENDORID
+    /* Send VendorID VID */
     {
 	int np = --numvidtosend > 0 ? ISAKMP_NEXT_v2V : ISAKMP_NEXT_NONE;
 
@@ -773,6 +786,7 @@ ikev2_parent_inI1outR1_tail(struct pluto_crypto_req_cont *pcrc
 			     , pluto_vendorid, strlen(pluto_vendorid), "Vendor ID"))
 	    return STF_INTERNAL_ERROR;
     }
+#endif
 
     close_message(&md->rbody);
     close_output_pbs(&reply_stream);
@@ -823,11 +837,13 @@ stf_status ikev2parent_inR1outI2(struct msg_digest *md)
     if( md->chain[ISAKMP_NEXT_v2N]
 		&& md->chain[ISAKMP_NEXT_v2N]->payload.v2n.isan_type ==  COOKIE)
     {
+		u_int8_t spisize;
+	    const pb_stream *dc_pbs;
 		DBG(DBG_CONTROLMORE 
 			,DBG_log("inR1OutI2 received a DOS COOKIE from the responder");
     	    DBG_log("resend the I1 with a cookie payload"));
-		u_int8_t spisize = md->chain[ISAKMP_NEXT_v2N]->payload.v2n.isan_spisize;
-	    const pb_stream *dc_pbs = &md->chain[ISAKMP_NEXT_v2N]->pbs;
+		spisize = md->chain[ISAKMP_NEXT_v2N]->payload.v2n.isan_spisize;
+	    dc_pbs = &md->chain[ISAKMP_NEXT_v2N]->pbs;
     	clonetochunk(st->st_dcookie,  (dc_pbs->cur + spisize)
 		 , (pbs_left(dc_pbs) - spisize), "saved received dcookie");
 
@@ -1064,11 +1080,11 @@ static stf_status ikev2_encrypt_msg(struct msg_digest *md,
 	hmac_init_chunk(&ctx, pst->st_oakley.integ_hasher, *authkey);
 	hmac_update(&ctx, authstart, authloc-authstart);
 	hmac_final(authloc, &ctx);
-	
-	if(DBGP(DBG_PARSING)) {
+
+	DBG(DBG_PARSING,
 	    DBG_dump("data being hmac:", authstart, authloc-authstart);
 	    DBG_dump("out calculated auth:", authloc, pst->st_oakley.integ_hasher->hash_integ_len); 
-	}
+           );
     }
     
     return STF_OK;
@@ -1104,7 +1120,7 @@ stf_status ikev2_decrypt_msg(struct msg_digest *md
 
     authstart=md->packet_pbs.start;
     iv     = e_pbs->cur;
-    encend = e_pbs->roof - 12;
+    encend = e_pbs->roof - pst->st_oakley.integ_hasher->hash_integ_len;
     
     /* start by checking authenticator */
     {
@@ -1115,13 +1131,14 @@ stf_status ikev2_decrypt_msg(struct msg_digest *md
 	hmac_update(&ctx, authstart, encend-authstart);
 	hmac_final(b12, &ctx);
 	
-	if(DBGP(DBG_PARSING)) {
+	DBG(DBG_PARSING,
 	    DBG_dump("data being hmac:", authstart, encend-authstart);
 	    DBG_dump("R2 calculated auth:", b12, pst->st_oakley.integ_hasher->hash_integ_len); 
 	    DBG_dump("R2  provided  auth:", encend, pst->st_oakley.integ_hasher->hash_integ_len);
-	}
+	);
 	
 	/* compare first 96 bits == 12 bytes */
+	/* It is not always 96 bytes, it depends upon which integ algo is used*/
 	if(memcmp(b12, encend, pst->st_oakley.integ_hasher->hash_integ_len)!=0) {
 	    openswan_log("R2 failed to match authenticator");
 	    return STF_FAIL;
@@ -1155,10 +1172,10 @@ stf_status ikev2_decrypt_msg(struct msg_digest *md
 	    return STF_FAIL;
 	}
 	
-	if(DBGP(DBG_CRYPT)) {
+	DBG(DBG_CRYPT,
 	    DBG_dump("decrypted payload:", encstart, enclen);
 	    DBG_log("striping %u bytes as pad", padlen+1);
-	}
+	);
 	
 	init_pbs(&md->clr_pbs, encstart, enclen - (padlen+1), "cleartext");
     }
@@ -1378,6 +1395,7 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
 	 * SA2i, TSi and TSr and (USE_TRANSPORT_MODE notification in transport mode) for it .
 	 */
 	if(c0) {
+		chunk_t child_spi, notifiy_data;
 	    st->st_connection = c0;
 
 	    ikev2_emit_ipsec_sa(md,&e_pbs_cipher,ISAKMP_NEXT_v2TSi,c0, policy);
@@ -1389,7 +1407,6 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
 
 	    if( !(st->st_connection->policy & POLICY_TUNNEL) ) {
 		DBG_log("Initiator child policy is transport mode, sending USE_TRANSPORT_MODE");
-		chunk_t child_spi, notifiy_data;
 		memset(&child_spi, 0, sizeof(child_spi));
 		memset(&notifiy_data, 0, sizeof(notifiy_data));
 		ship_v2N (ISAKMP_NEXT_NONE, ISAKMP_PAYLOAD_NONCRITICAL, 0,
@@ -1481,12 +1498,6 @@ stf_status ikev2parent_inI2outR2(struct msg_digest *md)
 	return STF_FATAL;
     }
 
-    /* process AUTH payload */
-    if(!md->chain[ISAKMP_NEXT_v2AUTH]) {
-	openswan_log("no authentication payload found");
-	return STF_FATAL;
-    }
-
     /* now. we need to go calculate the g^xy */
     {
 	struct dh_continuation *dh = alloc_thing(struct dh_continuation
@@ -1580,6 +1591,15 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	if(ret != STF_OK) return ret;
     }
 
+
+    /*Once the message has been decrypted, then only we can check for auth payload*/
+    /*check the presense of auth payload now so that it does not crash in rehash_state if auth payload has not been received*/
+    if(!md->chain[ISAKMP_NEXT_v2AUTH]) {
+        openswan_log("no authentication payload found");
+        return STF_FAIL;
+    }
+
+
     /* if it decrypted okay, then things are good, this packet is
      * well received, and we should change state.
      */
@@ -1626,6 +1646,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	    ikev2_decode_cr(md, &st->st_connection->requested_ca);
     }
 
+    /* process AUTH payload now */
     /* now check signature from RSA key */
     switch(md->chain[ISAKMP_NEXT_v2AUTH]->payload.v2a.isaa_type)
     {
@@ -1640,7 +1661,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	if(authstat != STF_OK) {
 	    openswan_log("RSA authentication failed");
 	    SEND_NOTIFICATION(AUTHENTICATION_FAILED);
-	    return STF_FAIL;
+	    return STF_FATAL;
 	}
 	break;
     }
@@ -1653,7 +1674,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	if(authstat != STF_OK) {
 	    openswan_log("PSK authentication failed AUTH mismatch!");
 	    SEND_NOTIFICATION(AUTHENTICATION_FAILED);
-	    return STF_FAIL;
+	    return STF_FATAL;
 	}
 	break;
     }
@@ -1661,7 +1682,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	openswan_log("authentication method: %s not supported"
 		     , enum_name(&ikev2_auth_names
 				 ,md->chain[ISAKMP_NEXT_v2AUTH]->payload.v2a.isaa_type));
-	return STF_FAIL;
+	return STF_FATAL;
     }
 
     /* good. now create child state */
@@ -1992,8 +2013,11 @@ stf_status ikev2parent_inR2(struct msg_digest *md)
     }
 
     {
-	if ( md->chain[ISAKMP_NEXT_v2N] ) {
-	    if ( (md->chain[ISAKMP_NEXT_v2N]->payload.v2n.isan_type == USE_TRANSPORT_MODE )) {
+	struct payload_digest *p;	
+
+	for(p = md->chain[ISAKMP_NEXT_v2N]; p != NULL; p = p->next)
+	{
+	    if ( p->payload.v2n.isan_type == USE_TRANSPORT_MODE ) {
 		if ( st->st_connection->policy & POLICY_TUNNEL) {
 		/*This means we did not send USE_TRANSPORT, however responder is sending it in now (inR2), seems incorrect*/
 			DBG(DBG_CONTROLMORE,
@@ -2007,18 +2031,17 @@ stf_status ikev2parent_inR2(struct msg_digest *md)
 			st->st_esp.attrs.encapsulation = ENCAPSULATION_MODE_TRANSPORT;
 			}
 		}
+	    break;
 	    }
 	}
 
-        if ( md->chain[ISAKMP_NEXT_v2N] == NULL ||
-               !(md->chain[ISAKMP_NEXT_v2N]->payload.v2n.isan_type == USE_TRANSPORT_MODE ) ) {
+        if (!p) {
                 if ( !(st->st_connection->policy & POLICY_TUNNEL) ) {
                 /*This means we sent USE_TRANSPORT, however responder did not send it or did not agree with that*/
                         DBG(DBG_CONTROLMORE,
 			DBG_log("Initiator policy is transport, responder did not send USE_TRANSPORT_MODE, so falling back to tunnel mode (rfc 4306)"));
                 }
         }
-
     }
 
 	
@@ -2057,7 +2080,7 @@ static bool ikev2_get_dcookie(u_char *dcookie,  chunk_t st_ni
 	SHA1Init(&ctx_sha1);
 	SHA1Update(&ctx_sha1, st_ni.ptr, st_ni.len);
 	SHA1Update(&ctx_sha1, addr_buff, addr_length);
-	SHA1Update(&ctx_sha1, spiI, sizeof(spiI));
+	SHA1Update(&ctx_sha1, spiI, sizeof(*spiI));
 	SHA1Update(&ctx_sha1, ikev2_secret_of_the_day
 		 , SHA1_DIGEST_SIZE);
 	SHA1Final(dcookie, &ctx_sha1);
@@ -2099,6 +2122,7 @@ send_v2_notification(struct state *p1st, u_int16_t type
     u_char buffer[1024];
     pb_stream reply;
     pb_stream rbody;
+	chunk_t child_spi;
 	/* this function is not generic enough yet just enough for 6msg 
 	 * TBD accept HDR FLAGS as arg. default ISAKMP_FLAGS_R
 	 * TBD when there is a child SA use that SPI in the notify paylod.
@@ -2141,7 +2165,6 @@ send_v2_notification(struct state *p1st, u_int16_t type
 	}
 		
     } 
-	chunk_t child_spi;
 	child_spi.ptr = NULL;
 	child_spi.len = 0;
 
@@ -2163,10 +2186,10 @@ bool ship_v2N (unsigned int np, u_int8_t  critical,
 				    u_int8_t protoid, chunk_t *spi, 
 					u_int16_t type, chunk_t *n_data, pb_stream *rbody)
 {
-	DBG(DBG_CONTROLMORE
-   		,DBG_log("Adding a v2N Payload"));  
    	struct ikev2_notify n;
    	pb_stream n_pbs;
+	DBG(DBG_CONTROLMORE
+   		,DBG_log("Adding a v2N Payload"));  
    	n.isan_np =  np;
    	n.isan_critical = critical;
    	n.isan_protoid =  protoid;
