@@ -1,6 +1,9 @@
 /* do X.509 operations for IKEv2
  *
  * Copyright (C) 2008 Antony Antony <antony@xelerance.com>
+ * Copyright (C) 2008 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2008 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2009-2010 Paul Wouters <paul@xelerance.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -129,8 +132,6 @@ ikev2_send_cert( struct state *st, struct msg_digest *md
     /*   send own (Initiator CERT) */
     {
 	pb_stream cert_pbs;
-	struct isakmp_cert cert_hd;
-	cert_hd.isacert_type = mycert.type;
 	
         DBG_log("I am sending my certificate");
 	
@@ -173,15 +174,15 @@ ikev2_send_certreq( struct state *st, struct msg_digest *md
 	}
     else
 	{
+	    generalName_t *ca = NULL;
 	DBG(DBG_CONTROL
 	    , DBG_log("connection->kind is not CK_PERMANENT (instance), so collect CAs"));
-	    generalName_t *ca = NULL;
 	    
 	    if (collect_rw_ca_candidates(md, &ca))
 		{
+		    generalName_t *gn;
 	            DBG(DBG_CONTROL
 	                , DBG_log("connection is RW, lookup CA candidates"));
-		    generalName_t *gn;
 		    
 		    for (gn = ca; gn != NULL; gn = gn->next)
 			{
@@ -226,7 +227,6 @@ doi_send_ikev2_cert_thinking(struct state *st)
     cert_t mycert = st->st_connection->spd.this.cert;
     enum ipsec_cert_type certtype = mycert.type;
     enum certpolicy policy = st->st_connection->spd.this.sendcert;
-    bool gotcertrequest = st->hidden_variables.st_got_certrequest;
     bool send_cert	 = FALSE;
 
     struct connection *c  = st->st_connection;
@@ -249,13 +249,14 @@ doi_send_ikev2_cert_thinking(struct state *st)
 		     (c->policy & POLICY_RSASIG) ? "" : "no",
 		     prettypolicy(c->policy)));
 
-    DBG(DBG_CONTROL
-	, DBG_log(" sendcert: %s and I did%s get a certificate request "
+    DBG(DBG_CONTROL,
+	bool gotcertrequest = st->hidden_variables.st_got_certrequest;
+	DBG_log(" sendcert: %s and I did%s get a certificate request "
 		  , enum_show(&certpolicy_type_names, policy)
-		  , gotcertrequest ? "" : " not"));
+		  , gotcertrequest ? "" : " not")
+       );
 
-    DBG(DBG_CONTROL
-	, DBG_log(" so %ssend cert.", send_cert ? "" : "do not "));
+    DBG(DBG_CONTROL, DBG_log(" so %ssend cert.", send_cert ? "" : "do not "));
 
     if(!send_cert) {
 	if(!(c->policy & POLICY_RSASIG))
