@@ -1,5 +1,12 @@
 /* information about connections between hosts and clients
- * Copyright (C) 1998-2002  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2010  D. Hugh Redelmeier.
+ * Copyright (C) 2007 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2007 Ken Bantoft <ken@xelerance.com>
+ * Copyright (C) 2009 Stefan Arentz <stefan@arentz.ca>
+ * Copyright (C) 2009-2010 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2007-2010 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2010 Avesh Agarwal <avagarwa@redhat.com>
+ * Copyright (C) 2010 Tuomo Soini <tis@foobar.fi>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -302,7 +309,7 @@ restart_connections_by_peer(struct connection *c)
 }
 
 /* (Possibly) Opportunistic Initiation:
- * Knowing clients (single IP addresses), try to build an tunnel.
+ * Knowing clients (single IP addresses), try to build a tunnel.
  * This may involve discovering a gateway and instantiating an
  * Opportunistic connection.  Called when a packet is caught by
  * a %trap, or when whack --oppohere --oppothere is used.
@@ -339,7 +346,6 @@ enum find_oppo_step {
     fos_done
 };
 
-#ifdef DEBUG
 static const char *const oppo_step_name[] = {
     "fos_start",
     "fos_myid_ip_txt",
@@ -354,7 +360,6 @@ static const char *const oppo_step_name[] = {
     "fos_his_client",
     "fos_done"
 };
-#endif /* DEBUG */
 
 struct find_oppo_bundle {
     enum find_oppo_step step;
@@ -401,10 +406,10 @@ cannot_oppo(struct connection *c
 	struct state *st;
 
 	passert(c->kind == CK_TEMPLATE);
-	passert(c->policy_next->kind == CK_PERMANENT);
+	passert(nc->kind == CK_PERMANENT);
 
 	DBG(DBG_OPPO, DBG_log("OE failed for %s to %s, but %s overrides shunt"
-			      , ocb, pcb, c->policy_next->name));
+			      , ocb, pcb, nc->name));
 
 	/*
 	 * okay, here we need add to the "next" policy, which is ought
@@ -469,25 +474,23 @@ cannot_oppo(struct connection *c
 	return;
     }
 
-#ifdef KLIPS
+#ifdef KLIPS /* This should really be 'if opportunistic is supported' - netlink has acquires too */
     if (b->held)
     {
 	int failure_shunt = b->failure_shunt;
 
 	/* Replace HOLD with b->failure_shunt.
-	 * If no b->failure_shunt specified, use SPI_PASS -- THIS MAY CHANGE.
+	 * If no failure_shunt specified, use SPI_PASS -- THIS MAY CHANGE.
 	 */
-	if (b->failure_shunt == 0)
+	if (failure_shunt == 0)
 	{
-	    DBG(DBG_OPPO, DBG_log("no explicit failure shunt for %s to %s; installing %%pass"
+	    DBG(DBG_OPPO, DBG_log("no explicit failure shunt for %s to %s; removing spurious hold shunt"
 				  , ocb, pcb));
-	    failure_shunt = SPI_PASS;
 	}
-
 	(void) replace_bare_shunt(&b->our_client, &b->peer_client
 	    , b->policy_prio
 	    , failure_shunt
-	    , failure_shunt == SPI_PASS
+	    , failure_shunt != 0
 	    , b->transport_proto
 	    , ughmsg);
     }
@@ -753,10 +756,10 @@ initiate_ondemand_body(struct find_oppo_bundle *b
 	cannot_oppo(NULL, b, "impossible IP address");
 	work = 0;
     }
-    else if ((c = find_connection_for_clients(&sr
+    else if (!(c = find_connection_for_clients(&sr
 					      , &b->our_client
 					      , &b->peer_client
-					      , b->transport_proto)) == NULL)
+					      , b->transport_proto)))
     {
 	/* No connection explicitly handles the clients and there
 	 * are no Opportunistic connections -- whine and give up.
@@ -831,10 +834,10 @@ initiate_ondemand_body(struct find_oppo_bundle *b
 	enum find_oppo_step next_step;
 	err_t ugh = ac_ugh;
 	char mycredentialstr[IDTOA_BUF];
-	char cib[CONN_INST_BUF];
 
-	DBG(DBG_CONTROL, DBG_log("creating new instance from \"%s\"%s"
-				 , c->name
+	DBG(DBG_CONTROL,
+	    char cib[CONN_INST_BUF];
+	    DBG_log("creating new instance from \"%s\"%s" , c->name
 				 , (fmt_conn_instance(c, cib), cib)));
 
 	idtoa(&sr->this.id, mycredentialstr, sizeof(mycredentialstr));
@@ -1526,7 +1529,7 @@ ISAKMP_SA_established(struct connection *c, so_serial_t serial)
 	{
 	    struct connection *next = d->ac_next;	/* might move underneath us */
 
-	    if (d->kind >= CK_PERMANENT
+	    if ( ((d->kind == CK_PERMANENT) || (d->kind == CK_INSTANCE) || (d->kind == CK_GOING_AWAY))
 	    && same_id(&c->spd.this.id, &d->spd.this.id)
 	    && same_id(&c->spd.that.id, &d->spd.that.id)
 	    && (!sameaddr(&c->spd.that.host_addr, &d->spd.that.host_addr)
@@ -1716,13 +1719,10 @@ void connection_check_phase2(void)
 	
 	if(pending_check_timeout(c)) {
 	    struct state *p1st;
-	    enum connection_kind kind;
 	    openswan_log("pending Quick Mode with %s \"%s\" took too long -- replacing phase 1" 
 			 , ip_str(&c->spd.that.host_addr)
 			 , c->name);
 
-	    kind = c->kind;
-
 	    p1st = find_phase1_state(c, ISAKMP_SA_ESTABLISHED_STATES|PHASE1_INITIATOR_STATES);
 
 	    if(p1st) {
