@@ -397,6 +397,7 @@ ipsecdoi_replace(struct state *st
 		 , lset_t policy_add, lset_t policy_del
 		 , unsigned long try)
 {
+	initiator_function *initiator;
     int whack_sock = dup_any(st->st_whack_sock);
     lset_t policy = st->st_policy;
 
@@ -407,7 +408,7 @@ ipsecdoi_replace(struct state *st
 	policy = policy & ~policy_del;
 	policy = policy | policy_add;
 
-	initiator_function *initiator = pick_initiator(c, policy);
+	initiator = pick_initiator(c, policy);
 	passert(!HAS_IPSEC_POLICY(policy));
 	if(initiator) {
 	    (void) initiator(whack_sock, st->st_connection, st, policy
@@ -486,7 +487,7 @@ has_preloaded_public_key(struct state *st)
  */
 
 bool
-extract_peer_id(struct id *peer, const pb_stream const* id_pbs)
+extract_peer_id(struct id *peer, const pb_stream *id_pbs)
 {
     switch (peer->kind)
     {
@@ -511,13 +512,9 @@ extract_peer_id(struct id *peer, const pb_stream const* id_pbs)
     case ID_USER_FQDN:
 	if (memchr(id_pbs->cur, '@', pbs_left(id_pbs)) == NULL)
 	{
-	    char idbuf[IDTOA_BUF];
-	    int len = pbs_left(id_pbs);
-	    if(len>(IDTOA_BUF-1)) len = IDTOA_BUF;
-
-	    memcpy(idbuf, id_pbs->cur, len-1);
-	    idbuf[len]='\0';
-	    loglog(RC_LOG_SERIOUS, "peer's ID_USER_FQDN contains no @: %s", idbuf);
+	    loglog(RC_LOG_SERIOUS, "peer's ID_USER_FQDN contains no @: %.*s"
+		, (int) pbs_left(id_pbs)
+		, id_pbs->cur);
 	    /* return FALSE; */
 	}
 	/* FALLTHROUGH */
@@ -607,7 +604,7 @@ decode_peer_id(struct msg_digest *md, bool initiator, bool aggrmode)
      * things for checking the hash
      */
     st->st_peeridentity_protocol = id->isaid_doi_specific_a;
-    st->st_peeridentity_port = id->isaid_doi_specific_b;
+    st->st_peeridentity_port = ntohs(id->isaid_doi_specific_b);
 
     {
 	char buf[IDTOA_BUF];
@@ -688,7 +685,13 @@ decode_peer_id(struct msg_digest *md, bool initiator, bool aggrmode)
 	    }
 
 	    st->st_connection = r;	/* kill reference to c */
-	    set_cur_connection(r);
+
+	    /* this ensures we don't move cur_connection from NULL to
+	     * something, requiring a reset_cur_connection() */
+	    if (cur_connection == c) {
+		set_cur_connection(r);
+	    }
+
 	    connection_discard(c);
 	}
 	else if (c->spd.that.has_id_wildcards)
