@@ -1,9 +1,12 @@
 /* routines that interface with the kernel's IPsec mechanism
  * Copyright (C) 1997 Angelos D. Keromytis.
- * Copyright (C) 1998-2002  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2010  D. Hugh Redelmeier.
  * Copyright (C) 2003-2008 Michael Richardson <mcr@xelerance.com>
- * Copyright (C) 2007-2009 Paul Wouters <paul@xelerance.com>
- * Copyright (C) 2008-2009 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2007-2010 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2008-2010 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2010 Bart Trojanowski <bart@jukie.net>
+ * Copyright (C) 2009-2010 Tuomo Soini <tis@foobar.fi>
+ * Copyright (C) 2010 Avesh Agarwal <avagarwa@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -118,7 +121,7 @@ DBG_bare_shunt_log(const char *op, const struct bare_shunt *bs)
             subnettot(&(bs)->his, 0, hist, sizeof(hist));
             satot(&(bs)->said, 0, sat, sizeof(sat));
             fmt_policy_prio(bs->policy_prio, prio);
-            DBG_log("%s bare shunt %p %s:%d -%d-> %s:%d => %s %s    %s"
+            DBG_log("%s bare shunt %p %s:%d --%d--> %s:%d => %s %s    %s"
                 , op, (const void *)(bs), ourst, ourport, (bs)->transport_proto, hist, hisport
                 , sat, prio, (bs)->why);
         });
@@ -133,7 +136,7 @@ record_and_initiate_opportunistic(const ip_subnet *ours
 {
     passert(samesubnettype(ours, his));
 
-    /* Add to bare shunt list.
+    /* Add the kernel shunt to the pluto bare shunt list.
      * We need to do this because the shunt was installed by KLIPS
      * which can't do this itself.
      */
@@ -307,6 +310,7 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
 	peer_str[ADDRTOT_BUF],
 	peerid_str[IDTOA_BUF],
 	metric_str[sizeof("PLUTO_METRIC")+5],
+	connmtu_str[sizeof("PLUTO_MTU")+5],
 	peerclient_str[SUBNETTOT_BUF],
 	peerclientnet_str[ADDRTOT_BUF],
 	peerclientmask_str[ADDRTOT_BUF],
@@ -352,6 +356,10 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
     if (c->metric)
     	snprintf(metric_str, sizeof(metric_str), "PLUTO_METRIC=%d", c->metric);
 
+    connmtu_str[0]='\0';
+    if (c->connmtu)
+    	snprintf(connmtu_str, sizeof(connmtu_str), "PLUTO_MTU=%d", c->connmtu);
+
     secure_xauth_username_str[0]='\0';
     if (st != NULL && st->st_xauth_username) {
 	size_t len;
@@ -419,13 +427,19 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
 		    "PLUTO_PEER_PROTOCOL='%u' "
 		    "PLUTO_PEER_CA='%s' "
 		    "PLUTO_STACK='%s' "
-		    "%s "           /* possible metric */
+		    "%s "           /* optional metric */
+		    "%s "           /* optional mtu */
 		    "PLUTO_CONN_POLICY='%s' "
-		    "%s "           /* XAUTH username */
-		    "%s "           /* PLUTO_MY_SRCIP */
+		    "PLUTO_CONN_ADDRFAMILY='ipv%d' "
+#ifdef XAUTH
+		    "%s "           /* XAUTH username - if any */
+#endif
+		    "%s "           /* PLUTO_MY_SRCIP - if any */
+#ifdef XAUTH
 		    "PLUTO_CISCO_DNS_INFO='%s' "
 		    "PLUTO_CISCO_DOMAIN_INFO='%s' "
 		    "PLUTO_PEER_BANNER='%s' "
+#endif /* XAUTH */
 #ifdef HAVE_NM
 		    "PLUTO_NM_CONFIGURED='%u' "
 #endif
@@ -450,12 +464,18 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
 		    , secure_peerca_str
 		    , kernel_ops->kern_name
 		    , metric_str
+		    , connmtu_str
 		    , prettypolicy(c->policy)
+		    , (c->addr_family == AF_INET) ? 4 : 6
+#ifdef XAUTH
 		    , secure_xauth_username_str
+#endif
 		    , srcip_str
-		    , c->cisco_dns_info
-		    , c->cisco_domain_info
-		    , c->server_banner
+#ifdef XAUTH
+		    , c->cisco_dns_info ? c->cisco_dns_info : ""
+		    , c->cisco_domain_info ? c->cisco_domain_info : ""
+		    , c->cisco_banner ? c->cisco_banner : ""
+#endif /* XAUTH */
 #ifdef HAVE_NM
 		    , c->nmconfigured
 #endif
@@ -467,7 +487,7 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
 	return ((result>=blen) || (result<0))? -1 : result;
 }
 
-static bool
+bool
 do_command(struct connection *c, struct spd_route *sr, const char *verb, struct state *st)
 {
     const char *verb_suffix;
@@ -590,16 +610,15 @@ could_route(struct connection *c)
         /* We ignore this if the stack supports overlapping, and this
          * connection was marked that overlapping is OK.  Below we will
          * check the other eroute, ero. */
-        if (!kernel_ops->overlap_supported || !ero || ero == c
-                        || !LIN(POLICY_OVERLAPIP, c->policy))
-                return route_impossible;  /* another connection already using the
+	if (!compatible_overlapping_connections(c, ero))
+		return route_impossible;  /* another connection already using the
                                              eroute. TODO: NETKEY can do this? */
     }
 
     /* if there is an eroute for another connection, there is a problem */
     if (ero != NULL && ero != c)
     {
-        struct connection *ero2, *ero_top;
+        struct connection *ero2;
         struct connection *inside, *outside;
 
         /*
@@ -657,7 +676,7 @@ could_route(struct connection *c)
         }
 
         /* look along the chain of policies for one with the same name */
-        ero_top = ero;
+
 
         for (ero2 = ero; ero2 != NULL; ero2 = ero->policy_next)
         {
@@ -883,14 +902,14 @@ raw_eroute(const ip_address *this_host
            , const char *opname USED_BY_DEBUG)
 {
     char text_said[SATOT_BUF];
-    int sport = ntohs(portof(&this_client->addr));
-    int dport = ntohs(portof(&that_client->addr));
     bool result;
 
     set_text_said(text_said, that_host, spi, proto);
 
     DBG(DBG_CONTROL | DBG_KLIPS,
         {
+            int sport = ntohs(portof(&this_client->addr));
+            int dport = ntohs(portof(&that_client->addr));
             char mybuf[SUBNETTOT_BUF];
             char peerbuf[SUBNETTOT_BUF];
 
@@ -968,7 +987,7 @@ clear_narrow_holds(
 	    (void) replace_bare_shunt(&p->ours.addr, &p->his.addr
 		    , BOTTOM_PRIO
 		    , SPI_PASS	/* not used */
-		    , FALSE, 0
+		    , FALSE, transport_proto
 		    , "removing clashing narrow holds");
 
 	    /* restart from beginning as we just removed and entry */
@@ -1034,7 +1053,7 @@ replace_bare_shunt(const ip_address *src, const ip_address *dst
                                 
                                 bs->ours = this_broad_client;
                                 bs->his =  that_broad_client;
-                                bs->transport_proto = 0;
+                                bs->transport_proto = transport_proto;
                                 bs->said.proto = SA_INT;
                                 bs->why = clone_str(why, "bare shunt story");
                                 bs->policy_prio = policy_prio;
@@ -1084,9 +1103,12 @@ replace_bare_shunt(const ip_address *src, const ip_address *dst
                 struct bare_shunt **bs_pp = bare_shunt_ptr(&this_client
                                                            , &that_client, 0);
                 
+		passert(bs_pp != NULL);
                 if (repl)
                     {
-                        /* change over to new bare eroute */
+                        /* change over to new bare eroute
+			 * ours, his, transport_proto are the same.
+			 */
                         struct bare_shunt *bs = *bs_pp;
                         
                         pfree(bs->why);
@@ -1480,18 +1502,6 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 	    outgoing_ref_set  = TRUE;
 	}
 
-	if(inbound) {
-	    /*
-	     * set corresponding outbound SA. We can do this on
-	     * each SA in the bundle without harm.
-	     */
-	    said_next->refhim = refhim;
-	} else if (!outgoing_ref_set) {
-	    /* on outbound, pick up the SAref if not already done */
-	    said_next->ref    = refhim;
-	    outgoing_ref_set  = TRUE;
-	}
-
         if (!kernel_ops->add_sa(said_next, replace)) {
 	    DBG_log("add_sa ipcomp failed");
             goto fail;
@@ -2148,19 +2158,6 @@ init_kernel(void)
 	/* FALL THROUGH */
 #endif
 
-#if defined(KLIPS_MAST) 
-    case USE_MASTKLIPS:
-        if (stat("/proc/sys/net/ipsec/debug_mast", &buf) == 0) {
-	    kern_interface = USE_MASTKLIPS;
-	    openswan_log("Using KLIPSng (mast) IPsec interface code on %s"
-			 , kversion);
-	    kernel_ops = &mast_kernel_ops;
-	    break;
-	} else
-	    openswan_log("No Kernel MASTKLIPS interface detected");
-	/* FALL THROUGH */
-#endif
-
 #if defined(KLIPS) 
     case USE_KLIPS:
 	if (stat("/proc/net/pf_key", &buf) == 0) {
@@ -2174,6 +2171,19 @@ init_kernel(void)
 	/* FALL THROUGH */
 #endif
 
+#if defined(KLIPS_MAST) 
+    case USE_MASTKLIPS:
+        if (stat("/proc/sys/net/ipsec/debug_mast", &buf) == 0) {
+	    kern_interface = USE_MASTKLIPS;
+	    openswan_log("Using KLIPSng (mast) IPsec interface code on %s"
+			 , kversion);
+	    kernel_ops = &mast_kernel_ops;
+	    break;
+	} else
+	    openswan_log("No Kernel MASTKLIPS interface detected");
+	/* FALL THROUGH */
+#endif
+
 #if defined(BSD_KAME) 
     case USE_BSDKAME:
 	kern_interface = USE_BSDKAME;
@@ -2370,8 +2380,7 @@ route_and_eroute(struct connection *c USED_BY_KLIPS
 {
     struct spd_route *esr;
     struct spd_route *rosr;
-    struct connection *ero      /* who, if anyone, owns our eroute? */
-        , *ro = route_owner(c, sr, &rosr, &ero, &esr);
+    struct connection *ero, *ro;      /* who, if anyone, owns our eroute? */
     bool eroute_installed = FALSE
         , firewall_notified = FALSE
         , route_installed = FALSE;
@@ -2379,9 +2388,10 @@ route_and_eroute(struct connection *c USED_BY_KLIPS
     bool new_eroute = FALSE;
 #endif
 
-    struct connection *ero_top;
     struct bare_shunt **bspp;
 
+    ro = route_owner(c, sr, &rosr, &ero, &esr);
+
     DBG(DBG_CONTROLMORE,
         DBG_log("route_and_eroute with c: %s (next: %s) ero:%s esr:{%p} ro:%s rosr:{%p} and state: %lu"
                 , c->name
@@ -2393,7 +2403,7 @@ route_and_eroute(struct connection *c USED_BY_KLIPS
                 , st ? st->st_serialno : 0));
 
     /* look along the chain of policies for one with the same name */
-    ero_top = ero;
+
 
 #if 0
     /* XXX - mcr this made sense before, and likely will make sense
@@ -2566,10 +2576,10 @@ route_and_eroute(struct connection *c USED_BY_KLIPS
         }
         else
         {
-            char cib[CONN_INST_BUF];
             sr->routing = RT_ROUTED_TUNNEL;
 
             DBG(DBG_CONTROL,
+                char cib[CONN_INST_BUF];
                 DBG_log("route_and_eroute: instance \"%s\"%s, setting eroute_owner {spd=%p,sr=%p} to #%ld (was #%ld) (newest_ipsec_sa=#%ld)"
                         , st->st_connection->name
                         , (fmt_conn_instance(st->st_connection, cib), cib)
@@ -2746,8 +2756,15 @@ install_ipsec_sa(struct state *st, bool inbound_also USED_BY_KLIPS)
     }
 
    if (st->st_connection->remotepeertype == CISCO) {
-	if(!do_command(st->st_connection, &st->st_connection->spd, "updateresolvconf", st)) {
-	DBG(DBG_CONTROL, DBG_log("Updating resolv.conf failed, you may need to update it manually"));
+
+	sr = st->st_connection->spd.next;
+	st->st_connection->spd.eroute_owner = sr->eroute_owner;
+	st->st_connection->spd.routing = sr->routing;
+
+	if(!st->st_connection->newest_ipsec_sa) {
+		if(!do_command(st->st_connection, &st->st_connection->spd, "updateresolvconf", st)) {
+		DBG(DBG_CONTROL, DBG_log("Updating resolv.conf failed, you may need to update it manually"));
+		}
 	}
    }
 
@@ -2789,7 +2806,11 @@ delete_ipsec_sa(struct state *st USED_BY_KLIPS, bool inbound_only USED_BY_KLIPS)
 		     */
 		    sr->routing = (c->policy & POLICY_FAIL_MASK) == POLICY_FAIL_NONE
 			? RT_ROUTED_PROSPECTIVE : RT_ROUTED_FAILURE;
-		    
+
+		    if (sr == &c->spd && c->remotepeertype == CISCO) {
+			continue;
+		    }
+
 		    (void) do_command(c, sr, "down", st);
 		    if ((c->policy & POLICY_DONT_REKEY)
 			&& c->kind == CK_INSTANCE)
@@ -2819,7 +2840,7 @@ delete_ipsec_sa(struct state *st USED_BY_KLIPS, bool inbound_only USED_BY_KLIPS)
 	}
 	(void) teardown_half_ipsec_sa(st, TRUE);
 
-	if (st->st_connection->remotepeertype == CISCO) {
+	if (st->st_connection->remotepeertype == CISCO && st->st_serialno == st->st_connection->newest_ipsec_sa) {
 		if(!do_command(st->st_connection, &st->st_connection->spd, "restoreresolvconf", st)) {
 		DBG(DBG_CONTROL, DBG_log("Restoring resolv.conf failed, you may need to do it manually"));
 		}
