@@ -206,6 +206,9 @@ extern int fmt_common_shell_out(char *buf, int blen, struct connection *c
 extern bool pfkey_plumb_mast_device(int mast_dev);
 #endif
 
+/* many bits reach in to use this, but maybe shouldn't */
+extern bool do_command(struct connection *c, struct spd_route *sr, const char *verb, struct state *st);
+
 #if defined(linux)
 extern bool do_command_linux(struct connection *c, struct spd_route *sr
 			     , const char *verb, struct state *st);
@@ -353,6 +356,16 @@ extern bool eroute_connection(struct spd_route *sr
 			      , const struct pfkey_proto_info *proto_info
 			      , unsigned int op, const char *opname);
 
+static inline bool
+compatible_overlapping_connections(struct connection *a, struct connection *b)
+{
+	return kernel_ops->overlap_supported
+		&& a && b
+		&& a != b
+		&& LIN(POLICY_OVERLAPIP, a->policy)
+		&& LIN(POLICY_OVERLAPIP, a->policy);
+}
+
 #ifdef KLIPS
 extern const struct kernel_ops klips_kernel_ops;
 #endif
