@@ -29,9 +29,8 @@
 
 #include <openswan.h>
 #include <net/pfkeyv2.h>
-#include <netkey/keydb.h>
+#include <netipsec/keydb.h>
 #include <netinet/in.h>
-#include <netinet6/ipsec.h>
 #include "libbsdkame/libpfkey.h"         /* this is a copy of a freebsd libipsec/ file */
 
 #include "sysdep.h"
@@ -165,7 +164,7 @@ bsdkame_process_raw_ifaces(struct raw_iface *rifaces)
 
 		    q->ip_dev = id;
 		    id->id_rname = clone_str(ifp->name, "real device name");
-		    id->id_vname = clone_str(ifp->name, "virtual device name");
+		    id->id_vname = clone_str(ifp->name, "virtual device name bsd");
 		    id->id_count++;
 
 		    q->ip_addr = ifp->addr;
@@ -1114,6 +1113,7 @@ const struct kernel_ops bsdkame_kernel_ops = {
     set_debug: bsdkame_set_debug,
     remove_orphaned_holds: bsdkame_remove_orphaned_holds,
     process_ifaces: bsdkame_process_raw_ifaces,
+    .overlap_supported = FALSE
 };
 
 
