@@ -55,6 +55,7 @@
 
 static int next_free_mast_device=-1;
 int  useful_mastno=-1;
+extern char *pluto_listen;
 
 #ifndef DEFAULT_UPDOWN
 # define DEFAULT_UPDOWN "ipsec _updown"
@@ -147,7 +148,7 @@ allocate_mast_device(void)
 }
 
 static int
-init_useful_mast(ip_address addr, char *vname)
+init_useful_mast(ip_address addr UNUSED, char *vname)
 {
     int mastno;
 
@@ -162,7 +163,14 @@ init_useful_mast(ip_address addr, char *vname)
 	mastdevice[mastno]=MAST_INUSE;
 	pfkey_plumb_mast_device(mastno);
     }
-    
+
+
+#if 0
+    /*
+     * The mast interface does not need an IP address, as we route
+     *  into it using using route table 50 from the main routing table
+     */
+
     /* now configure an IP address on the mast number */
     {
 	char cmd[512];
@@ -175,6 +183,7 @@ init_useful_mast(ip_address addr, char *vname)
 	
 	invoke_command("plumb","", cmd);
     }
+#endif
     return mastno;
 }
 
@@ -186,6 +195,16 @@ mast_process_raw_ifaces(struct raw_iface *rifaces)
     struct iface_port *firstq=NULL;
     char useful_mast_name[256];
     bool found_mast=FALSE;
+    ip_address lip; /* --listen filter option */
+
+    if(pluto_listen) {
+	err_t e;
+	e = ttoaddr(pluto_listen,0,0,&lip);
+	if (e) {
+		DBG_log("invalid listen= option ignored: %s\n", e);
+		pluto_listen = NULL;
+	}
+    }
 
     strcpy(useful_mast_name, "useless");
     { int new_useful=recalculate_mast_device_list(rifaces);
@@ -219,6 +238,15 @@ mast_process_raw_ifaces(struct raw_iface *rifaces)
 	if (strncmp(ifp->name, "lo", 2) == 0)
 	    continue;
 
+	/* ignore if --listen is specified and we do not match */
+	if (pluto_listen!=NULL) {
+	   if (!sameaddr(&lip, &ifp->addr)) {
+		openswan_log("skipping interface %s with %s"
+			     , ifp->name , ip_str(&ifp->addr));
+		continue;
+	   }
+	}
+
 	/*
 	 * see if this is a new thing: search old interfaces list.
 	 */
@@ -271,7 +299,7 @@ mast_process_raw_ifaces(struct raw_iface *rifaces)
 		}
 
 		vname = clone_str(useful_mast_name
-				  , "virtual device name");
+				  , "virtual device name mast");
 		fd = create_socket(ifp, vname, pluto_port);
 		
 		if (fd < 0) 
@@ -417,11 +445,14 @@ mast_do_command(struct connection *c, struct spd_route *sr
 		       , "2>&1 "   /* capture stderr along with stdout */
 		       "PLUTO_MY_REF=%u "
 		       "PLUTO_PEER_REF=%u "
+		       "PLUTO_SAREF_TRACKING=%s "
 		       "PLUTO_VERB='%s%s' "
 		       "%s"        /* other stuff   */
 		       "%s"        /* actual script */
 		       , ref
 		       , refhim
+		       , (c->policy & POLICY_SAREF_TRACK_CONNTRACK) ? "conntrack" : 
+			( (c->policy & POLICY_SAREF_TRACK) ? "yes" : "no")
 		       , verb, verb_suffix
 		       , common_shell_out_str
 		       , sr->this.updown == NULL? DEFAULT_UPDOWN : sr->this.updown))
@@ -435,21 +466,23 @@ mast_do_command(struct connection *c, struct spd_route *sr
 
 static bool
 mast_raw_eroute(const ip_address *this_host UNUSED
-		, const ip_subnet *this_client UNUSED
-		, const ip_address *that_host UNUSED
-		, const ip_subnet *that_client UNUSED
-		, ipsec_spi_t spi UNUSED
-		, unsigned int proto UNUSED
-		, unsigned int transport_proto UNUSED
-		, unsigned int satype UNUSED
-		, const struct pfkey_proto_info *proto_info UNUSED
-		, time_t use_lifetime UNUSED
-		, enum pluto_sadb_operations op UNUSED
-		, const char *text_said UNUSED)
+               , const ip_subnet *this_client UNUSED
+               , const ip_address *that_host UNUSED
+               , const ip_subnet *that_client UNUSED
+               , ipsec_spi_t spi UNUSED
+               , unsigned int proto UNUSED
+               , unsigned int transport_proto UNUSED
+               , unsigned int satype UNUSED
+               , const struct pfkey_proto_info *proto_info UNUSED
+               , time_t use_lifetime UNUSED
+               , enum pluto_sadb_operations op UNUSED
+               , const char *text_said UNUSED)
 {
-	
-	/* actually, we did all the work with iptables in _updown */
-	return TRUE;
+    /* actually, we did all the work with iptables in _updown */
+    DBG_log("mast_raw_eroute called op=%u said=%s", op, text_said);
+    return pfkey_raw_eroute(this_host, this_client, that_host, that_client,
+		    spi, proto, transport_proto, satype,
+		    proto_info, use_lifetime, op, text_said);
 }
 
 /* Add/replace/delete a shunt eroute.
@@ -466,17 +499,94 @@ mast_shunt_eroute(struct connection *c UNUSED
 		   , enum pluto_sadb_operations op UNUSED
 		  , const char *opname UNUSED)
 {
-    DBG_log("mast_shunt_eroute called");
+    DBG_log("mast_shunt_eroute called op=%u/%s", op, opname);
     return TRUE;
 }
 
+/**
+ * replace existing iptables rule using the updown.mast script.
+ * @param st - new state
+ * @param sr - new route
+ * @return TRUE if add was successful, FALSE otherwise
+ */
+static bool
+mast_sag_eroute_replace(struct state *st, struct spd_route *sr)
+{
+	struct connection *c = st->st_connection;
+	struct state *old_st;
+	bool success;
+
+	/* The state, st, has the new SAref values, but we need to remove
+	 * the rule based on the previous state with the old SAref values.
+	 * So we have to find it the hard way (it's a cpu hog). */
+	old_st = state_with_serialno(sr->eroute_owner);
+	if (!old_st)
+		old_st = st;
+
+	DBG_log("mast_sag_eroute_replace state #%d{ref=%d refhim=%d} "
+			"with #%d{ref=%d refhim=%d}",
+			(int)old_st->st_serialno,
+			(int)old_st->st_ref,
+			(int)old_st->st_refhim,
+			(int)st->st_serialno,
+			(int)st->st_ref,
+			(int)st->st_refhim);
+
+	/* add the new rule */
+	success = mast_do_command(c, sr, "spdadd", st);
+
+	/* drop the old rule -- we ignore failure */
+	if (old_st->st_serialno != st->st_serialno)
+	    (void)mast_do_command(c, sr, "spddel", old_st);
+
+	return success;
+}
+
 /* install or remove eroute for SA Group */
 static bool
 mast_sag_eroute(struct state *st, struct spd_route *sr
 		, enum pluto_sadb_operations op, const char *opname UNUSED)
 {
-    switch(op)
-    {
+    bool ok;
+    bool addop = FALSE;
+
+    DBG_log("mast_sag_eroute called op=%u/%s", op, opname);
+
+    /* handle ops we have to do no work for */
+    switch(op) {
+    default:
+	bad_case(op);
+	return FALSE;
+
+    case ERO_ADD:
+    case ERO_ADD_INBOUND:
+	addop = TRUE;
+	/* fallthrough expected */
+    case ERO_REPLACE:
+    case ERO_REPLACE_INBOUND:
+    case ERO_DELETE:
+    case ERO_DEL_INBOUND:
+	/* these one require more work... */
+	break;
+    }
+
+    /* first try to update the routing policy */
+    ok = pfkey_sag_eroute(st, sr, op, opname);
+    if (!ok) {
+        DBG_log("mast_sag_eroute failed to %s/%d pfkey eroute", opname, op);
+        if (addop)
+            /* If the pfkey op failed, and we were adding a new SA,
+             * then it's OK to fail early. */
+            return FALSE;
+    }
+
+    /* now run the iptable updown script */
+    switch(op) {
+    case ERO_ADD_INBOUND:
+    case ERO_REPLACE_INBOUND:
+    case ERO_DEL_INBOUND:
+	return TRUE;
+
     case ERO_ADD:
 	return mast_do_command(st->st_connection, sr, "spdadd", st);
 	
@@ -484,15 +594,12 @@ mast_sag_eroute(struct state *st, struct spd_route *sr
 	return mast_do_command(st->st_connection, sr, "spddel", st);
 
     case ERO_REPLACE:
-	(void)mast_do_command(st->st_connection, sr, "spddel", st);
-	return mast_do_command(st->st_connection, sr, "spdadd", st);
-	
-    case ERO_ADD_INBOUND:
-    case ERO_REPLACE_INBOUND:
-    case ERO_DEL_INBOUND:
-	return TRUE;
+	return mast_sag_eroute_replace(st, sr);
+
+    default:
+	/* this should never happen */
+	return FALSE;
     }
-    return FALSE;
 }
 
 const struct kernel_ops mast_kernel_ops = {
