@@ -1,6 +1,6 @@
 /* netlink interface to the kernel's IPsec mechanism
  *
- * Copyright (C) 2003,2008 Herbert Xu.
+ * Copyright (C) 2003-2008 Herbert Xu
  * Copyright (C) 2006-2008 Michael Richardson <mcr@xelerance.com>
  * Copyright (C) 2006 Ken Bantoft <ken@xelerance.com>
  * Copyright (C) 2007 Bart Trojanowski <bart@jukie.net>
@@ -8,10 +8,12 @@
  * Copyright (C) 2009 Carsten Schlote <c.schlote@konzeptpark.de>
  * Copyright (C) 2008 Andreas Steffen
  * Copyright (C) 2008 Neil Horman <nhorman@redhat.com>
- * Copyright (C) 2008-2009 David McCullough <david_mccullough@securecomputing.com>
- * Copyright (C) 2006-2009 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2008-2010 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2006-2010 Paul Wouters <paul@xelerance.com>
  * Copyright (C) 2010 Tuomo Soini <tis@foobar.fi>
  * Copyright (C) 2010 Mika Ilmaranta <ilmis@foobar.fi>
+ * Copyright (C) 2010 Roman Hoog Antink <rha@open.ch>
+ * Copyright (C) 2010 D. Hugh Redelmeier
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -75,6 +77,8 @@
 # define DEFAULT_UPDOWN "ipsec _updown"
 #endif
 
+extern char *pluto_listen;
+
 extern const struct pfkey_proto_info null_proto_info[2];
 
 static const struct pfkey_proto_info broad_proto_info[2] = { 
@@ -457,7 +461,7 @@ netlink_raw_eroute(const ip_address *this_host
 		   , const ip_subnet *that_client
 		   , ipsec_spi_t spi
 		   , unsigned int proto
-		   , unsigned int transport_proto UNUSED
+		   , unsigned int transport_proto
 		   , enum eroute_type esatype
 		   , const struct pfkey_proto_info *proto_info
 		   , time_t use_lifetime UNUSED
@@ -476,10 +480,10 @@ netlink_raw_eroute(const ip_address *this_host
     int dir;
     int family;
     int policy;
-    int satype;
     bool ok;
     bool enoent_ok;
     ip_subnet local_that_client;
+    int satype = 0;
 
     policy = IPSEC_POLICY_IPSEC;
 
@@ -529,6 +533,8 @@ netlink_raw_eroute(const ip_address *this_host
 	    }
 	    break;
     }
+    DBG(DBG_NETKEY,
+	DBG_log("satype(%d) is not used in netlink_raw_eroute.",satype));
 
     /* Bug #1004 fix.
      * There really isn't "client" with NETKEY and transport mode
@@ -583,6 +589,30 @@ netlink_raw_eroute(const ip_address *this_host
 
     req.u.p.sel.sport = portof(&this_client->addr);
     req.u.p.sel.dport = portof(&that_client->addr);
+
+    /* As per RFC 4301/5996, icmp type is put in the most significant 8 bits
+     * and icmp code is in the least significant 8 bits of port field. 
+     * Although Openswan does not have any configuration options for 
+     * icmp type/code values, it is possible to specify icmp type and code 
+     * using protoport option. For example, icmp echo request (type 8/code 0) 
+     * needs to be encoded as 0x0800 in the port field and can be specified 
+     * as left/rightprotoport=icmp/2048. Now with NETKEY, icmp type and code
+     * need to be passed as source and destination ports, respectively.
+     * therefore, this code extracts upper 8 bits and lower 8 bits and puts
+     * into source and destination ports before passing to NETKEY. */
+
+    if( transport_proto == IPPROTO_ICMP || transport_proto == IPPROTO_ICMPV6) {
+	u_int16_t icmp_type;
+	u_int16_t icmp_code;
+
+	icmp_type = ntohs(req.u.p.sel.sport) >> 8;
+	icmp_code = ntohs(req.u.p.sel.sport) & 0xFF;
+
+	req.u.p.sel.sport = htons(icmp_type);
+	req.u.p.sel.dport = htons(icmp_code);
+	
+    }
+
     req.u.p.sel.sport_mask = (req.u.p.sel.sport) ? ~0:0;
     req.u.p.sel.dport_mask = (req.u.p.sel.dport) ? ~0:0;
     ip2xfrm(&this_client->addr, &req.u.p.sel.saddr);
@@ -797,7 +827,30 @@ netlink_add_sa(struct kernel_sa *sa, bool replace)
     }
 
     aead = get_aead_alg(sa->encalg);
-    if (aead)
+    if (sa->esatype == ET_IPCOMP)
+    {
+	struct xfrm_algo algo;
+	const char *name;
+
+	name = sparse_name(calg_list, sa->encalg);
+	if (!name) {
+	    loglog(RC_LOG_SERIOUS, "unknown compression algorithm: %u"
+		, sa->encalg);
+	    return FALSE;
+	}
+
+	strcpy(algo.alg_name, name);
+	algo.alg_key_len = 0;
+
+	attr->rta_type = XFRMA_ALG_COMP;
+	attr->rta_len = RTA_LENGTH(sizeof(algo));
+
+	memcpy(RTA_DATA(attr), &algo, sizeof(algo));
+
+	req.n.nlmsg_len += attr->rta_len;
+	attr = (struct rtattr *)((char *)attr + attr->rta_len);
+    }
+    else if (aead)
     {
 	struct xfrm_algo_aead algo;
 
@@ -815,7 +868,7 @@ netlink_add_sa(struct kernel_sa *sa, bool replace)
 	req.n.nlmsg_len += attr->rta_len;
 	attr = (struct rtattr *)((char *)attr + attr->rta_len);
     }
-    else
+    else if (sa->esatype == ET_ESP)
     {
 	struct xfrm_algo algo;
 	const char *name;
@@ -841,30 +894,6 @@ netlink_add_sa(struct kernel_sa *sa, bool replace)
 	attr = (struct rtattr *)((char *)attr + attr->rta_len);
     }
 
-    if (sa->esatype == ET_IPCOMP)
-    {
-	struct xfrm_algo algo;
-	const char *name;
-
-	name = sparse_name(calg_list, sa->encalg);
-	if (!name) {
-	    loglog(RC_LOG_SERIOUS, "unknown compression algorithm: %u"
-		, sa->encalg);
-	    return FALSE;
-	}
-
-	strcpy(algo.alg_name, name);
-	algo.alg_key_len = 0;
-
-	attr->rta_type = XFRMA_ALG_COMP;
-	attr->rta_len = RTA_LENGTH(sizeof(algo));
-
-	memcpy(RTA_DATA(attr), &algo, sizeof(algo));
-
-	req.n.nlmsg_len += attr->rta_len;
-	attr = (struct rtattr *)((char *)attr + attr->rta_len);
-    }
-
 #ifdef NAT_TRAVERSAL
     if (sa->natt_type)
     {
@@ -1094,7 +1123,7 @@ xfrm_to_ip_address(unsigned family, const xfrm_address_t *src, ip_address *dst)
 static void
 netlink_acquire(struct nlmsghdr *n)
 {
-    struct xfrm_user_acquire *acquire;
+    struct xfrm_user_acquire ac1, *acquire;
     const xfrm_address_t *srcx, *dstx;
     int src_proto, dst_proto;
     ip_address src, dst;
@@ -1111,7 +1140,11 @@ netlink_acquire(struct nlmsghdr *n)
 	return;
     }
 
-    acquire = NLMSG_DATA(n);
+    /* to get rid of complaints about strict alignment: */
+    /* structure copy it first */
+    memcpy(&ac1, NLMSG_DATA(n), sizeof(struct xfrm_user_acquire));
+    acquire = &ac1;   /* then use it. */
+
     srcx = &acquire->sel.saddr;
     dstx = &acquire->sel.daddr;
     family = acquire->policy.sel.family;
@@ -1172,7 +1205,7 @@ netlink_shunt_expire(struct xfrm_userpolicy_info *pol)
 static void
 netlink_policy_expire(struct nlmsghdr *n)
 {
-    struct xfrm_user_polexpire *upe;
+    struct xfrm_user_polexpire up1, *upe;
     struct {
 	struct nlmsghdr n;
 	struct xfrm_userpolicy_id id;
@@ -1191,7 +1224,8 @@ netlink_policy_expire(struct nlmsghdr *n)
 	return;
     }
 
-    upe = NLMSG_DATA(n);
+    memcpy(&up1, NLMSG_DATA(n), sizeof(up1));
+    upe = &up1;
     req.id.dir = upe->pol.dir;
     req.id.index = upe->pol.index;
     req.n.nlmsg_flags = NLM_F_REQUEST;
@@ -1457,10 +1491,8 @@ netlink_sag_eroute(struct state *st, struct spd_route *sr
         proto_info[i].reqid = sr->reqid + 2;
     }
 
-    if (i == sizeof(proto_info) / sizeof(proto_info[0]) - 1)
-    {
-        impossible();   /* no transform at all! */
-    }
+    /* check for no transform at all */
+    passert(!(!st->st_ipcomp.present && !st->st_esp.present && !st->st_ah.present));
 
     if (tunnel)
     {
@@ -1507,6 +1539,8 @@ netlink_shunt_eroute(struct connection *c
                    , enum pluto_sadb_operations op 
 		   , const char *opname)
 {
+    ipsec_spi_t spi;
+
     DBG(DBG_CONTROL, DBG_log("request to %s a %s policy with netkey kernel --- experimental"
 		, opname
 		, enum_name(&routing_story, rt_kind)));
@@ -1517,7 +1551,7 @@ netlink_shunt_eroute(struct connection *c
      * The satype has no meaning, but is required for PF_KEY header!
      * The SPI signifies the kind of shunt.
      */
-    ipsec_spi_t spi = shunt_policy_spi(c, rt_kind == RT_ROUTED_PROSPECTIVE);
+    spi = shunt_policy_spi(c, rt_kind == RT_ROUTED_PROSPECTIVE);
 
     if (spi == 0)
     {
@@ -1636,6 +1670,16 @@ static void
 netlink_process_raw_ifaces(struct raw_iface *rifaces)
 {
     struct raw_iface *ifp;
+    ip_address lip; /* --listen filter option */
+    if(pluto_listen) {
+	err_t e;
+	e = ttoaddr(pluto_listen,0,0,&lip);
+	if (e) {
+		DBG_log("invalid listen= option ignored: %s\n", e);
+		pluto_listen = NULL;
+	}
+	DBG(DBG_CONTROL, DBG_log("Only looking to listen on %s\n", ip_str(&lip)));
+    }
 
     /* Find all virtual/real interface pairs.
      * For each real interface...
@@ -1738,10 +1782,21 @@ netlink_process_raw_ifaces(struct raw_iface *rifaces)
 	    }
 	}
 
+
 	/* We've got all we need; see if this is a new thing:
 	 * search old interfaces list.
 	 */
 add_entry:
+	/* last check before we actually add the entry due to ugly goto code */
+	/* ignore if --listen is specified and we do not match */
+	if (pluto_listen!=NULL) {
+	    if (!sameaddr(&lip, &ifp->addr)) {
+		openswan_log("skipping interface %s with %s"
+		, ifp->name , ip_str(&ifp->addr));
+		continue;
+	    }
+	}
+
 	{
 	    struct iface_port **p = &interfaces;
 
@@ -1773,7 +1828,7 @@ add_entry:
 
 		    q->ip_dev = id;
 		    id->id_rname = clone_str(ifp->name, "real device name");
-		    id->id_vname = clone_str(v->name, "virtual device name");
+		    id->id_vname = clone_str(v->name, "virtual device name netlink");
 		    id->id_count++;
 
 		    q->ip_addr = ifp->addr;
@@ -1982,5 +2037,13 @@ const struct kernel_ops netkey_kernel_ops = {
      * if netlink  specific changes are needed.
      */
     remove_orphaned_holds: pfkey_remove_orphaned_holds,
+    .overlap_supported = FALSE
 };
 #endif /* linux && NETKEY_SUPPORT */
+
+/*
+ * Local Variables:
+ * c-basic-offset:4
+ * c-style: pluto
+ * End:
+ */
