@@ -2,11 +2,14 @@
  * common routines for interfaces that use pfkey to talk to kernel
  *
  * Copyright (C) 1997 Angelos D. Keromytis.
- * Copyright (C) 1998-2002  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2010  D. Hugh Redelmeier.
  * Copyright (C) 2003 Herbert Xu.
- * Copyright (C) 2006 Bart Trojanowski <bart@jukie.net>
+ * Copyright (C) 2006-2010 Bart Trojanowski <bart@jukie.net>
  * Copyright (C) 2003-2007  Michael Richardson <mcr@xelerance.com>
- * Copyright (C) 2007-2008  Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2007-2010  Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2009-2010 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2010 Henry N <henrynmail-oswan@yahoo.de>
+ * Copyright (C) 2010 Ajay.V.Sarraju
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -102,7 +105,6 @@ static sparse_names pfkey_type_names = {
 	NE(K_SADB_X_NAT_T_NEW_MAPPING),
 	NE(K_SADB_X_PLUMBIF),
 	NE(K_SADB_X_UNPLUMBIF),
-	NE(K_SADB_MAX),	
 	{ 0, sparse_end }
 };
 
@@ -142,7 +144,6 @@ static sparse_names pfkey_ext_names = {
 	NE(K_SADB_X_EXT_NAT_T_OA),
 	NE(K_SADB_X_EXT_PLUMBIF),
 	NE(K_SADB_X_EXT_SAREF),
-	NE(K_SADB_EXT_MAX),
 	{ 0, sparse_end }
 };
 #endif /* NEVER */
@@ -365,6 +366,10 @@ pfkey_get_response(pfkey_buf *buf, pfkey_seq_t seq)
     return FALSE;
 }
 
+/* Note ideally, this entire file should not be required for non-klips/mast
+ * and this ifdef can go. Or this function should be moved to kernel_klips.c
+ */
+#ifdef KLIPS
 /* Process a K_SADB_REGISTER message from the kernel.
  * This will be a response to one of ours, but it may be asynchronous
  * (if kernel modules are loaded and unloaded).
@@ -402,6 +407,7 @@ klips_pfkey_register_response(const struct sadb_msg *msg)
 	break;
     }
 }
+#endif
 
 /* Processs a K_SADB_ACQUIRE message from KLIPS.
  * Try to build an opportunistic connection!
@@ -671,7 +677,7 @@ finish_pfkey_msg(struct sadb_ext *extensions[K_SADB_EXT_MAX + 1]
 		  case ENOENT:
 		    loglog(RC_LOG_SERIOUS, "requested algorithm is not available in the kernel");
 		    success=FALSE;
-		    break;
+		    /* fall through to get error message */
 		    
 		  default:
 		  logerr:
@@ -796,12 +802,14 @@ netlink_register_proto(unsigned satype, const char *satypename)
 static int kernelop2klips(enum pluto_sadb_operations op)
 {
     int klips_op=0;
+    int klips_flags=0;
 
     /* translate sadb_operations -> KLIPS speak */
     switch (op)
     {
     case ERO_REPLACE:
-	klips_op = (K_SADB_X_ADDFLOW | (SADB_X_SAFLAGS_REPLACEFLOW << KLIPS_OP_FLAG_SHIFT));
+	klips_op = K_SADB_X_ADDFLOW;
+	klips_flags = SADB_X_SAFLAGS_REPLACEFLOW;
 	break;
 
     case ERO_ADD:
@@ -811,21 +819,31 @@ static int kernelop2klips(enum pluto_sadb_operations op)
     case ERO_DELETE:
 	klips_op = K_SADB_X_DELFLOW;
 	break;
-	    
+
     case ERO_ADD_INBOUND:
-	klips_op = (K_SADB_X_ADDFLOW | (SADB_X_SAFLAGS_INFLOW << KLIPS_OP_FLAG_SHIFT));
+	klips_op = K_SADB_X_ADDFLOW;
+	klips_flags = SADB_X_SAFLAGS_INFLOW;
 	break;
 
     case ERO_DEL_INBOUND:
-	klips_op = (K_SADB_X_DELFLOW | (SADB_X_SAFLAGS_INFLOW << KLIPS_OP_FLAG_SHIFT));
+	klips_op = K_SADB_X_DELFLOW;
+	klips_flags = SADB_X_SAFLAGS_INFLOW;
 	break;
 
     case ERO_REPLACE_INBOUND:
-	klips_op = (K_SADB_X_ADDFLOW | (SADB_X_SAFLAGS_REPLACEFLOW|SADB_X_SAFLAGS_INFLOW << KLIPS_OP_FLAG_SHIFT));
+	klips_op = K_SADB_X_ADDFLOW;
+	klips_flags = SADB_X_SAFLAGS_REPLACEFLOW | SADB_X_SAFLAGS_INFLOW;
 	break;
     }
 
-    return klips_op;
+#if defined(KLIPS_MAST)
+    /* In mast mode, we never want to set an eroute.
+     * Setting the POLICYONLY disables eroutes. */
+    if (kernel_ops->type == USE_MASTKLIPS)
+	klips_flags |= SADB_X_SAFLAGS_POLICYONLY;
+#endif
+
+    return klips_op | (klips_flags << KLIPS_OP_FLAG_SHIFT);
 }
 
 #ifdef KLIPS
@@ -907,6 +925,19 @@ pfkey_raw_eroute(const ip_address *this_host
 	    return FALSE;
 	}
     }
+#if defined(KLIPS_MAST)
+    /* in mast mode, deletes also include the extension flags */
+    else if (kernel_ops->type == USE_MASTKLIPS) {
+	if (!(pfkey_build(pfkey_sa_build(&extensions[K_SADB_EXT_SA]
+					 , K_SADB_EXT_SA
+					 , spi	/* in network order */
+					 , 0, 0, 0, 0, klips_op >> KLIPS_OP_FLAG_SHIFT)
+			  , "pfkey_sa del flow", text_said, extensions)))
+	{
+	    return FALSE;
+	}
+    }
+#endif
 
     if (!pfkeyext_address(K_SADB_X_EXT_ADDRESS_SRC_FLOW, &sflow_ska
 			  , "pfkey_addr_sflow", text_said, extensions))
@@ -1072,6 +1103,9 @@ bool pfkey_add_sa(struct kernel_sa *sa, bool replace)
 	    int error;
 
 	    error = pfkey_msg_parse(&pfb.msg, NULL, replies, EXT_BITS_IN);
+	    if (error) {
+		plog("success on unparsable message - cannot happen");
+	    }
 #ifdef KLIPS_MAST	    
 	    if(replies[K_SADB_X_EXT_SAREF]) {
 		    struct sadb_x_saref *sar = (struct sadb_x_saref *)replies[K_SADB_X_EXT_SAREF];
@@ -1410,6 +1444,15 @@ read_proto(const char * s, size_t * len, int * transport_proto)
 
     l = *len;
     p = memchr(s, ':', l);
+	if (p && memchr(p+1, ':', l - (p - s) - 1)) {
+		/* multiple ':'s means IPv6 address, so no port
+		   unless it's in []'s */
+		p = memchr(s, ']', l);
+		if (p && *(p+1) == ':')
+			p++;
+		else
+			p = NULL;
+	}
     if (p == 0) {
         *transport_proto = 0;
         return 0;
@@ -1560,14 +1603,14 @@ scan_proc_shunts(void)
             /* our client */
 
             context = "source subnet field malformed: ";
-            ugh = ttosubnet((char *)ff[0].ptr, ff[0].len, AF_INET, &eri.ours);
+            ugh = ttosubnet((char *)ff[0].ptr, ff[0].len, 0, &eri.ours);
             if (ugh != NULL)
                 break;
 
             /* his client */
 
             context = "destination subnet field malformed: ";
-            ugh = ttosubnet((char *)ff[2].ptr, ff[2].len, AF_INET, &eri.his);
+            ugh = ttosubnet((char *)ff[2].ptr, ff[2].len, 0, &eri.his);
             if (ugh != NULL)
                 break;
 
@@ -1611,8 +1654,6 @@ scan_proc_shunts(void)
                 if (bare_shunt_ptr(&eri.ours, &eri.his, eri.transport_proto) == NULL
                 && shunt_owner(&eri.ours, &eri.his) == NULL)
                 {
-                    int ourport = ntohs(portof(&eri.ours.addr));
-                    int hisport = ntohs(portof(&eri.his.addr));
                     char ourst[SUBNETTOT_BUF];
                     char hist[SUBNETTOT_BUF];
                     char sat[SATOT_BUF];
@@ -1622,6 +1663,8 @@ scan_proc_shunts(void)
                     satot(&eri.said, 0, sat, sizeof(sat));
 
                     DBG(DBG_CONTROL,
+                        int ourport = ntohs(portof(&eri.ours.addr));
+                        int hisport = ntohs(portof(&eri.his.addr));
                         DBG_log("add orphaned shunt %s:%d -> %s:%d => %s:%d"
                             , ourst, ourport, hist, hisport, sat, eri.transport_proto)
                      )
@@ -1790,11 +1833,13 @@ void pfkey_set_debug(int cur_debug
 		     , openswan_keying_debug_func_t debug_func
 		     , openswan_keying_debug_func_t error_func)
 {
+#ifdef DEBUG
     pfkey_lib_debug = (cur_debug&DBG_PFKEY ?
 		       PF_KEY_DEBUG_PARSE_MAX : PF_KEY_DEBUG_PARSE_NONE);
     
     pfkey_debug_func = debug_func;
     pfkey_error_func = error_func;
+#endif
 }
 
 void pfkey_remove_orphaned_holds(int transport_proto
