@@ -4,9 +4,10 @@
  *
  * Copyright (C) 1998-2001  D. Hugh Redelmeier.
  * Copyright (C) 2003-2008  Michael Richardson <mcr@xelerance.com>
- * Copyright (C) 2003-2009 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2003-2010 Paul Wouters <paul@xelerance.com>
  * Copyright (C) 2008 David McCullough <david_mccullough@securecomputing.com>
  * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
+ * Copyright (C) 2010 Tuomo Soini <tis@foobar.fi>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -129,6 +130,7 @@ static int print_secrets(struct secret *secret
     char idb2[IDTOA_BUF];
     const char *kind = "?";
     const char *more = "";
+    struct id_list *ids;
     
     switch(pks->kind) {
     case PPK_PSK: kind="PSK"; break;
@@ -139,7 +141,7 @@ static int print_secrets(struct secret *secret
 	return 1;
     }
 
-    struct id_list *ids = osw_get_idlist(secret);
+    ids = osw_get_idlist(secret);
     strcpy(idb1,"%any");
     strcpy(idb2,"");
 
@@ -482,12 +484,11 @@ RSA_check_signature_gen(struct state *st
 	int pathlen;
 
 	pp = &pluto_pubkeys;
-	pathlen = pathlen;      /* make sure it used even with !X509 */
 
 	{
-	  char buf[IDTOA_BUF];
 	  
 	  DBG(DBG_CONTROL,
+	      char buf[IDTOA_BUF];
 	      dntoa_or_null(buf, IDTOA_BUF, c->spd.that.ca, "%any");
 	      DBG_log("required CA is '%s'", buf));
 	}
@@ -502,13 +503,10 @@ RSA_check_signature_gen(struct state *st
 	    {
 		time_t tnow;
 
-		{
-		  char buf[IDTOA_BUF];
-		  
-		  DBG(DBG_CONTROL,
-		      dntoa_or_null(buf, IDTOA_BUF, key->issuer, "%any");
-		      DBG_log("key issuer CA is '%s'", buf));
-		}
+		DBG(DBG_CONTROL,
+		    char buf[IDTOA_BUF];
+		    dntoa_or_null(buf, IDTOA_BUF, key->issuer, "%any");
+		    DBG_log("key issuer CA is '%s'", buf));
 
 		/* check if found public key has expired */
 		time(&tnow);
@@ -659,8 +657,8 @@ osw_get_secret(const struct connection *c
 	free_public_key(my_public_key);
 	return best;
     }
-#if defined(AGGRESSIVE)
-    if (his_id_was_instantiated(c) && !(c->policy & POLICY_AGGRESSIVE))
+#if defined(AGGRESSIVE) 
+    if (his_id_was_instantiated(c) && (!(c->policy & POLICY_AGGRESSIVE)) && isanyaddr(&c->spd.that.host_addr) )
     {
 	DBG(DBG_CONTROL,
 	    DBG_log("instantiating him to 0.0.0.0"));
@@ -679,7 +677,11 @@ osw_get_secret(const struct connection *c
 	      && (((c->kind == CK_TEMPLATE)
 		   && (c->spd.that.id.kind == ID_NONE))
 		 || ((c->kind == CK_INSTANCE)
-		     && (id_is_ipaddr(&c->spd.that.id)))))
+		     && (id_is_ipaddr(&c->spd.that.id))
+		     /* Check if we are a road warrior instantiation, not a vnet: instantiation */
+		     && (isanyaddr(&c->spd.that.host_addr)))
+		 )
+	    )
     {
 	DBG(DBG_CONTROL,
 	    DBG_log("replace him to 0.0.0.0"));
@@ -927,13 +929,16 @@ add_public_key(const struct id *id
 /*
  *  list all public keys in the chained list
  */
-void list_public_keys(bool utc)
+void list_public_keys(bool utc, bool check_pub_keys)
 {
     struct pubkey_list *p = pluto_pubkeys;
 
-    whack_log(RC_COMMENT, " ");
-    whack_log(RC_COMMENT, "List of Public Keys:");
-    whack_log(RC_COMMENT, " ");
+    if(!check_pub_keys)
+    {
+	whack_log(RC_COMMENT, " ");
+	whack_log(RC_COMMENT, "List of Public Keys:");
+	whack_log(RC_COMMENT, " ");
+    }
 
     while (p != NULL)
     {
@@ -944,27 +949,35 @@ void list_public_keys(bool utc)
 	    char id_buf[IDTOA_BUF];
 	    char expires_buf[TIMETOA_BUF];
 	    char installed_buf[TIMETOA_BUF];
+	    char *check_expiry_msg = NULL;
 
-	    idtoa(&key->id, id_buf, IDTOA_BUF);
-	    whack_log(RC_COMMENT, "%s, %4d RSA Key %s (%s private key), until %s %s"
-		      , timetoa(&key->installed_time, utc,
-				installed_buf, sizeof(installed_buf))
-		      , 8*key->u.rsa.k
-		      , key->u.rsa.keyid
-		      , (has_private_rawkey(key) ? "has" : "no")
-		      , timetoa(&key->until_time, utc,
-				expires_buf, sizeof(expires_buf))
-		      , check_expiry(key->until_time
-				     , PUBKEY_WARNING_INTERVAL
-				     , TRUE));
+	    check_expiry_msg = check_expiry(key->until_time
+					    , PUBKEY_WARNING_INTERVAL
+					    , TRUE);
 
-	    whack_log(RC_COMMENT,"       %s '%s'",
-		enum_show(&ident_names, key->id.kind), id_buf);
-
-	    if (key->issuer.len > 0)
+	    if(!check_pub_keys || (check_pub_keys && strncmp(check_expiry_msg, "ok", 2)))
 	    {
-		dntoa(id_buf, IDTOA_BUF, key->issuer);
-		whack_log(RC_COMMENT,"       Issuer '%s'", id_buf);
+		idtoa(&key->id, id_buf, IDTOA_BUF);
+		whack_log(RC_COMMENT, "%s, %4d RSA Key %s (%s private key), until %s %s"
+			  , timetoa(&key->installed_time, utc,
+				    installed_buf, sizeof(installed_buf))
+			  , 8*key->u.rsa.k
+			  , key->u.rsa.keyid
+			  , (has_private_rawkey(key) ? "has" : "no")
+			  , timetoa(&key->until_time, utc,
+				    expires_buf, sizeof(expires_buf))
+			  , check_expiry(key->until_time
+					 , PUBKEY_WARNING_INTERVAL
+					 , TRUE));
+
+		whack_log(RC_COMMENT,"       %s '%s'",
+			  enum_show(&ident_names, key->id.kind), id_buf);
+
+		if (key->issuer.len > 0)
+		{
+		    dntoa(id_buf, IDTOA_BUF, key->issuer);
+		    whack_log(RC_COMMENT,"       Issuer '%s'", id_buf);
+		}
 	    }
 	}
 	p = p->next;
