@@ -1,5 +1,13 @@
 /* Openswan NAT-Traversal
  * Copyright (C) 2002-2003 Mathieu Lafon - Arkoon Network Security
+ * Copyright (C) 2005-2007 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2005 Ken Bantoft <ken@xelerance.com>
+ * Copyright (C) 2006 Bart Trojanowski <bart@jukie.net>
+ * Copyright (C) 2007-2010 Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2009 Tuomo Soini <tis@foobar.fi>
+ * Copyright (C) 2009 Gilles Espinasse <g.esp@free.fr>
+ * Copyright (C) 2009 David McCullough <david_mccullough@securecomputing.com>
+ * Copyright (C) 2011 Shinichi Furuso <Shinichi.Furuso@jp.sony.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -668,53 +676,67 @@ void nat_traversal_show_result (u_int32_t nt, u_int16_t sport)
 
 int nat_traversal_espinudp_socket (int sk, const char *fam, u_int32_t type)
 {
-	int r = 0;
-	static enum { auto_style, new_style, old_style } style = auto_style;
+	int r = -1;
+	struct ifreq ifr;
+	int *fdp = (int *) &ifr.ifr_data;
 
-	if (style == auto_style || style == new_style) {
-		struct ifreq ifr;
-		int *fdp = (int *) &ifr.ifr_data;
-
-		if (style == auto_style)
-			loglog(RC_LOG_SERIOUS, "NAT-Traversal: Trying new style NAT-T");
-
-		memset(&ifr, 0, sizeof(ifr));
-		strcpy(ifr.ifr_name, "ipsec0");
-		fdp[0] = sk;
-		fdp[1] = type;
-		r = ioctl(sk, IPSEC_UDP_ENCAP_CONVERT, &ifr);
-		if (r == -1)
-			loglog(RC_LOG_SERIOUS,
-				   "NAT-Traversal: ESPINUDP(%d) setup failed for "
-				   "new style NAT-T family %s (errno=%d)"
-				   , type, fam, errno);
-		else
-			style = new_style;
+	DBG(DBG_NATT, DBG_log("NAT-Traversal: Trying new style NAT-T"));
+	memset(&ifr, 0, sizeof(ifr));
+	switch(kern_interface) {
+		case USE_MASTKLIPS:
+			strcpy(ifr.ifr_name, "ipsec0"); /* using mast0 will break it! */
+			break;
+		case USE_KLIPS:
+			strcpy(ifr.ifr_name, "ipsec0");
+			break;
+		case USE_NETKEY:
+			/* Let's hope we have at least one ethernet device */
+			strcpy(ifr.ifr_name, "eth0");
+			break;
+		case USE_BSDKAME:
+			/* Let's hope we have at least one ethernet device */
+			strcpy(ifr.ifr_name, "en0");
+			break;
+		default:
+			/* We have nothing , really prob just abort and return -1 */
+			strcpy(ifr.ifr_name, "eth0");
+			break;
+	}
+	fdp[0] = sk;
+	fdp[1] = type;
+	r = ioctl(sk, IPSEC_UDP_ENCAP_CONVERT, &ifr);
+	if (r == -1) { 
+		DBG(DBG_NATT, DBG_log("NAT-Traversal: ESPINUDP(%d) setup failed for "
+			   "new style NAT-T family %s (errno=%d)"
+			   , type, fam, errno));
+	} else {
+		DBG(DBG_NATT, DBG_log("NAT-Traversal: ESPINUDP(%d) setup succeeded for "
+			   "new style NAT-T family %s" , type, fam));
+		return r;
 	}
 
-	if (style == auto_style || style == old_style) {
-
-		if (style == auto_style)
-			loglog(RC_LOG_SERIOUS, "NAT-Traversal: Trying old style NAT-T");
-
-		r = setsockopt(sk, SOL_UDP, UDP_ESPINUDP, &type, sizeof(type));
-		if (r == -1)
-			loglog(RC_LOG_SERIOUS,
-				   "NAT-Traversal: ESPINUDP(%d) setup failed for "
-				   "old style NAT-T family %s (errno=%d)"
-				   , type, fam, errno);
-		else
-			style = old_style;
+#if defined(KLIPS)
+	DBG(DBG_NATT, DBG_log("NAT-Traversal: Trying old style NAT-T"));
+	r = setsockopt(sk, SOL_UDP, UDP_ESPINUDP, &type, sizeof(type));
+	if (r == -1) {
+		DBG(DBG_NATT, DBG_log("NAT-Traversal: ESPINUDP(%d) setup failed for "
+			   "old style NAT-T family %s (errno=%d)"
+			   , type, fam, errno));
 	}
-
-	if (r == -1 && (errno == ENOPROTOOPT)) {
-		loglog(RC_LOG_SERIOUS,
-		       "NAT-Traversal: ESPINUDP(%d) not supported by kernel for family %s"
-		       , type, fam);
-		disable_nat_traversal(type);
+	else {
+		DBG(DBG_NATT, DBG_log("NAT-Traversal: ESPINUDP(%d) setup succeeded for "
+			   "new style NAT-T family %s" , type, fam));
+		return r;
 	}
+# else
+	DBG(DBG_NATT, DBG_log("NAT-Traversal: ESPINUDP() setup for old style NAT-T family not available - KLIPS support not compiled in"));
+# endif
 
-	return r;
+	loglog(RC_LOG_SERIOUS,
+	       "NAT-Traversal: ESPINUDP(%d) not supported by kernel for family %s"
+	       , type, fam);
+	disable_nat_traversal(type);
+	return -1;
 }
 
 void nat_traversal_new_ka_event (void)
@@ -856,8 +878,8 @@ static void nat_traversal_find_new_mapp_state (struct state *st, void *data)
 {
 	struct _new_mapp_nfo *nfo = (struct _new_mapp_nfo *)data;
 
-	if((!nfo->st->st_clonedfrom &&
-	    (st->st_serialno == nfo->st->st_clonedfrom &&
+	if((nfo->st->st_clonedfrom &&
+	    (st->st_serialno == nfo->st->st_clonedfrom ||
 	     st->st_clonedfrom == nfo->st->st_clonedfrom)) ||
 	   st->st_serialno == nfo->st->st_serialno) {
 		char b1[ADDRTOT_BUF];
@@ -990,14 +1012,16 @@ void nat_traversal_change_port_lookup(struct msg_digest *md, struct state *st)
 	      && st->st_localport == st->st_interface->port))
 	    
 	{
-	    char b1[ADDRTOT_BUF], b2[ADDRTOT_BUF];
 
 	    DBG(DBG_NATT,
+	        char b1[ADDRTOT_BUF];
+	        char b2[ADDRTOT_BUF];
 		DBG_log("NAT-T connection has wrong interface definition %s:%u vs %s:%u"
 			, (addrtot(&st->st_localaddr, 0, b1, sizeof(b1)),b1)
 			, st->st_localport
 			, (addrtot(&st->st_interface->ip_addr, 0, b2, sizeof(b2)),b2)
-			, st->st_interface->port));
+			, st->st_interface->port)
+	    );
 
 	    for (i = interfaces; i !=  NULL; i = i->next) {
 		if ((sameaddr(&st->st_localaddr, &i->ip_addr))
@@ -1059,10 +1083,6 @@ void process_pfkey_nat_t_new_mapping(
 		ugh = "only AF_INET supported";
 	}
 	else {
-		char text_said[SATOT_BUF];
-		char _srca[ADDRTOT_BUF], _dsta[ADDRTOT_BUF];
-		ip_said said;
-
 		initaddr((const void *) &((const struct sockaddr_in *)srca)->sin_addr,
 			sizeof(((const struct sockaddr_in *)srca)->sin_addr),
 			srca->sa_family, &(nfo.src));
@@ -1073,6 +1093,11 @@ void process_pfkey_nat_t_new_mapping(
 		nfo.dport = ntohs(((const struct sockaddr_in *)dsta)->sin_port);
 
 		DBG(DBG_NATT,
+			char text_said[SATOT_BUF];
+			char _srca[ADDRTOT_BUF];
+			char _dsta[ADDRTOT_BUF];
+			ip_said said;
+
 			initsaid(&nfo.src, nfo.sa->sadb_sa_spi, SA_ESP, &said);
 			satot(&said, 0, text_said, SATOT_BUF);
 			addrtot(&nfo.src, 0, _srca, ADDRTOT_BUF);
