@@ -264,7 +264,7 @@ static inline void pcr_init(struct pluto_crypto_req *r
 	, enum pluto_crypto_requests pcr_type
 	, enum crypto_importance pcr_pcim)
 {
-    memset(r, 0, sizeof(r));
+    memset(r, 0, sizeof(*r));
     r->pcr_len  = sizeof(struct pluto_crypto_req);
     r->pcr_type = pcr_type;
     r->pcr_pcim = pcr_pcim;
