@@ -2,10 +2,11 @@
  * Copyright (C) 1997      Angelos D. Keromytis.
  * Copyright (C) 1998-2001 D. Hugh Redelmeier.
  * Copyright (C) 2003-2008 Michael C Richardson <mcr@xelerance.com> 
- * Copyright (C) 2003-2009 Paul Wouters <paul@xelerance.com> 
+ * Copyright (C) 2003-2010 Paul Wouters <paul@xelerance.com> 
  * Copyright (C) 2007 Ken Bantoft <ken@xelerance.com>
  * Copyright (C) 2008-2009 David McCullough <david_mccullough@securecomputing.com>
  * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
+ * Copyright (C) 2009-2010 Tuomo Soini <tis@foobar.fi>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -34,6 +35,7 @@
 #include <sys/un.h>
 #include <fcntl.h>
 #include <getopt.h>
+#include <netinet/in.h>
 #include <resolv.h>
 
 #include <openswan.h>
@@ -110,8 +112,11 @@
 #endif
 
 const char *ctlbase = "/var/run/pluto";
+char *pluto_listen = NULL;
 
+#ifdef DEBUG
 openswan_passert_fail_t openswan_passert_fail = passert_fail;
+#endif
 
 /** usage - print help messages
  *
@@ -145,6 +150,7 @@ usage(const char *mess)
 	    " \\\n\t"
 	    "[--interface <ifname|ifaddr>]"
 	    " [--ikeport <port-number>]"
+	    "[--listen <ifaddr>]"
 	    " \\\n\t"
 	    "[--ctlbase <path>]"
 	    " \\\n\t"
@@ -333,7 +339,9 @@ main(int argc, char **argv)
 
     global_argv = argv;
     global_argc = argc;
+#ifdef DEBUG
     openswan_passert_fail = passert_fail;
+#endif
 
     /* see if there is an environment variable */
     coredir = getenv("PLUTO_CORE_DIR");
@@ -372,6 +380,7 @@ main(int argc, char **argv)
 	    { "use-mastklips",   no_argument, NULL, 'M' },
 	    { "use-bsdkame",   no_argument, NULL, 'F' },
 	    { "interface", required_argument, NULL, 'i' },
+	    { "listen", required_argument, NULL, 'L' },
 	    { "ikeport", required_argument, NULL, 'p' },
 	    { "ctlbase", required_argument, NULL, 'b' },
 	    { "secretsfile", required_argument, NULL, 's' },
@@ -424,6 +433,8 @@ main(int argc, char **argv)
 	    { "impair-delay-adns-txt-answer", no_argument, NULL, IMPAIR_DELAY_ADNS_TXT_ANSWER + DBG_OFFSET },
 	    { "impair-bust-mi2", no_argument, NULL, IMPAIR_BUST_MI2 + DBG_OFFSET },
 	    { "impair-bust-mr2", no_argument, NULL, IMPAIR_BUST_MR2 + DBG_OFFSET },
+	    { "impair-sa-creation", no_argument, NULL, IMPAIR_SA_CREATION + DBG_OFFSET },
+	    { "impair-die-oninfo", no_argument, NULL, IMPAIR_DIE_ONINFO + DBG_OFFSET },
 	    { "impair-jacob-two-two", no_argument, NULL, IMPAIR_JACOB_TWO_TWO + DBG_OFFSET },
 #endif
 	    { 0,0,0,0 }
@@ -504,6 +515,19 @@ main(int argc, char **argv)
 	    kern_interface = USE_KLIPS;
 	    continue;
 
+	case 'L':	/* --listen ip_addr */
+	    {
+	    ip_address lip;
+	     err_t e = ttoaddr(optarg,0,0,&lip);
+	    if(e) {
+		openswan_log("invalid listen argument ignored: %s\n",e);
+	    } else {
+		pluto_listen = clone_str(optarg, "pluto_listen");
+		openswan_log("bind() will be filtered for %s\n",pluto_listen);
+	    }
+            }
+	   continue;
+
 	case 'M':       /* --use-mast */
 	    kern_interface = USE_MASTKLIPS;
 	    continue;
@@ -568,10 +592,14 @@ main(int argc, char **argv)
 		usage("too many --interface specifications");
 	    continue;
 
-	case 'p':	/* --port <portnumber> */
+	/* 
+	 * This option does not really work, as this is the "left"
+	 * site only, you also need --to --ikeport again later on
+	 * It will result in: yourport -> 500, still not bypassing filters
+	 */
+	case 'p':	/* --ikeport <portnumber> */
 	    if (optarg == NULL || !isdigit(optarg[0]))
 		usage("missing port number");
-
 	    {
 		char *endptr;
 		long port = strtol(optarg, &endptr, 0);
@@ -639,9 +667,11 @@ main(int argc, char **argv)
 	case '4':	/* --disable_port_floating */
 	    nat_t_spf = FALSE;
 	    continue;
+#ifdef DEBUG
 	case '5':	/* --debug-nat_t */
 	    base_debugging |= DBG_NATT;
 	    continue;
+#endif
 #endif
 	case '6':	/* --virtual_private */
 	    virtual_private = optarg;
@@ -664,6 +694,11 @@ main(int argc, char **argv)
 	usage("unexpected argument");
     reset_debugging();
 
+#ifdef HAVE_NO_FORK
+	fork_desired = FALSE;
+	nhelpers = 0;
+#endif
+
     /* if a core dir was set, chdir there */
     if(coredir) 
 	if(chdir(coredir) == -1) {
@@ -889,6 +924,26 @@ main(int argc, char **argv)
 	openswan_log("core dump dir: %s", coredir);
     }
 
+#ifdef LEAK_DETECTIVE
+	openswan_log("LEAK_DETECTIVE support [enabled]");
+#else
+	openswan_log("LEAK_DETECTIVE support [disabled]");
+#endif
+
+#ifdef HAVE_OCF
+       {
+        struct stat buf;
+	errno=0;
+
+	if( stat("/dev/crypto",&buf) != -1) 
+		openswan_log("OCF support for IKE via /dev/crypto [enabled]");
+	else 
+		openswan_log("OCF support for IKE via /dev/crypto [failed:%s]", strerror(errno));
+       }
+#else
+	openswan_log("OCF support for IKE [disabled]");
+#endif
+
    /* Check for SAREF support */
 #ifdef KLIPS_MAST
 #include <ipsec_saref.h>
@@ -916,8 +971,39 @@ main(int argc, char **argv)
 
 
 	close(sk);
-}
+    }
 #endif
+
+#ifdef HAVE_LIBNSS
+	openswan_log("NSS support [enabled]");
+#else
+	openswan_log("NSS support [disabled]");
+#endif
+
+#ifdef HAVE_STATSD
+	openswan_log("HAVE_STATSD notification via /bin/openswan-statsd enabled");
+#else
+	openswan_log("HAVE_STATSD notification support not compiled in");
+#endif
+
+
+/** Log various impair-* functions if they were enabled */
+
+    if(DBGP(IMPAIR_BUST_MI2))
+	openswan_log("Warning: IMPAIR_BUST_MI2 enabled");
+    if(DBGP(IMPAIR_BUST_MR2))
+	openswan_log("Warning: IMPAIR_BUST_MR2 enabled");
+    if(DBGP(IMPAIR_SA_CREATION))
+	openswan_log("Warning: IMPAIR_SA_CREATION enabled");
+    if(DBGP(IMPAIR_JACOB_TWO_TWO))
+	openswan_log("Warning: IMPAIR_JACOB_TWO_TWO enabled");
+    if(DBGP(IMPAIR_DIE_ONINFO))
+	openswan_log("Warning: IMPAIR_DIE_ONINFO enabled");
+    if(DBGP(IMPAIR_DELAY_ADNS_KEY_ANSWER))
+	openswan_log("Warning: IMPAIR_DELAY_ADNS_KEY_ANSWER enabled");
+    if(DBGP(IMPAIR_DELAY_ADNS_TXT_ANSWER))
+	openswan_log("Warning: IMPAIR_DELAY_ADNS_TXT_ANSWER enabled");
+
 /** Initialize all of the various features */
 
 #ifdef NAT_TRAVERSAL
@@ -1004,6 +1090,10 @@ exit_pluto(int status)
     free_acerts();             /* free chain of X.509 attribute certificates */
     free_ocsp();               /* free ocsp cache */
 
+    osw_conf_free_oco();	/* free global_oco containing path names */
+
+    free_myFQDN();	    /* free myid FQDN */
+
     free_ifaces();          /* free interface list from memory */
     stop_adns();            /* Stop async DNS process (if running) */
     free_md_pool();         /* free the md pool */
@@ -1011,6 +1101,9 @@ exit_pluto(int status)
     NSS_Shutdown();
 #endif
     delete_lock();          /* delete any lock files */
+#ifdef LEAK_DETECTIVE
+    report_leaks();         /* report memory leaks now, after all free()s */
+#endif /* LEAK_DETECTIVE */
     close_log();            /* close the logfiles */
     exit(status);           /* exit, with our error code */
 }
