@@ -1,3 +1,4 @@
+
 /* whack communicating routines
  * Copyright (C) 1997 Angelos D. Keromytis.
  * Copyright (C) 1998-2001  D. Hugh Redelmeier.
@@ -451,7 +452,7 @@ void whack_process(int whackfd, struct whack_message msg)
 
     if (msg.whack_list & LIST_PUBKEYS)
     {
-	list_public_keys(msg.whack_utc);
+	list_public_keys(msg.whack_utc, msg.whack_check_pub_keys);
     }
 
     if (msg.whack_reread & REREAD_CACERTS)
@@ -710,7 +711,7 @@ whack_handle(int whackctlfd)
 	{
 	    if (msg.magic == WHACK_BASIC_MAGIC)
 	    {
-		/* Only basic commands.  Simpler inter-version compatability. */
+		/* Only basic commands.  Simpler inter-version compatibility. */
 		if (msg.whack_status)
 		    show_status();
 
