@@ -143,10 +143,10 @@ out_sa(pb_stream *outs
 
 	if (revised_sadb && ((st->st_policy) & POLICY_COMPRESS)) {
 
-	    struct db_trans *ipcomp_trans = malloc (sizeof (struct db_trans));
+	    struct db_trans *ipcomp_trans = alloc_thing(struct db_trans, "ipcomp_trans");
 
 	    /* allocate space for 2 proposals */
-	    struct db_prop *ipcomp_prop = malloc (sizeof (struct db_prop) * 2);
+	    struct db_prop *ipcomp_prop = alloc_bytes( (sizeof (struct db_prop) * 2), "ipcomp_prop");
 
 	    if (ipcomp_trans && ipcomp_prop) {
 
@@ -176,11 +176,11 @@ out_sa(pb_stream *outs
 
 	    }
 	    else {
-		/* couldn't malloc something, so skip adding the proposal */
+		/* couldn't alloc something, so skip adding the proposal */
 		if (ipcomp_trans)
-		    free (ipcomp_trans);
+		    pfreeany (ipcomp_trans);
 		if (ipcomp_prop)
-		    free (ipcomp_prop);
+		    pfreeany (ipcomp_prop);
 	    }
 	}
     }
@@ -222,22 +222,7 @@ out_sa(pb_stream *outs
 	int valid_prop_cnt;
 	
 	pc = &sadb->prop_conjs[pcn];
-
-	/*
-	 * figure out how many proposals we are going to make,
-	 * so we'll know when we are finished! This repeats some calculations
-	 * but this seems like the best method to avoid empty proposals.
-	 */
-	valid_prop_cnt = 0;
-	for (pn = 0; pn < pc->prop_cnt; pn++)
-	{
-	    struct db_prop *p;
-
-	    p = &pc->props[pn];
-
-	    valid_prop_cnt++;
-	}
-	    
+	valid_prop_cnt = pc->prop_cnt;
 	DBG(DBG_EMITTING, 
 	    DBG_log("out_sa pcn: %d has %d valid proposals",
 		    pcn, valid_prop_cnt));
@@ -571,8 +556,6 @@ lset_t preparse_isakmp_sa_body(pb_stream *sa_pbs)
     struct isakmp_proposal proposal;
     pb_stream trans_pbs;
     struct isakmp_transform trans;
-    u_char *attr_start;
-    size_t attr_len;
     struct isakmp_attribute a;
     pb_stream attr_pbs;
     u_int32_t ipsecdoisit;
@@ -602,9 +585,6 @@ lset_t preparse_isakmp_sa_body(pb_stream *sa_pbs)
 	    &trans_pbs))
 	    return LEMPTY;
 
-	attr_start = trans_pbs.cur;
-	attr_len = pbs_left(&trans_pbs);
-
 	while (pbs_left(&trans_pbs) != 0) {
 	    if (!in_struct(&a, &isakmp_oakley_attribute_desc, &trans_pbs,
 		&attr_pbs))
@@ -802,6 +782,9 @@ parse_isakmp_sa_body(
 	u_int16_t life_type;
 	struct trans_attrs ta;
 	err_t ugh = NULL;	/* set to diagnostic when problem detected */
+#ifdef IKE_ALG
+	char ugh_buf[256];	/* room for building a diagnostic */
+#endif
 	zero(&ta);
 
 	life_type = 0;
@@ -880,7 +863,7 @@ parse_isakmp_sa_body(
 	    {
 		case OAKLEY_ENCRYPTION_ALGORITHM | ISAKMP_ATTR_AF_TV:
 #ifdef IKE_ALG
-		    if (ike_alg_enc_ok(val, 0, c->alg_info_ike, &ugh)) {
+		    if (ike_alg_enc_ok(val, 0, c->alg_info_ike, &ugh, ugh_buf, sizeof(ugh_buf))) {
 		    /* if (ike_alg_enc_present(val)) { */
 			ta.encrypt = val;
 			ta.encrypter = crypto_get_encrypter(val);
@@ -1139,7 +1122,7 @@ parse_isakmp_sa_body(
 		     * check if this keylen is compatible with 
 		     * specified alg_info_ike
 		     */
-		    if (!ike_alg_enc_ok(ta.encrypt, val, c->alg_info_ike, &ugh)) {
+		    if (!ike_alg_enc_ok(ta.encrypt, val, c->alg_info_ike, NULL, ugh_buf, sizeof(ugh_buf))) {
 			ugh = "peer proposed key_len not valid for encrypt algo setup specified";
 		    }
 		    ta.enckeylen=val;
