@@ -1346,11 +1346,13 @@ void fmt_state(struct state *st, const time_t n
 	}
 	if (st->st_esp.present)
 	{
+#if defined(linux) && defined(NETKEY_SUPPORT)
 	    time_t ago;
-
+#endif
 	    add_said(&c->spd.that.host_addr, st->st_esp.attrs.spi, SA_ESP);
 /* needs proper fix, via kernel_ops? */
 #if defined(linux) && defined(NETKEY_SUPPORT)
+
 	    if (get_sa_info(st, FALSE, &ago))
 	    {
 		snprintf(state_buf2, state_buf2_len,
@@ -1462,9 +1464,9 @@ show_states_status(void)
 	  st = array[i];
 	  fmt_state(st, n, state_buf, sizeof(state_buf)
 		, state_buf2, sizeof(state_buf2));
-	  whack_log(RC_COMMENT, state_buf);
+	  whack_log(RC_COMMENT, "%s", state_buf);
 	  if (state_buf2[0] != '\0')
-		whack_log(RC_COMMENT, state_buf2);
+		whack_log(RC_COMMENT, "%s", state_buf2);
 
 	  /* show any associated pending Phase 2s */
 	  if (IS_PHASE1(st->st_state))
