@@ -192,10 +192,10 @@ static struct vid_struct _vid_tab[] = {
 		"SSH Communications Security IPSEC Express version 4.2.0")
 
 
-	/* note: md5('CISCO-UNITY') = 12f5f28c457168a9702d9fe274cc02d4 */
-	{ VID_CISCO_UNITY, VID_KEEP, NULL, "Cisco-Unity",
-		"\x12\xf5\xf2\x8c\x45\x71\x68\xa9\x70\x2d\x9f\xe2\x74\xcc\x01\x00",
-		16 },
+	/* note: It's not md5/sha1? 'IKE Fragmentation' = 4048b7d56ebce88525e7de7f00d6c2d3c0000000 */
+	{ VID_CISCO_IKE_FRAGMENTATION, VID_KEEP, NULL, "Cisco IKE Fragmentation",
+		"\x40\x48\xb7\xd5\x6e\xbc\xe8\x85\x25\xe7\xde\x7f\x00\xd6\xc2\xd3\xc0\x00\x00\x00",
+		20 },
 
 	{ VID_CISCO3K, VID_KEEP | VID_SUBSTRING_MATCH, 
           NULL, "Cisco VPN 3000 Series" , "\x1f\x07\xf7\x0e\xaa\x65\x14\xd3\xb0\xfa\x96\x54\x2a\x50", 14},
@@ -203,6 +203,10 @@ static struct vid_struct _vid_tab[] = {
 	{ VID_CISCO_IOS, VID_KEEP | VID_SUBSTRING_MATCH, 
 	  NULL, "Cisco IOS Device", "\x3e\x98\x40\x48", 4},
 
+	/* note: md5('CISCO-UNITY') = 12f5f28c457168a9702d9fe274cc02d4 */
+	{ VID_CISCO_UNITY, VID_KEEP, NULL, "Cisco-Unity",
+		"\x12\xf5\xf2\x8c\x45\x71\x68\xa9\x70\x2d\x9f\xe2\x74\xcc\x01\x00",
+		16 },
 
 	/*
 	 * Timestep VID seen:
@@ -372,9 +376,13 @@ static const char _hexdig[] = "0123456789abcdef";
 
 static int _vid_struct_init = 0;
 
-/** 
+/* 
  * Setup VendorID structs, and populate them
- *
+ * FIXME: This functions leaks a little bit, but these are one time leaks:
+ * leak: 3 * vid->data, item size: 6
+ * leak: self-vendor ID, item size: 37
+ * leak: init_pluto_vendorid, item size: 13
+ * leak: 2 * vid->data, item size: 13
  */
 void init_vendorid(void)
 {
@@ -385,7 +393,7 @@ void init_vendorid(void)
 	for (vid = _vid_tab; vid->id; vid++) {
 	    if(vid->flags & VID_SELF) {
 		char *d;
-		vid->vid = strdup(init_pluto_vendorid());
+		vid->vid = clone_str(init_pluto_vendorid(),"init_pluto_vendorid");
 		vid->vid_len = strlen(vid->vid);
 		d = alloc_bytes(strlen(vid->descr)+4
 				+strlen(ipsec_version_code())
@@ -398,12 +406,12 @@ void init_vendorid(void)
 	    }
 	    else if (vid->flags & VID_STRING) {
 		/** VendorID is a string **/
-		vid->vid = strdup(vid->data);
+		vid->vid = clone_str(vid->data,"vid->data");
 		vid->vid_len = strlen(vid->data);
 	    }
 	    else if (vid->flags & VID_MD5HASH) {
 		/** VendorID is a string to hash with MD5 **/
-		unsigned char *vidm =  malloc(MD5_DIGEST_SIZE);
+		unsigned char *vidm =  alloc_bytes(MD5_DIGEST_SIZE,"VendorID MD5");
 		vid->vid = (char *)vidm;
 		if (vidm) {
 		    unsigned const char *d = (unsigned const char *)vid->data;
@@ -417,7 +425,7 @@ void init_vendorid(void)
 		/** FreeS/WAN 2.00+ specific hash **/
 #define FSWAN_VID_SIZE 12
 		unsigned char hash[MD5_DIGEST_SIZE];
-		char *vidm =  malloc(FSWAN_VID_SIZE);
+		char *vidm =  alloc_bytes(FSWAN_VID_SIZE,"fswan VID");
 		vid->vid = vidm;
 		if (vidm) {
 		    osMD5Init(&ctx);
