@@ -111,6 +111,8 @@ help(void)
 	    " [--tunnel]"
 	    " [--pfs]"
 	    " \\\n   "
+	    " [--pfsgroup [modp1024] | [modp1536] | [modp2048] | [modp3072] | [modp4096] | [modp6144] | [modp8192]]"
+	    " \\\n   "
 	    " [--ikelifetime <seconds>]"
 	    " [--ipseclifetime <seconds>]"
 	    " \\\n   "
@@ -223,6 +225,7 @@ help(void)
 	    " (--listen | --unlisten)"
 	    "\n\n"
 	"list: whack [--utc]"
+	    " [--checkpubkeys]"
 	    " [--listpubkeys]"
 	    " [--listcerts]"
 	    " [--listcacerts]"
@@ -394,6 +397,7 @@ enum option_enums {
 
 #   define LST_FIRST LST_UTC   /* first list option */
     LST_UTC,
+    LST_CHECKPUBKEYS,
     LST_PUBKEYS,
     LST_CERTS,
     LST_CACERTS,
@@ -473,6 +477,7 @@ enum option_enums {
     CD_MODECFGWINS1,
     CD_MODECFGWINS2,
     CD_METRIC,
+    CD_CONNMTU,
     CD_TUNNELIPV4,
     CD_TUNNELIPV6,
     CD_CONNIPV4,
@@ -600,6 +605,7 @@ static const struct option long_opts[] = {
     /* list options */
 
     { "utc", no_argument, NULL, LST_UTC + OO },
+    { "checkpubkeys", no_argument, NULL, LST_CHECKPUBKEYS + OO },
     { "listpubkeys", no_argument, NULL, LST_PUBKEYS + OO },
     { "listcerts", no_argument, NULL, LST_CERTS + OO },
     { "listcacerts", no_argument, NULL, LST_CACERTS + OO },
@@ -689,6 +695,7 @@ static const struct option long_opts[] = {
 #endif
 #endif
     { "metric", required_argument, NULL, CD_METRIC + OO + NUMERIC_ARG },
+    { "mtu", required_argument, NULL, CD_CONNMTU + OO + NUMERIC_ARG },
     { "sendcert", required_argument, NULL, END_SENDCERT + OO },
     { "certtype", required_argument, NULL, END_CERTTYPE + OO + NUMERIC_ARG },
     { "ipv4", no_argument, NULL, CD_CONNIPV4 + OO },
@@ -710,7 +717,7 @@ static const struct option long_opts[] = {
 #endif
 #ifdef DEBUG
     { "debug-none", no_argument, NULL, DBGOPT_NONE + OO },
-    { "debug-all]", no_argument, NULL, DBGOPT_ALL + OO },
+    { "debug-all", no_argument, NULL, DBGOPT_ALL + OO },
     { "debug-raw", no_argument, NULL, DBGOPT_RAW + OO },
     { "debug-crypt", no_argument, NULL, DBGOPT_CRYPT + OO },
     { "debug-parsing", no_argument, NULL, DBGOPT_PARSING + OO },
@@ -1221,6 +1228,11 @@ main(int argc, char **argv)
             msg.whack_list |= LELEM(c - LST_PUBKEYS);
             continue;
 
+	case LST_CHECKPUBKEYS:   /* --checkpubkeys */
+	    msg.whack_list |= LELEM(LST_PUBKEYS - LST_PUBKEYS);
+	    msg.whack_check_pub_keys = TRUE;
+	    continue;
+
         case LST_ALL:   /* --listall */
             msg.whack_list = LIST_ALL;
             continue;
@@ -1666,6 +1678,10 @@ main(int argc, char **argv)
 	    msg.metric = opt_whole;
 	    continue;
 
+	case CD_CONNMTU:
+	    msg.connmtu = opt_whole;
+	    continue;
+
 	case OPT_TPMEVAL:
 #ifdef TPM
 	    msg.tpmeval = strdup(optarg);
