@@ -15,7 +15,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: x509more.h,v 1.1 2003-10-31 02:45:23 mcr Exp $
+ * RCSID $Id: x509more.h,v 1.1 2003/10/31 02:45:23 mcr Exp $
  */
 
 #ifndef _X509MORE_H
@@ -24,13 +24,17 @@
 #include "packet.h"
 #include "demux.h"
 #include "server.h"
+#include "secrets.h"
 
 /* forward reference */
 struct msg_digest;
 
 extern void decode_cert(struct msg_digest *md);
+extern void ikev2_decode_cert(struct msg_digest *md);
+extern void ikev2_decode_cr(struct msg_digest *md, generalName_t **requested_ca);
 extern void decode_cr(struct msg_digest *md, generalName_t **requested_ca);
 extern bool collect_rw_ca_candidates(struct msg_digest *md, generalName_t **top);
 extern bool build_and_ship_CR(u_int8_t type, chunk_t ca, pb_stream *outs, u_int8_t np);
+extern bool ikev2_build_and_ship_CR(u_int8_t type, chunk_t ca, pb_stream *outs, u_int8_t np);
 extern err_t process_rsa_keyfile(struct RSA_private_key *rsak, int whackfd);
 #endif /* _X509MORE_H */
