@@ -5,6 +5,8 @@
  * Copyright (C) 2003-2008 Michael C Richardson <mcr@xelerance.com> 
  * Copyright (C) 2003-2010 Paul Wouters <paul@xelerance.com> 
  * Copyright (C) 2009 Avesh Agarwal <avagarwa@redhat.com>
+ * Copyright (C) 2009 Stefan Arentz <stefan@arentz.ca>
+ * Copyright (C) 2010 Tuomo Soini <tis@foobar.fi>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -35,6 +37,10 @@
 #include <sys/utsname.h>
 
 #include <arpa/nameser.h>
+/* older versions lack ipseckey support */
+#ifndef ns_t_ipseckey
+# define ns_t_ipseckey 45
+#endif
 
 #include "constants.h"
 #include "oswalloc.h"
@@ -267,7 +273,7 @@ void show_dnskey(struct secret *s
     char base64[8192];
     const struct private_key_stuff *pks = osw_get_pks(s);
     unsigned char *keyblob;
-    unsigned int keybloblen;
+    unsigned int keybloblen = 0;
 
     gethostname(qname, sizeof(qname));
 
@@ -324,7 +330,7 @@ void show_confkey(struct secret *s
     char base64[8192];
     const struct private_key_stuff *pks = osw_get_pks(s);
     unsigned char *keyblob;
-    unsigned int keybloblen;
+    unsigned int keybloblen = 0;
 
     if(pks->kind != PPK_RSA) {
         char *enumstr = "gcc is crazy";
@@ -476,7 +482,11 @@ int main(int argc, char *argv[])
     }
 
     if(verbose > 2) {
+#ifdef DEBUG
 	set_debugging(DBG_ALL);
+#else
+	fprintf(stderr, "verbosity cannot be set - compiled without DEBUG\n");
+#endif
     }
 
     /* now load file from indicated location */
