@@ -2,6 +2,8 @@
  * All-in-one program to set Security Association parameters
  * Copyright (C) 1996  John Ioannidis.
  * Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Richard Guy Briggs.
+ * Copyright (C) 2005-2007 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2007-2010 Paul Wouters <paul@xelerance.com>
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -14,7 +16,6 @@
  * for more details.
  */
 
-char spi_c_version[] = "RCSID $Id: spi.c,v 1.114 2005/08/18 14:04:40 ken Exp $";
 
 #include <asm/types.h>
 #include <sys/types.h>
@@ -140,7 +141,7 @@ spi --esp <algo> <SA> [<life> ][ --replay_window <replay-window> ] --enckey <eke
 spi --esp <algo> <SA> [<life> ][ --replay_window <replay-window> ] --enckey <ekey>\n\
 	where <algo> is:	3des\n\
 spi --comp <algo> <SA>\n\
-	where <algo> is:	deflate\n\
+	where <algo> is:	deflate | lzs\n\
 [ --sarefme=XXX ]  set the saref to use for this SA\n\
 [ --sarefhim=XXX ] set the saref to use for paired SA\n\
 [ --dumpsaref ] show the saref allocated\n\
@@ -496,10 +497,9 @@ main(int argc, char *argv[])
 {
 	char *endptr;
 	__u32 spi = 0;
-	int c, previous = -1;
+	int c;
 /*	int ret; */
 	ip_said said;
-	size_t sa_len;
 	const char* error_s;
 	char ipaddr_txt[ADDRTOT_BUF];
 	char ipsaid_txt[SATOT_BUF];
@@ -514,7 +514,7 @@ main(int argc, char *argv[])
 	unsigned char authalg, encryptalg;
 	struct sadb_ext *extensions[K_SADB_EXT_MAX + 1];
 	struct sadb_msg *pfkey_msg;
-	char *iv_opt, *akey_opt, *ekey_opt, *alg_opt, *edst_opt, *spi_opt, *proto_opt, *af_opt, *said_opt, *dst_opt, *src_opt;
+	char *edst_opt, *spi_opt, *proto_opt, *af_opt, *said_opt, *dst_opt, *src_opt;
 #if 0
 	ip_address pfkey_address_p_ska;
 	ip_address pfkey_ident_s_ska;
@@ -524,6 +524,8 @@ main(int argc, char *argv[])
 	u_int16_t sport, dport;
 	uint32_t life[life_maxsever][life_maxtype];
 	char *life_opt[life_maxsever][life_maxtype];
+	struct stat sts;
+	struct sadb_builds sab;
 	
 	progname = argv[0];
 	mypid = getpid();
@@ -534,7 +536,7 @@ main(int argc, char *argv[])
 	tool_init_log();
 
 	memset(&said, 0, sizeof(said));
-	iv_opt = akey_opt = ekey_opt = alg_opt = edst_opt = spi_opt = proto_opt = af_opt = said_opt = dst_opt = src_opt = NULL;
+	edst_opt = spi_opt = proto_opt = af_opt = said_opt = dst_opt = src_opt = NULL;
 	{
 		int i,j;
 		for(i = 0; i < life_maxsever; i++) {
@@ -548,10 +550,14 @@ main(int argc, char *argv[])
 	while((c = getopt_long(argc, argv, ""/*"H:P:Z:46dcA:E:e:s:a:w:i:D:S:hvgl:+:f:"*/, longopts, 0)) != EOF) {
 		switch(c) {
 		case 'g':
+#ifdef DEBUG
 			debug = 1;
 			pfkey_lib_debug = PF_KEY_DEBUG_PARSE_MAX;
 			cur_debugging = 0xffffffff;
 			argcount--;
+#else
+			fprintf(stderr, "%s: Cannot set debug - compiled without DEBUG\n", progname);
+#endif
 			break;
 
 		case 'R':
@@ -626,7 +632,6 @@ main(int argc, char *argv[])
 					progname,
 					alg);
 			}
-			alg_opt = optarg;
 			break;
 		case 'P':
 			if(alg) {
@@ -642,7 +647,6 @@ main(int argc, char *argv[])
 					progname,
 					alg);
 			}
-			alg_opt = optarg;
 			break;
 		case 'Z':
 			if(alg) {
@@ -652,6 +656,8 @@ main(int argc, char *argv[])
 			}
 			if       (!strcmp(optarg, "deflate")) {
 				alg = XF_COMPDEFLATE;
+			} else if (!strcmp(optarg, "lzs")) {
+				alg = XF_COMPLZS;
 			} else {
 				fprintf(stderr, "%s: Unknown compression algorithm '%s' follows '--comp' option.\n",
 					progname, optarg);
@@ -662,7 +668,6 @@ main(int argc, char *argv[])
 					progname,
 					alg);
 			}
-			alg_opt = optarg;
 			break;
 		case '4':
 			if(alg) {
@@ -677,7 +682,6 @@ main(int argc, char *argv[])
 					progname,
 					alg);
 			}
-			alg_opt = optarg;
 			break;
 		case '6':
 			if(alg) {
@@ -692,7 +696,6 @@ main(int argc, char *argv[])
 					progname,
 					alg);
 			}
-			alg_opt = optarg;
 			break;
 		case 'd':
 			if(alg) {
@@ -706,7 +709,6 @@ main(int argc, char *argv[])
 					progname,
 					alg);
 			}
-			alg_opt = optarg;
 			break;
 		case 'c':
 			if(alg) {
@@ -720,7 +722,6 @@ main(int argc, char *argv[])
 					progname,
 					alg);
 			}
-			alg_opt = optarg;
 			break;
 		case 'e':
 			if(said_opt) {
@@ -903,7 +904,6 @@ main(int argc, char *argv[])
 			}
 			memset(authkey, 0, authkeylen);
 			authkeylen = atodata(optarg, 0, (char *)authkey, authkeylen);
-			akey_opt = optarg;
 			break;
 		case 'E':
 			if(optarg[0] == '0') {
@@ -931,7 +931,6 @@ main(int argc, char *argv[])
 			}
 			memset(enckey, 0, enckeylen);
 			enckeylen = atodata(optarg, 0, (char *)enckey, enckeylen);
-			ekey_opt = optarg;
 			break;
 		case 'w':
 			replay_window = strtoul(optarg, &endptr, 0);
@@ -972,7 +971,6 @@ main(int argc, char *argv[])
 			}
 			memset(iv, 0, ivlen);
 			ivlen = atodata(optarg, 0, (char *)iv, ivlen);
-			iv_opt = optarg;
 			break;
 		case 'D':
 			if(dst_opt) {
@@ -1065,7 +1063,7 @@ main(int argc, char *argv[])
 			usage(progname, stderr);
 			exit(1);
 		case 'v':
-			fprintf(stdout, "%s, %s\n", progname, spi_c_version);
+			fprintf(stdout, "%s, %s\n", progname, ipsec_version_code());
 			exit(1);
 		case '+': /* optionsfrom */
 			optionsfrom(optarg, &argc, &argv, optind, stderr);
@@ -1083,14 +1081,12 @@ main(int argc, char *argv[])
 				progname, c);
 			exit(1);
 		}
-		previous = c;
 	}
 	if(debug) {
 		fprintf(stdout, "%s: All options processed.\n",
 				progname);
 	}
 
-        struct stat sts;
         if ( ((stat ("/proc/net/pfkey", &sts)) == 0) )  {
                 fprintf(stderr, "%s: NETKEY does not use the ipsec spi command. Use 'ip xfrm' instead.\n",progname);
                 exit(1);
@@ -1175,6 +1171,7 @@ main(int argc, char *argv[])
 	case XF_ESP3DESSHA196:
 	case XF_ESP3DES:
 	case XF_COMPDEFLATE:
+	case XF_COMPLZS:
 		if(!said_opt) {
 			if(isanyaddr(&edst)) {
 				fprintf(stderr, "%s: SA destination not specified.\n",
@@ -1202,7 +1199,6 @@ main(int argc, char *argv[])
 				progname);
 			exit(1);
 		}
-		sa_len = satot(&said, 0, sa, sizeof(sa));
 
 		if(debug) {
 			fprintf(stdout, "%s: SA valid.\n",
@@ -1228,6 +1224,7 @@ main(int argc, char *argv[])
 	case XF_ESP3DESSHA196:
 	case XF_ESP3DES:
 	case XF_COMPDEFLATE:
+	case XF_COMPLZS:
 #ifdef KERNEL_ALG
 	case XF_OTHER_ALG:
 #endif /* NO_KERNEL_ALG */
@@ -1312,6 +1309,9 @@ main(int argc, char *argv[])
 	case XF_COMPDEFLATE:
 		encryptalg = SADB_X_CALG_DEFLATE;
 		break;
+	case XF_COMPLZS:
+		encryptalg = SADB_X_CALG_LZS;
+		break;
 #ifdef KERNEL_ALG
 	case XF_OTHER_ALG:
 		encryptalg= esp_info->encryptalg;
@@ -1325,16 +1325,19 @@ main(int argc, char *argv[])
 		encryptalg = SADB_EALG_NONE;
 	}
 	if(!(alg == XF_CLR /* IE: pfkey_msg->sadb_msg_type == SADB_FLUSH */)) {
-	    struct sadb_builds sab = {
-		.sa_base.sadb_sa_exttype = SADB_EXT_SA,
-		.sa_base.sadb_sa_spi     = htonl(spi),
-		.sa_base.sadb_sa_replay  = replay_window,
-		.sa_base.sadb_sa_state   = K_SADB_SASTATE_MATURE,
-		.sa_base.sadb_sa_auth    = authalg,
-		.sa_base.sadb_sa_encrypt = encryptalg,
-		.sa_base.sadb_sa_flags   = 0,
-		.sa_base.sadb_x_sa_ref   = IPSEC_SAREF_NULL,
-	    };
+		sab.sa_base.sadb_sa_len        = 0;
+		sab.sa_base.sadb_sa_exttype    = SADB_EXT_SA;
+		sab.sa_base.sadb_sa_spi        = htonl(spi);
+		sab.sa_base.sadb_sa_replay     = replay_window;
+		sab.sa_base.sadb_sa_state      = K_SADB_SASTATE_MATURE;
+		sab.sa_base.sadb_sa_auth       = authalg;
+		sab.sa_base.sadb_sa_encrypt    = encryptalg;
+		sab.sa_base.sadb_sa_flags      = 0;
+		sab.sa_base.sadb_x_sa_ref      = IPSEC_SAREF_NULL;
+		sab.sa_base.sadb_x_reserved[0] = 0;
+		sab.sa_base.sadb_x_reserved[1] = 0;
+		sab.sa_base.sadb_x_reserved[2] = 0;
+		sab.sa_base.sadb_x_reserved[3] = 0;
 
 	    if((error = pfkey_sa_builds(&extensions[SADB_EXT_SA],sab))) {
 		fprintf(stderr, "%s: Trouble building sa extension, error=%d.\n",
