@@ -57,8 +57,6 @@ usage(char *name)
 {	
 	fprintf(stdout,"%s --create <virtual>\n", name);
 	fprintf(stdout,"%s --delete <virtual>\n", name);
-	fprintf(stdout,"%s --attach --virtual <virtual-device> --physical <physical-device>\n",
-		name);
 	fprintf(stdout,"%s --attach --virtual <virtual-device> --physical <physical-device>\n",
 		name);
 	fprintf(stdout,"%s --detach --virtual <virtual-device>\n",
@@ -180,10 +178,11 @@ main(int argc, char *argv[])
 	struct ifreq ifr;
 	struct ipsectunnelconf shc;
 	int s;
-	int c, previous = -1;
+	int c;
 	int argcount = argc;
 	int createdelete = 0;
 	char virtname[64];
+        struct stat sts;
      
 	memset(&ifr, 0, sizeof(ifr));
 	memset(&shc, 0, sizeof(shc));
@@ -256,10 +255,8 @@ main(int argc, char *argv[])
 			usage(progname);
 			break;
 		}
-		previous = c;
 	}
 
-        struct stat sts;
         if ( ((stat ("/proc/net/pfkey", &sts)) == 0) )  {
                 fprintf(stderr, "%s: NETKEY does not support virtual interfaces.\n",progname);
                 exit(1);
