@@ -1,4 +1,6 @@
 #!/bin/sh
+
+: ==== start ====
 TZ=GMT export TZ
 
 ipsec spi --clear
@@ -25,4 +27,4 @@ ipsec look
 
 # magic route command
 route add -host 192.0.1.1 gw 192.1.2.45 dev ipsec0
-
+: ==== end ====
