@@ -1,5 +1,7 @@
 #!/bin/sh
 
+: ==== start ====
+
 ipsec spi --clear
 ipsec eroute --clear
 
@@ -29,5 +31,8 @@ ipsec look
 # magic route command
 route add -host 192.0.1.1 gw 192.1.2.45 dev ipsec0
 
-ipsec pf_key --daemon /var/run/pf_key.pid >/tmp/pfkey.txt
+mkdir -p /var/run/pluto
+ipsec pf_key --daemon /var/run/pluto/pf_key.pid >/tmp/pfkey.txt
 echo send packets
+
+: ==== end ====
