@@ -4,11 +4,11 @@ TEST_TYPE=klipstest
 TESTNAME=east-none-02
 TESTHOST=east
 EXITONEMPTY=--exitonempty
-THREEEIGHT=true
+#THREEEIGHT=true
 PRIV_INPUT=../inputs/01-sunrise-sunset-sandwich-nic-ping.pcap
 REF_PUB_OUTPUT=spi1-output.txt
 REF_CONSOLE_OUTPUT=spi1-console.txt
-REF26_CONSOLE_OUTPUT=spi1-console26.txt
+REF26_CONSOLE_OUTPUT=spi1-console.txt
 REF_CONSOLE_FIXUPS="kern-list-fixups.sed nocr.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS script-only.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS klips-spi-sanitize.sed"
