@@ -24,4 +24,4 @@ route add -host 192.0.1.1 gw 192.1.2.45 dev ipsec0
 # monitor upbound ACQUIRE messages
 #ipsec pf_key &
 
-/usr/src/programs/ikeping/ikeping --verbose --wait 2 --inet 192.0.1.1 192.0.1.1 192.0.1.1 192.0.1.1
+/usr/obj/programs/ikeping/ikeping --verbose --wait 2 --inet 192.0.1.1 192.0.1.1 192.0.1.1 192.0.1.1
