@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
  * License for more details.
  *
- * RCSID $Id: version.in.c,v 1.1 2005-04-13 18:15:35 mcr Exp $
+ * RCSID $Id: version.in.c,v 1.1 2005/04/13 18:15:35 mcr Exp $
  */
 
 #ifdef __KERNEL__
@@ -21,7 +21,7 @@
 
 #include "openswan.h"
 
-#define	V	"xxx"		/* substituted in by Makefile */
+#define	V	"@IPSECVERSION@"	/* substituted in by Makefile */
 static const char openswan_number[] = V;
 static const char openswan_string[] = "Linux Openswan testing lib " V;
 
