@@ -12,7 +12,7 @@ WEST_ARPREPLY=1
 
 EAST_INPUT=../../klips/inputs/01-sunrise-sunset-ping.pcap
 REF_WEST_OUTPUT=../../klips/west-icmp-01/spi1-cleartext.txt
-REF_WEST_FILTER="no-arp-pcap2.pl"
+REF_WEST_FILTER=../../klips/fixups/no-arp-pcap2.pl
 WEST_ARPREPLY=true
 
 
