@@ -20,7 +20,7 @@ REF_EAST_CONSOLE_OUTPUT=east-console.txt
 REF26_EAST_CONSOLE_OUTPUT=east-console.txt
 REF_WEST_CONSOLE_OUTPUT=west-console.txt
 REF26_WEST_CONSOLE_OUTPUT=west-console.txt
-REF_WEST_FILTER="no-arp-pcap2.pl"
+REF_WEST_FILTER=../../klips/fixups/no-arp-pcap2.pl
 
 REF_CONSOLE_FIXUPS="kern-list-fixups.sed nocr.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS east-prompt-splitline.pl"
