@@ -12,6 +12,7 @@ REF_CONSOLE_FIXUPS="kern-list-fixups.sed nocr.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS east-prompt-splitline.pl"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS cutout.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS ipsec-setup-sanitize.sed"
+REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS script-only.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS host-ping-sanitize.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS host-dig-sanitize.sed"
 REF_CONSOLE_FIXUPS="$REF_CONSOLE_FIXUPS pluto-whack-sanitize.sed"
@@ -22,6 +23,7 @@ EAST_INIT_SCRIPT=../oe-road-01/eastinit.sh
 EAST_RUN2_SCRIPT=eastrun2.sh
 EAST_FINAL_SCRIPT=../oe-road-01/roadfinal.sh
 REF_EAST_CONSOLE_OUTPUT=east-console.txt
+REF26_EAST_CONSOLE_OUTPUT=east-console.txt
 
 ROADHOST=road
 ROAD_INIT_SCRIPT=../myid-road-04/roadinit.sh
@@ -29,6 +31,7 @@ ROAD_RUN_SCRIPT=../myid-road-04/roadrun.sh
 ROAD_RUN3_SCRIPT=roadrun3.sh
 ROAD_FINAL_SCRIPT=../myid-road-04/roadfinal.sh
 REF_ROAD_CONSOLE_OUTPUT=road-console.txt
+REF26_ROAD_CONSOLE_OUTPUT=road-console.txt
 
 SUNRISEHOST=sunrise
 SUNRISE_INIT_SCRIPT=../oe-road-01/sunriseinit.sh
