@@ -8,12 +8,12 @@ TESTNAME=x509-pluto-01
 EASTHOST=east
 WESTHOST=west
 
-THREEEIGHT=true
+#THREEEIGHT=true
 
 EAST_INPUT=../../klips/inputs/01-sunrise-sunset-ping.pcap
 REF_WEST_OUTPUT=../../klips/west-icmp-01/spi1-cleartext.txt
 WEST_ARPREPLY=true
-THREEEIGHT=true
+#THREEEIGHT=true
 
 REF_EAST_CONSOLE_OUTPUT=east-console.txt
 REF26_EAST_CONSOLE_OUTPUT=east-console.txt
