@@ -1,7 +1,7 @@
-#!/bin/sh
+#!/bin/bash
 
 #
-# $Id: runme.sh,v 1.3 2004-05-28 02:07:02 mcr Exp $
+# $Id: runme.sh,v 1.3 2004/05/28 02:07:02 mcr Exp $
 #
 # use this script to run a single test from within that test directory.
 # note that this assumes a "klipstest" type test.
@@ -10,20 +10,22 @@
 . ../../../umlsetup.sh
 . ../setup.sh
 . $UTILS/functions.sh
-. testparams.sh
+. ./testparams.sh
 
 if [ -z "${TEST_TYPE}" ]
 then
-    echo runme.sh now requires that testparams.sh defines TEST_TYPE=
+    echo >&2 "Error: runme.sh now requires that testparams.sh defines TEST_TYPE= (current value null or undef)"
+	exit 99
 fi
 
 ( cd .. && $TEST_TYPE $TESTNAME good )
+# FIXUP: test return value from above?
 
 perl ${OPENSWANSRCDIR}/testing/utils/regress-summarize-results.pl ${REGRESSRESULTS} notest
 
 
 # $Log: runme.sh,v $
-# Revision 1.3  2004-05-28 02:07:02  mcr
+# Revision 1.3  2004/05/28 02:07:02  mcr
 # 	update test case summary at end of manual run.
 #
 # Revision 1.2  2002/05/23 14:26:39  mcr
